package com.dragonsoft.dcuc.approve.facade;

import com.dragoninfo.dcuc.app.dto.ServiceResourceDTO;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragonsoft.duceap.base.annotations.parameter.QueryObject;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.core.search.Searchable;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import java.util.List;

/**
 * @author wangrs
 * @version 1.0
 * @date 2021/2/5
 */
@Api(tags = "引用接口")
@FeignClient(name = "reference")
public interface IReferenceFacade {
    /**
     * 应用数据
     *
     * @return 应用数据
     */
    @ApiOperation(value = "应用数据", notes = "应用数据")
    @RequestMapping(value = "v1/application/search", method = RequestMethod.POST)
    ResponseDTO<List<ApplyInfo>> queryApplicationList();

    /**
     * 服务资源
     *
     * @param searchable 查询条件
     * @return 服务资源分页
     */
    @ApiOperation(value = "服务资源", notes = "服务资源")
    @RequestMapping(value = "v1/service-resource/search", method = RequestMethod.POST)
    Page<ServiceResourceDTO> queryServiceResourceList(@QueryObject Searchable searchable);

    /**
     * 工作流管理端页面地址
     *
     * @return 地址
     */
    @ApiOperation(value = "工作流管理端页面地址", notes = "工作流管理端页面地址")
    @RequestMapping(value = "v1/workflow-url", method = RequestMethod.GET)
    ResponseDTO<String> workflowUrl();
}
