package com.dragonsoft.dcuc.approve.facade.api.v3;

import com.dragonsoft.dcuc.approve.model.resp.taskclass.ApiTaskClassRespVO;
import com.dragonsoft.dcuc.approve.model.resp.taskclass.ApiTaskDetailRespVO;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/11
 */
@Api(tags = "审批任务类型服务接口")
@FeignClient(name = "approve")
public interface IApiV3ApproveTaskClassFacade {

    /**
     * 获取任务类型列表
     *
     * @return 任务类型列表
     */
    @ApiOperation(value = "任务类型列表", notes = "任务类型列表")
    @GetMapping("api/v3/task-classes")
    ResponseDTO<List<ApiTaskClassRespVO>> taskClassList();

    /**
     * 根据任务代码获取任务详情
     *
     * @param taskCode 任务标识
     * @return 任务详情
     */
    @ApiOperation(value = "根据任务代码获取任务详情", notes = "根据任务代码获取任务详情")
    @GetMapping("api/v3/tasks/{taskCode}")
    ResponseDTO<ApiTaskDetailRespVO> taskDetail(@PathVariable("taskCode") String taskCode);

    /**
     * 根据用户、应用代码获取对应的任务列表
     *
     * @param userToken 用户令牌
     * @param appCode   应用代码
     * @return 任务列表
     */
    @ApiOperation(value = "根据用户、应用代码获取对应的任务列表", notes = "根据用户、应用代码获取对应的任务列表")
    @GetMapping("api/v3/user-tasks/{userToken}/apps/{appCode}")
    ResponseDTO<List<ApiTaskDetailRespVO>> getUserAppTaskList(@PathVariable("userToken") String userToken,
                                                              @PathVariable("appCode") String appCode);
}
