package com.dragonsoft.dcuc.approve.model.req;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import java.util.Date;
import java.util.List;

/**
 * @author huangzqa
 * @date 2021/7/20
 * @desc 审批单报送信息
 */
@Data
@ApiModel(description = "审批单报送信息")
public class ApproveRecordReportReqVO {

    /**
     * 流程编号
     */
    @ApiModelProperty(value = "流程编号")
    private String processNo;

    /**
     * 流程名称
     */
    @ApiModelProperty(value = "流程名称")
    private String processName;

    /**
     * @see com.dragonsoft.dcuc.approve.enumresources.ApproveTypeEnum
     */
    @ApiModelProperty(value = "审批类型，详见ApproveTypeEnum")
    @NotBlank
    private String approveType;

    /**
     * 审批单编号
     */
    @NotBlank
    @ApiModelProperty(value = "审批单编号")
    private String approveNo;

    /**
     * 审批单标题
     */
    @NotBlank
    @ApiModelProperty(value = "审批单标题")
    private String approveTitle;

    /**
     * 申请人身份证号码
     */
    @NotBlank
    @ApiModelProperty(value = "申请人身份证号码")
    private String applicantIdcard;

    /**
     * 申请人姓名
     */
    @NotBlank
    @ApiModelProperty(value = "申请人姓名")
    private String applicantName;

    /**
     * 申请人手机号码
     */
    @NotBlank
    @ApiModelProperty(value = "申请人手机号码")
    private String applicantPhoneNo;

    /**
     * 申请人单位代码
     */
    @NotBlank
    @ApiModelProperty(value = "申请人单位代码")
    private String applicantOrgCode;

    /**
     * 申请人单位名称
     */
    @NotBlank
    @ApiModelProperty(value = "申请人单位名称")
    private String applicantOrgName;

    /**
     * 申请时间
     */
    @NotBlank
    @ApiModelProperty(value = "申请时间")
    private String applyTime;

    /**
     * 应用名称
     */
    @NotBlank
    @ApiModelProperty(value = "应用名称")
    private String appName;

    /**
     * 应用编号
     */
    @NotBlank
    @ApiModelProperty(value = "应用编号")
    private String appCode;

    /**
     * 访问资源
     */
    @NotBlank
    @ApiModelProperty(value = "访问资源")
    private String accessResources;

    /**
     * 访问资源编号
     */
    @NotBlank
    @ApiModelProperty(value = "访问资源编号")
    private String accessResourcesNo;

    /**
     * 有效期类别（自定义：01；长期：02；）
     *
     * @see com.dragonsoft.dcuc.approve.enumresources.PermissionValidTypeEnum
     */
    @ApiModelProperty(value = "有效期类别,详情PermissionValidTypeEnum")
    private String permissionValidType;

    /**
     * 有效时间起始
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date startTime;

    /**
     * 有效时间终止
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date endTime;


    /**
     * 申请事由
     */
    @NotBlank
    @ApiModelProperty(value = "申请事由")
    private String applyReason;

    /**
     * 最终审批结果,详见dm_approval_result码表
     *
     * @see com.dragonsoft.dcuc.approve.enumresources.ApproveResultEnum
     */
    @NotBlank
    @ApiModelProperty(value = "最终审批结果,详见ApproveResultEnum枚举")
    private String finalApprovalResult;

    /**
     * 附件地址
     */
    @ApiModelProperty(value = "附件地址")
    private String attachmentUrl;

    /**
     * 案件/事件/线索/被查询人员类别
     */
    @ApiModelProperty(value = "案件/事件/线索/被查询人员类别")
    private String queryUserType;

    /**
     * 案件/事件/线索/被查询人员名称
     */
    @ApiModelProperty(value = "案件/事件/线索/被查询人员名称")
    private String queryUserName;

    /**
     * 案件/事件/线索/被查询人员编号
     */
    @ApiModelProperty(value = "案件/事件/线索/被查询人员编号")
    private String queryUserId;

    /**
     * 其他申请信息
     */
    @ApiModelProperty(value = "其他申请信息")
    private String otherApplyContext;

    /**
     * 审批流转信息list
     */
    @ApiModelProperty(value = "审批流转信息list")
    private List<ApproveCirculationReqVO> approveCirculationList;
}
