package com.dragonsoft.dcuc.approve.model.resp.v3;

import com.dragonsoft.dcuc.approve.enumresources.ZeroTrustBusinessRespEnum;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import lombok.Data;

/**
 * <p>
 * 用户令牌校验返回
 * </p>
 *
 * @author huangzqa
 * @date 2022/8/1
 */
@JsonPropertyOrder({"statusCode", "message"})
@Data
public class ZeroTrustMessageRespVO {

    /**
     * “0000” 表示令牌有效；
     * 其他值表示无效或接口调用出错
     */
    @JsonProperty("status_code")
    private String statusCode;

    /**
     * 状态码对应的详细描述
     */
    private String message;

    /**
     * 设置业务枚举
     *
     * @param zeroTrustBusinessRespEnum 业务枚举
     */
    public void setBusinessRespEnum(ZeroTrustBusinessRespEnum zeroTrustBusinessRespEnum) {
        setStatusCode(zeroTrustBusinessRespEnum.getValue());
        setMessage(zeroTrustBusinessRespEnum.getLabel());
    }


    /**
     * 设置业务枚举
     *
     * @param zeroTrustBusinessRespEnum 业务枚举
     */
    public static ZeroTrustMessageRespVO messageEnumMessage(ZeroTrustBusinessRespEnum zeroTrustBusinessRespEnum) {
        ZeroTrustMessageRespVO zeroTrustMessageRespVO = new ZeroTrustMessageRespVO();
        zeroTrustMessageRespVO.setBusinessRespEnum(zeroTrustBusinessRespEnum);
        return zeroTrustMessageRespVO;
    }

    /**
     * 设置请求参数异常
     *
     * @param message 异常信息
     */
    public static ZeroTrustMessageRespVO requestErrorMessage(String message) {
        return messageEnumMessage(ZeroTrustBusinessRespEnum.OPERATE_FAIL, message);
    }

    /**
     * 设置请求参数异常
     *
     * @param zeroTrustBusinessRespEnum 枚举
     * @param message          异常信息
     */
    public static ZeroTrustMessageRespVO messageEnumMessage(ZeroTrustBusinessRespEnum zeroTrustBusinessRespEnum, String message) {
        ZeroTrustMessageRespVO zeroTrustMessageRespVO = new ZeroTrustMessageRespVO();
        zeroTrustMessageRespVO.setStatusCode(zeroTrustBusinessRespEnum.getValue());
        zeroTrustMessageRespVO.setMessage(message);
        return zeroTrustMessageRespVO;
    }

    /**
     * 是否成功
     *
     * @return 是否成功
     */
    @JsonIgnore
    public boolean isRespSuccess() {
        return this.statusCode.equalsIgnoreCase(ZeroTrustBusinessRespEnum.SUCCESS.getValue());
    }

    /**
     * 是否失败
     *
     * @return 是否失败
     */
    @JsonIgnore
    public boolean isRespFail() {
        return !isRespSuccess();
    }

    /**
     * 转发Result返回
     *
     * @param <T> 泛型
     * @return Result返回
     */
    public <T> ZeroTrustResultDataVO<T> toResultRespVO() {
        ZeroTrustResultDataVO<T> objectZeroTrustResultDataVO = new ZeroTrustResultDataVO<>();
        objectZeroTrustResultDataVO.setStatusCode(statusCode);
        objectZeroTrustResultDataVO.setMessage(message);
        return objectZeroTrustResultDataVO;
    }
}
