package com.dragonsoft.dcuc.approve.model.resp.v3;

import com.dragonsoft.dcuc.approve.enumresources.ZeroTrustBusinessRespEnum;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * <p>
 * 返回数据
 * </p>
 *
 * @author huangzqa
 * @date 2022/8/1
 */
@JsonPropertyOrder({"statusCode", "message", "data"})
@EqualsAndHashCode(callSuper = true)
@Data
public class ZeroTrustResultDataVO<T> extends ZeroTrustMessageRespVO {

    /***
     * 返回内容
     */
    private T data;

    public static <T> ZeroTrustResultDataVO<T> success(T result) {
        ZeroTrustResultDataVO<T> objectZeroTrustResultDataVO = new ZeroTrustResultDataVO<>();
        objectZeroTrustResultDataVO.setBusinessRespEnum(ZeroTrustBusinessRespEnum.SUCCESS);
        objectZeroTrustResultDataVO.setData(result);
        return objectZeroTrustResultDataVO;
    }


    /**
     * 设置失败信息
     *
     * @param message 失败信息
     * @param <T>     泛型
     * @return 消息
     */
    public static <T> ZeroTrustResultDataVO<T> operateFailMessage(String message) {
        ZeroTrustResultDataVO<T> objectZeroTrustResultDataVO = new ZeroTrustResultDataVO<>();
        objectZeroTrustResultDataVO.setBusinessRespEnum(ZeroTrustBusinessRespEnum.OPERATE_FAIL);
        objectZeroTrustResultDataVO.setMessage(message);
        return objectZeroTrustResultDataVO;
    }

    /**
     * 设置枚举消息
     *
     * @param zeroTrustBusinessRespEnum 枚举类型
     * @param <T>                       泛型
     * @return 消息
     */
    public static <T> ZeroTrustResultDataVO<T> resultEnumMessage(ZeroTrustBusinessRespEnum zeroTrustBusinessRespEnum) {
        ZeroTrustResultDataVO<T> objectZeroTrustResultDataVO = new ZeroTrustResultDataVO<>();
        objectZeroTrustResultDataVO.setBusinessRespEnum(zeroTrustBusinessRespEnum);
        return objectZeroTrustResultDataVO;
    }

    /**
     * 设置枚举消息和内容
     *
     * @param zeroTrustBusinessRespEnum 枚举类型
     * @param data                      内容
     * @param <T>                       泛型
     * @return 消息
     */
    public static <T> ZeroTrustResultDataVO<T> resultEnumMessage(ZeroTrustBusinessRespEnum zeroTrustBusinessRespEnum, T data) {
        ZeroTrustResultDataVO<T> objectZeroTrustResultDataVO = new ZeroTrustResultDataVO<>();
        objectZeroTrustResultDataVO.setBusinessRespEnum(zeroTrustBusinessRespEnum);
        objectZeroTrustResultDataVO.setData(data);
        return objectZeroTrustResultDataVO;
    }
}
