package com.dragonsoft.dcuc.approve.business;

import com.dragoninfo.dcuc.duceap.enums.ManageDisposeTypeEnum;
import com.dragonsoft.dcuc.approve.model.dto.OperateReqDTO;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.req.AgentReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoUpdateReqVO;
import com.dragonsoft.dcuc.approve.model.resp.*;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.data.domain.Page;

import java.util.Date;
import java.util.List;

/**
 * 代码千万行,注释第一行,编码不规范,同事两行泪
 *
 * @author huang(jy)
 * @version 1.0
 * @date 2021/2/3 19:43
 */
public interface IApproveCommonBusiness {

    /**
     * 待办/已办列表
     *
     * @param searchable 搜索条件
     * @return 分页
     */
    Page<TaskVo> queryApproveList(Searchable searchable);

    /**
     * 表单申请提交/保存
     *
     * @param approvalInfoReqVO 审批信息
     * @return 状态
     */
    ResponseDTO<String> save(ApprovalInfoReqVO approvalInfoReqVO);

    /**
     * 基础保存
     *
     * @param approvalInfoReqVO     请求信息
     * @param manageDisposeTypeEnum 管理日志类型
     * @param oldCreateTime         创建时间
     * @return 审批信息
     */
    ApproveRecordInfo saveBase(ApprovalInfoReqVO approvalInfoReqVO, ManageDisposeTypeEnum manageDisposeTypeEnum, Date oldCreateTime);

    /**
     * 表单申请提交/保存
     *
     * @param approvalInfoUpdateReqVO 审批信息
     * @return 状态
     */
    ResponseDTO<String> update(ApprovalInfoUpdateReqVO approvalInfoUpdateReqVO);

    /**
     * 审批操作
     *
     * @param operateReqDTO 操作请求
     * @return 状态
     */
    ResponseDTO<OperateRespVO> approveOperate(OperateReqDTO operateReqDTO);

    /**
     * 转办
     *
     * @param agentReqVO 转办信息
     */
    void agent(AgentReqVO agentReqVO);


    /**
     * 查看待办数量和已办数量
     * 已办:finish
     * 待办:unfinish
     *
     * @param searchable 搜索条件
     * @return 数量
     */
    Integer queryApproveQty(Searchable searchable);

    /**
     * 获取我的请求数量
     *
     * @return 数量
     */
    Integer queryMyRequestQty();

    /***
     * 获取审批意见
     * @param processInstanceId 流程实例ID
     * @return 状态
     */
    ResponseDTO<List<ApproveCommentVo>> approveComments(String processInstanceId);

    /**
     * 删除流程实例
     *
     * @param id 申请单ID
     * @return 状态
     */
    ResponseStatus deleteProcessInstance(String id);

    /**
     * 查看审批详情
     *
     * @param id id
     * @return 审批详情
     */
    ResponseDTO<ApprovalInfoRespVO> queryApproveDetail(String id);



    /***
     * 获取流程图状态信息
     * @param processInstanceId 流程实例ID
     * @return 状态
     */
    ResponseDTO<List<ProcessStatusVo>> approveStatus(String processInstanceId);

    /**
     * 获取审批状态
     *
     * @param businessId 业务ID
     * @return 状态
     */
    ResponseDTO<List<ProcessStatusVo>> approveStatusByBusinessId(String businessId);

}
