package com.dragonsoft.dcuc.approve.business;

import com.dragoninfo.dcuc.duceap.enums.ManageDisposeTypeEnum;
import com.dragonsoft.dcuc.approve.model.dto.ApproveLogDTO;
import com.dragonsoft.dcuc.approve.enumresources.ApproveResultEnum;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.CommitTypeEnum;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.resp.OperateRespVO;
import com.dragonsoft.dcuc.approve.model.dto.LogDto;
import com.dragonsoft.duceap.duwf.api.model.ProcessInstanceVO;

/**
 * 代码千万行,注释第一行,编码不规范,同事两行泪
 *
 * @author huang(jy)
 * @version 1.0
 * @date 2021/5/27 19:43
 */
public interface IApproveLogBusiness {

    /**
     * 操作日志/请求日志构建
     *
     * @param logDto                日志Dto
     * @param workFlowOperationEnum 操作类型
     */
    void buildOperateLog(LogDto logDto, ApproveWorkFlowOperationEnum workFlowOperationEnum, Boolean flowFinish);

    /**
     * 构建审批流程管理日志-新建/修改
     *
     * @param logDto                日志Dto
     * @param commitTypeEnum        类型（保存/保存并提交）
     * @param manageDisposeTypeEnum 决定类型
     */
    void buildFlowManageLogInsert(LogDto logDto, CommitTypeEnum commitTypeEnum, ManageDisposeTypeEnum manageDisposeTypeEnum,
                                  Boolean flowFinish);

    /**
     * 构建审批流程管理日志-修改
     *
     * @param logDto             日志Dto
     * @param needSaveOperateLog 是否需要保存操作日志
     */
    void buildFlowManageLogUpdate(LogDto logDto, Boolean needSaveOperateLog, Boolean flowFinish);

    /**
     * 构建审批流程管理日志-删除
     *
     * @param logDto 日志Dto
     */
    void buildFlowManageLogDelete(LogDto logDto);

    /**
     * 处理日志消息进行保存
     *
     * @param approveLogDTO 消息内容
     */
    void saveLog(ApproveLogDTO approveLogDTO);

    /**
     * 构建日志Dto
     *
     * @param approvalInfoDTO      审批信息
     * @param vo                实例信息
     * @param approveResultEnum 返回信息
     * @param failMsg           错误消息
     * @return 日志信息
     */
    LogDto buildLogDto(ApprovalInfoDTO approvalInfoDTO, ProcessInstanceVO vo, ApproveResultEnum approveResultEnum,
                       String failMsg, boolean needSendRedListRisk, OperateRespVO operateRespVO);
}
