package com.dragonsoft.dcuc.approve.business.api;

import com.dragonsoft.dcuc.approve.enumresources.ReqBizDataEnum;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoReqVO;
import com.dragonsoft.dcuc.approve.model.req.v3.ZeroTrustApproveCreateReqV3VO;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustAppTokenInfoRespVO;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;

import java.util.Map;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/11/14
 */
public interface IApiV3ApproveBusiness {


    /**
     * 检查业务数据
     *
     * @param bizDataMap 业务数据
     * @return 状态
     */
    ResponseStatus checkBizData(Map<String, Object> bizDataMap);

    /**
     * 获取业务数据
     *
     * @param bizDataMap     业务数据
     * @param reqBizDataEnum 业务数据枚举
     * @param <T>            泛型
     * @return 值
     */
    <T> T getBizData(Map<String, Object> bizDataMap, ReqBizDataEnum reqBizDataEnum);

    /**
     * 转换成VO
     *
     * @param zeroTrustApproveCreateReqV3VO 请求信息
     * @param appTokenInfo                  应用令牌信息
     * @param processType                   流程类型
     * @return 请求
     */
    ResponseDTO<ApprovalInfoReqVO> parseToReqVO(ZeroTrustApproveCreateReqV3VO zeroTrustApproveCreateReqV3VO,
                                                ZeroTrustAppTokenInfoRespVO appTokenInfo, String processType);

    /**
     * 添加业务数据
     *
     * @param approvalInfoReqVO 请求信息
     * @param bizData           业务数据
     * @return 状态
     */
    ResponseDTO<ApprovalInfoReqVO> addBizData(ApprovalInfoReqVO approvalInfoReqVO, Map<String, Object> bizData);
}
