package com.dragonsoft.dcuc.approve.business.api.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.dragoninfo.dcuc.common.Constants;
import com.dragoninfo.dcuc.common.utils.LangUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.business.api.IApiV3ApproveBusiness;
import com.dragonsoft.dcuc.approve.business.external.IDcucBusiness;
import com.dragonsoft.dcuc.approve.enumresources.*;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoReqVO;
import com.dragonsoft.dcuc.approve.model.req.ResourceInfoReqVO;
import com.dragonsoft.dcuc.approve.model.req.v3.ZeroTrustApproveAuthInfoVO;
import com.dragonsoft.dcuc.approve.model.req.v3.ZeroTrustApproveCreateReqV3VO;
import com.dragonsoft.dcuc.approve.model.req.v3.ZeroTrustApproveTaskInfoVO;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustAppItemUserTokenInfoRespVO;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustAppTokenInfoRespVO;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.ApiPoliceResultVo;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.UserContactVo;
import com.dragonsoft.dcuc.approve.service.resource.IAjInfoService;
import com.dragonsoft.dcuc.approve.service.resource.IJqInfoService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/11/14
 */
@Slf4j
@Component
public class ApiV3ApproveBusinessImpl implements IApiV3ApproveBusiness {

    private IDcucBusiness dcucBusiness;

    private IAjInfoService ajInfoService;

    private IJqInfoService jqInfoService;

    @Autowired
    public void setJqInfoService(IJqInfoService jqInfoService) {
        this.jqInfoService = jqInfoService;
    }

    @Autowired
    public void setAjInfoService(IAjInfoService ajInfoService) {
        this.ajInfoService = ajInfoService;
    }

    @Autowired
    public void setDcucBusiness(IDcucBusiness dcucBusiness) {
        this.dcucBusiness = dcucBusiness;
    }

    @Override
    public ResponseStatus checkBizData(Map<String, Object> bizDataMap) {
        if (CollUtil.isEmpty(bizDataMap)) {
            bizDataMap = Collections.emptyMap();
        }

        StringBuilder errorMessageBuilder = new StringBuilder();
        for (ReqBizDataEnum reqBizDataEnum : ReqBizDataEnum.values()) {
            String value = reqBizDataEnum.getValue();
            boolean containsKey = bizDataMap.containsKey(value);
            if (!containsKey) {
                if (reqBizDataEnum.isNeed()) {
                    String errorMessage = String.format("字段：%s 必填", value);
                    errorMessageBuilder.append(errorMessage).append(Constants.CHINESE_COMMA);
                }
            }
        }
        String errorMessage = errorMessageBuilder.toString();
        if (StrUtil.isNotBlank(errorMessage)) {
            return ResponseStatus.fail(LangUtil.subLastSymbol(errorMessage, Constants.CHINESE_COMMA));
        }

        return ResponseStatus.success();
    }

    @Override
    @SuppressWarnings("unchecked")
    public <T> T getBizData(Map<String, Object> bizDataMap, ReqBizDataEnum reqBizDataEnum) {
        Assert.notEmpty(bizDataMap);
        Assert.notNull(reqBizDataEnum);
        Object object = bizDataMap.get(reqBizDataEnum.getValue());
        switch (reqBizDataEnum) {
            case BIZ_TERM_START:
            case BIZ_TERM_END:
                String temString = object.toString();
                try {
                    object = DateUtils.getDate(temString, DateConst.HYPHEN_DISPLAY_TIME);
                } catch (ParseException e) {
                    log.error("parse to date error :{}", temString, e);
                    return null;
                }
                break;
            default:
                break;
        }

        if (ObjectUtil.isNull(object)) {
            return null;
        }

        return (T) object;
    }


    @Override
    public ResponseDTO<ApprovalInfoReqVO> parseToReqVO(ZeroTrustApproveCreateReqV3VO zeroTrustApproveCreateReqV3VO,
                                                       ZeroTrustAppTokenInfoRespVO appTokenInfo, String processType) {
        final Map<String, Object> bizData = zeroTrustApproveCreateReqV3VO.getBizData();
        ResourceInfoReqVO resourceInfoReqVO = new ResourceInfoReqVO();
        if (MapUtil.isNotEmpty(bizData)) {
            BeanUtil.copyProperties(bizData, resourceInfoReqVO);
        }
        String otherApplyContext = "";
        if (MapUtil.isNotEmpty(bizData)) {
            otherApplyContext = JsonUtils.toJSONString(bizData);
        }
        String title = zeroTrustApproveCreateReqV3VO.getTitle();
        String processDefId = zeroTrustApproveCreateReqV3VO.getProcessDefId();
        String approveTaskId = zeroTrustApproveCreateReqV3VO.getTaskId();
        String callbackUrl = zeroTrustApproveCreateReqV3VO.getCallbackUrl();
        Map<String, List<Map<String, Object>>> bo = zeroTrustApproveCreateReqV3VO.getBo();
        String approveTaskClassCode = zeroTrustApproveCreateReqV3VO.getTaskClass();
        String appTokenId = zeroTrustApproveCreateReqV3VO.getAppTokenId();

        ZeroTrustAppItemUserTokenInfoRespVO userToken = appTokenInfo.getUserToken();
        String appCode = appTokenInfo.getAppId();
        String applicantIdcard = userToken.getPid();
        String applicantName = userToken.getName();
        String applicantIp = userToken.getIp();
        String orgCode = userToken.getOrgCode();
        String content = "";
        if (MapUtil.isNotEmpty(bo)) {
            content = JsonUtils.toJSONString(bo);
        }
        final SecurityUser securityUser = UserContextUtils.getCurrentUser();
        String applicantOrgName = securityUser.getSecurityOrgName();

        String applicantPhoneNo = "";
        final ApiPoliceResultVo apiPoliceResultVo = dcucBusiness.policeDetailByIdcard(applicantIdcard);
        if (ObjectUtil.isNotNull(apiPoliceResultVo)) {
            final UserContactVo userContact = apiPoliceResultVo.getUserContact();
            if (ObjectUtil.isNotNull(userContact)) {
                applicantPhoneNo = userContact.getMobileWork();
            }
        }

        ApprovalInfoReqVO approvalInfoReqVO = new ApprovalInfoReqVO();
        approvalInfoReqVO.setProcessTitle(title);
        approvalInfoReqVO.setProcessType(processType);
        approvalInfoReqVO.setBusinessCode(processDefId);
        approvalInfoReqVO.setAppCode(appCode);
        approvalInfoReqVO.setApplyContent(content);
        approvalInfoReqVO.setApplyReason(title + "申请");
        approvalInfoReqVO.setApplicantIdcard(applicantIdcard);
        approvalInfoReqVO.setApplicantName(applicantName);
        approvalInfoReqVO.setApplicantOrgCode(orgCode);
        approvalInfoReqVO.setApplicantOrgName(applicantOrgName);
        approvalInfoReqVO.setApplicantPhoneNo(applicantPhoneNo);
        approvalInfoReqVO.setPermissionValidType(PermissionValidTypeEnum.CUSTOM.getValue());
        approvalInfoReqVO.setOperateType(CommitTypeEnum.APPLY_COMMIT.getValue());
        approvalInfoReqVO.setResourceInfoDTO(resourceInfoReqVO);
        approvalInfoReqVO.setCallbackUrl(callbackUrl);
        approvalInfoReqVO.setApproveTaskClass(approveTaskClassCode);
        approvalInfoReqVO.setApproveTaskId(approveTaskId);
        approvalInfoReqVO.setFlowType(FlowTypeEnum.OTHER.getValue());
        approvalInfoReqVO.setApplicantIp(applicantIp);
        approvalInfoReqVO.setApplicantAppToken(appTokenId);

        approvalInfoReqVO.setOtherApplyContext(otherApplyContext);
        final ResponseDTO<ApprovalInfoReqVO> addBizDataStatus = addBizData(approvalInfoReqVO, bizData);
        if (ResponseUtil.isFail(addBizDataStatus)) {
            return ResponseUtil.dtoFail(addBizDataStatus.getMessage());
        }

        return ResponseUtil.newInstance(approvalInfoReqVO);
    }

    @Override
    public ResponseDTO<ApprovalInfoReqVO> addBizData(ApprovalInfoReqVO approvalInfoReqVO, final Map<String, Object> bizData) {
        Assert.notNull(approvalInfoReqVO);
        Assert.notEmpty(bizData);
        Date startTime = getBizData(bizData, ReqBizDataEnum.BIZ_TERM_START);
        Date startEnd = getBizData(bizData, ReqBizDataEnum.BIZ_TERM_END);
        approvalInfoReqVO.setStartTime(startTime);
        approvalInfoReqVO.setEndTime(startEnd);

        BizTypeEnum bizTypeEnum = null;
        final Object bizTypeObject = getBizData(bizData, ReqBizDataEnum.BIZ_TYPE);
        if (ObjectUtil.isNotNull(bizTypeObject)) {
            bizTypeEnum = EnumUtils.enumOf(BizTypeEnum.class, bizTypeObject.toString());
        }
        if (bizTypeObject == null) {
            return ResponseUtil.dtoFail("请传入业务类型");
        }
        if (ObjectUtil.isNull(bizTypeEnum)) {
            return ResponseUtil.dtoFail(String.format("暂不支持该业务类型：%s", bizTypeObject));
        }

        if (bizTypeEnum.equals(BizTypeEnum.AUTH)) {
            return parseAuthApprovalInfoReqVO(approvalInfoReqVO, bizData);
        } else if (bizTypeEnum.equals(BizTypeEnum.TASK)) {
            return parseTaskApprovalInfoReqVO(approvalInfoReqVO, bizData);
        }
        throw new ApplicationException(String.format("暂无适配业务类型：%s", bizTypeEnum));
    }

    /**
     * 转化权限类型
     *
     * @param approvalInfoReqVO 请求信息
     * @param bizData           业务数据
     * @return 转化后的数据
     */
    protected ResponseDTO<ApprovalInfoReqVO> parseAuthApprovalInfoReqVO(ApprovalInfoReqVO approvalInfoReqVO, final Map<String, Object> bizData) {
        final Object content = getBizData(bizData, ReqBizDataEnum.BIZ_CONTENT);
        final ZeroTrustApproveAuthInfoVO zeroTrustApproveAuthInfoVO = new ZeroTrustApproveAuthInfoVO();
        BeanUtil.copyProperties(content, zeroTrustApproveAuthInfoVO);
        final String flowType = zeroTrustApproveAuthInfoVO.getFlowType();
        FlowTypeEnum flowTypeEnum = EnumUtils.enumOf(FlowTypeEnum.class, flowType);
        if (ObjectUtil.isNull(flowTypeEnum)) {
            return ResponseUtil.dtoFail(String.format("找不到对应的flowType，值：【%s】", flowType));
        }
        approvalInfoReqVO.setFlowType(flowType);
        final ResourceInfoReqVO resourceInfo = zeroTrustApproveAuthInfoVO.getResourceInfo();

        approvalInfoReqVO.setResourceInfoDTO(resourceInfo);
        return ResponseUtil.newInstance(approvalInfoReqVO);
    }


    /**
     * 转化任务类型
     *
     * @param approvalInfoReqVO 请求信息
     * @param bizData           业务数据
     * @return 转化后的数据
     */
    protected ResponseDTO<ApprovalInfoReqVO> parseTaskApprovalInfoReqVO(ApprovalInfoReqVO approvalInfoReqVO, final Map<String, Object> bizData) {
        final Object content = getBizData(bizData, ReqBizDataEnum.BIZ_CONTENT);
        final ZeroTrustApproveTaskInfoVO zeroTrustApproveTaskInfoVO = new ZeroTrustApproveTaskInfoVO();
        BeanUtil.copyProperties(content, zeroTrustApproveTaskInfoVO);
        String jqbh = zeroTrustApproveTaskInfoVO.getJqbh();
        String ajbh = zeroTrustApproveTaskInfoVO.getAjbh();
        if (StrUtil.isAllBlank(jqbh, ajbh)) {
            return ResponseUtil.dtoFail("警情编号、案件编号其中一个必填");
        }
        if (StrUtil.isNotBlank(jqbh)) {
            // 校验警情编号是否存在
            if (!jqInfoService.exitsJqbh(jqbh)) {
                return ResponseUtil.dtoFail(String.format("警情编号：%s 不存在", jqbh));
            }
        }

        if (StrUtil.isNotBlank(ajbh)) {
            // 校验案件编号是否存在
            if (!ajInfoService.exitsAjbh(ajbh)) {
                return ResponseUtil.dtoFail(String.format("案件编号：%s 不存在", ajbh));
            }
        }

        approvalInfoReqVO.setFlowType(FlowTypeEnum.TASK.getValue());

        return ResponseUtil.newInstance(approvalInfoReqVO);
    }
}
