package com.dragonsoft.dcuc.approve.business.api.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.dragonsoft.dcuc.approve.business.api.IApproveApiBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.model.req.v3.ZeroTrustApproveCreateReqV3VO;
import com.dragonsoft.smtools.loader.SMFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/12
 */
@Slf4j
@Component
public class ApproveApiBusinessImpl implements IApproveApiBusiness {

    private SMFactory smFactory;

    @Autowired
    public void setSmFactory(SMFactory smFactory) {
        this.smFactory = smFactory;
    }

    @Override
    public boolean checkCallerSign(ZeroTrustApproveCreateReqV3VO zeroTrustApproveCreateReqV3VO) {
        Assert.notNull(zeroTrustApproveCreateReqV3VO);

        String appTokenId = zeroTrustApproveCreateReqV3VO.getAppTokenId();
        String processDefId = zeroTrustApproveCreateReqV3VO.getProcessDefId();
        String title = zeroTrustApproveCreateReqV3VO.getTitle();
        String taskClass = zeroTrustApproveCreateReqV3VO.getTaskClass();
        String taskId = zeroTrustApproveCreateReqV3VO.getTaskId();
        String callbackUrl = zeroTrustApproveCreateReqV3VO.getCallbackUrl();

        List<String> strings = Arrays.asList(appTokenId, processDefId, title, taskClass, taskId, callbackUrl);
        String origin = String.join(StrUtil.COMMA, strings);
        log.info("校验签名原文：{}", origin);
        String generalOrigin = smFactory.getSM3().summary(origin).toString();
        String callerSign = zeroTrustApproveCreateReqV3VO.getCallerSign();

        log.info("生成后的签名信息：{}", generalOrigin);
        log.info("传入的签名信息：{}", callerSign);
        boolean isTrue = generalOrigin.equalsIgnoreCase(callerSign);
        log.info("签名信息判断结果：{}", isTrue);
        return isTrue;
    }

    @Override
    public boolean isStanderTaskNo(String taskNo) {
        if (StrUtil.isBlank(taskNo)) {
            return false;
        }
        if (!taskNo.startsWith(ApproveConstants.TASK_ID_PREFIX)) {
            return false;
        }
        return taskNo.length() == ApproveConstants.TAK_ID_LENGTH;
    }
}
