package com.dragonsoft.dcuc.approve.business.external.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.dragoninfo.dcuc.common.utils.LangUtil;
import com.dragonsoft.dcuc.approve.business.external.IBimBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveBeanConstants;
import com.dragonsoft.dcuc.approve.enumresources.ZeroTrustBusinessRespEnum;
import com.dragonsoft.dcuc.approve.model.req.bim.ZeroTrustAppTokenInfoReqVO;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustAppTokenInfoRespVO;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustUserTokenInfoReqVO;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustUserTokenInfoRespVO;
import com.dragonsoft.dcuc.approve.model.resp.v3.ZeroTrustResultRespVO;
import com.dragonsoft.dcuc.approve.properties.BimProperties;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.smtools.loader.SMFactory;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.HttpMethod;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.net.URI;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/6/7
 */
@Slf4j
@Component
public class BimBusinessImpl implements IBimBusiness {

    private BimProperties bimProperties;

    private RestTemplate restTemplate;

    private ObjectMapper objectMapper;

    private SMFactory smFactory;

    @Autowired
    public void setSmFactory(SMFactory smFactory) {
        this.smFactory = smFactory;
    }

    @Qualifier(ApproveBeanConstants.DRAGON_OBJECT_MAPPER)
    @Autowired
    public void setObjectMapper(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    @Autowired
    public void setBimProperties(BimProperties bimProperties) {
        this.bimProperties = bimProperties;
    }

    @Qualifier(ApproveBeanConstants.SKIP_SSL_REST_TEMPLATE)
    @Autowired
    public void setRestTemplate(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    @Override
    public ZeroTrustUserTokenInfoRespVO getUserTokenInfo(String useTokenId) {
        String userInfoUrl = "/idp/rest/getUserTokenInfo";
        Assert.notBlank(useTokenId);
        String requestName = "获取用户令牌信息";

        String reqUrl = bimProperties.getIdentityUrl() + userInfoUrl;

        Boolean checkTokenSign = bimProperties.getCheckTokenSign();

        ZeroTrustUserTokenInfoReqVO zeroTrustUserTokenInfoReqVO = new ZeroTrustUserTokenInfoReqVO();
        zeroTrustUserTokenInfoReqVO.setUserTokenId(useTokenId);

        log.info("{} 请求路径:{} 参数 :{}", requestName, reqUrl, JsonUtils.toJSONString(zeroTrustUserTokenInfoReqVO));
        RequestEntity<ZeroTrustUserTokenInfoReqVO> httpEntity = new RequestEntity<>(zeroTrustUserTokenInfoReqVO, HttpMethod.POST, URI.create(reqUrl));

        TypeReference<ZeroTrustResultRespVO<ZeroTrustUserTokenInfoRespVO>> parameterizedTypeReference =
                new TypeReference<ZeroTrustResultRespVO<ZeroTrustUserTokenInfoRespVO>>() {
                };

        ResponseEntity<String> responseEntity = restTemplate.exchange(httpEntity, String.class);
        log.info("{}返回参数 :{}", requestName, JsonUtils.toJSONString(responseEntity));

        if (responseEntity.getStatusCode().is2xxSuccessful()) {
            String responseEntityJsonBody = responseEntity.getBody();

            ZeroTrustResultRespVO<ZeroTrustUserTokenInfoRespVO> responseEntityBody = null;
            try {
                responseEntityBody = objectMapper.readValue(responseEntityJsonBody, parameterizedTypeReference);
            } catch (JsonProcessingException e) {
                log.error("解析JSON异常", e);
            }
            if (responseEntityBody != null) {
                if (responseEntityBody.getStatusCode().equalsIgnoreCase(ZeroTrustBusinessRespEnum.SUCCESS.getValue())) {

                    // 校验令牌签名
                    if (checkTokenSign) {
                        boolean b = checkUserTokenSign(responseEntityJsonBody);
                        if (!b) {
                            return null;
                        }
                    }

                    return responseEntityBody.getResult();
                } else {
                    log.error("{} statusCode:{} , message:{}", requestName, responseEntityBody.getStatusCode(), responseEntityBody.getMessage());
                }
            } else {
                log.error("{} 返回 isnull:{}", requestName, JsonUtils.toJSONString(responseEntity));
            }

        } else {
            log.error("{} 请求 error :{}", requestName, JsonUtils.toJSONString(responseEntity));
        }
        return null;
    }

    @Override
    public ZeroTrustAppTokenInfoRespVO getAppTokenInfo(String appTokenId) {
        String appInfoUrl = "/idp/rest/getAppTokenInfo";
        Assert.notBlank(appTokenId);
        String requestName = "获取应用令牌信息";

        String reqUrl = bimProperties.getIdentityUrl() + appInfoUrl;

        Boolean checkTokenSign = bimProperties.getCheckTokenSign();

        ZeroTrustAppTokenInfoReqVO zeroTrustAppTokenInfoReqVO = new ZeroTrustAppTokenInfoReqVO();
        zeroTrustAppTokenInfoReqVO.setAppTokenId(appTokenId);

        log.info("{} 请求路径:{} 参数 :{}", requestName, reqUrl, JsonUtils.toJSONString(zeroTrustAppTokenInfoReqVO));
        RequestEntity<ZeroTrustAppTokenInfoReqVO> httpEntity = new RequestEntity<>(zeroTrustAppTokenInfoReqVO, HttpMethod.POST, URI.create(reqUrl));

        TypeReference<ZeroTrustResultRespVO<ZeroTrustAppTokenInfoRespVO>> parameterizedTypeReference =
                new TypeReference<ZeroTrustResultRespVO<ZeroTrustAppTokenInfoRespVO>>() {
                };

        ResponseEntity<String> responseEntity = restTemplate.exchange(httpEntity, String.class);
        log.info("{}返回参数 :{}", requestName, JsonUtils.toJSONString(responseEntity));

        if (responseEntity.getStatusCode().is2xxSuccessful()) {
            String responseEntityJsonBody = responseEntity.getBody();

            ZeroTrustResultRespVO<ZeroTrustAppTokenInfoRespVO> responseEntityBody = null;
            try {
                responseEntityBody = objectMapper.readValue(responseEntityJsonBody, parameterizedTypeReference);
            } catch (JsonProcessingException e) {
                log.error("解析JSON异常", e);
            }
            if (responseEntityBody != null) {
                if (responseEntityBody.getStatusCode().equalsIgnoreCase(ZeroTrustBusinessRespEnum.SUCCESS.getValue())) {

                    // 校验令牌签名
                    if (checkTokenSign) {
                        boolean b = checkAppTokenSign(responseEntityJsonBody);
                        if (!b) {
                            log.error("校验应用令牌前面失败");
                            return null;
                        }
                    }

                    return responseEntityBody.getResult();
                } else {
                    log.error("{} statusCode:{} , message:{}", requestName, responseEntityBody.getStatusCode(), responseEntityBody.getMessage());
                }
            } else {
                log.error("{} 返回 isnull:{}", requestName, JsonUtils.toJSONString(responseEntity));
            }

        } else {
            log.error("{} 请求 error :{}", requestName, JsonUtils.toJSONString(responseEntity));
        }
        return null;
    }

    /**
     * 校验应用令牌签名
     *
     * @param appTokenJson 应用令牌信息
     * @return 是否成功
     */
    @SuppressWarnings("unchecked")
    @SneakyThrows(JsonProcessingException.class)
    public boolean checkAppTokenSign(String appTokenJson) {

        TypeReference<LinkedHashMap<String, Object>> objectTypeReference = new TypeReference<LinkedHashMap<String, Object>>() {
        };
        LinkedHashMap<String, Object> body = objectMapper.readValue(appTokenJson, objectTypeReference);
        LinkedHashMap<String, Object> userTokenInfo = (LinkedHashMap<String, Object>) body.getOrDefault("result", Collections.emptyMap());
        LinkedHashMap<String, Object> userToken = (LinkedHashMap<String, Object>) userTokenInfo.getOrDefault("userToken", Collections.emptyMap());
        String userTokenString = generalAppTokenCheckUserTokenString(userToken);
        log.info("生成后的用户令牌信息：{}", userTokenString);
        userTokenInfo.put("userToken", userTokenString);
        return checkAppTokenSign(userTokenInfo);
    }

    /**
     * 校验用户令牌签名
     *
     * @param userTokenJson 用户令牌JSON
     * @return 状态
     */
    @SuppressWarnings("unchecked")
    @SneakyThrows(JsonProcessingException.class)
    public boolean checkUserTokenSign(String userTokenJson) {

        TypeReference<LinkedHashMap<String, Object>> objectTypeReference = new TypeReference<LinkedHashMap<String, Object>>() {
        };
        LinkedHashMap<String, Object> body = objectMapper.readValue(userTokenJson, objectTypeReference);
        LinkedHashMap<String, Object> userTokenInfo = (LinkedHashMap<String, Object>) body.getOrDefault("result", Collections.emptyMap());
        return checkUserTokenSign(userTokenInfo);
    }

    /**
     * 生成应用令牌校验签名的用户令牌字符串
     *
     * @param userTokenMap 用户令牌信息
     * @return 用户令牌字符串
     */
    protected String generalAppTokenCheckUserTokenString(LinkedHashMap<String, Object> userTokenMap) {
        if (CollUtil.isEmpty(userTokenMap)) {
            throw new IllegalArgumentException();
        }
        String symbol = StrUtil.COMMA + " ";

        StringBuilder userTokenStringBuilder = new StringBuilder("{");

        for (String key : userTokenMap.keySet()) {
            userTokenStringBuilder.append(key).append("=");
            String value = userTokenMap.getOrDefault(key, "").toString();
            userTokenStringBuilder.append(value).append(symbol);
        }

        String string = userTokenStringBuilder.toString();
        String subLastSymbol = LangUtil.subLastSymbol(string, symbol);
        return subLastSymbol + "}";
    }


    /**
     * 校验用户令牌签名
     *
     * @param useTokenInfoMap 用户令牌信息
     * @return 签名结果
     */
    protected boolean checkUserTokenSign(Map<String, Object> useTokenInfoMap) {
        String userTokenId = useTokenInfoMap.getOrDefault("userTokenId", "").toString();
        String createTime = useTokenInfoMap.getOrDefault("createTime", "").toString();
        String expireAt = useTokenInfoMap.getOrDefault("expireAt", "").toString();
        String pid = useTokenInfoMap.getOrDefault("pid", "").toString();
        String orgCode = useTokenInfoMap.getOrDefault("orgCode", "").toString();
        String ip = useTokenInfoMap.getOrDefault("ip", "").toString();
        String mid = useTokenInfoMap.getOrDefault("mid", "").toString();
        String env = useTokenInfoMap.getOrDefault("env", "").toString();
        String sign = useTokenInfoMap.getOrDefault("sign", "").toString();

        String origin = "userTokenId=" + userTokenId +
                "&createTime=" + createTime + "&expireAt=" + expireAt + "&pid=" + pid + "&orgCode=" + orgCode
                + "&ip=" + ip + "&mid=" + mid + "&env=" + env;
        log.info("用户令牌校验签名签名原文：{}", origin);

        String generalSign = smFactory.getSM3().summary(origin).toString().toLowerCase(Locale.ROOT);
        log.info("用户令牌校验签名签名后的值：{}", generalSign);
        log.info("用户令牌校验签名令牌信息中的签名值：{}", sign);

        boolean ignoreCase = generalSign.equalsIgnoreCase(sign);
        log.info("用户令牌：{} 校验签名签名结果：{}", userTokenId, ignoreCase);

        return ignoreCase;
    }

    /**
     * 校验应用令牌签名
     *
     * @param apTokenInfoMap 应用令牌信息
     * @return 签名结果
     */
    protected boolean checkAppTokenSign(Map<String, Object> apTokenInfoMap) {
        String appTokenId = apTokenInfoMap.getOrDefault("appTokenId", "").toString();
        String createTime = apTokenInfoMap.getOrDefault("createTime", "").toString();
        String expireAt = apTokenInfoMap.getOrDefault("expireAt", "").toString();
        String appId = apTokenInfoMap.getOrDefault("appId", "").toString();
        String userToken = apTokenInfoMap.getOrDefault("userToken", "").toString();
        String sign = apTokenInfoMap.getOrDefault("sign", "").toString();

        String origin = "appTokenId=" + appTokenId + "&createTime=" + createTime + "&expireAt=" +
                expireAt + "&appId=" + appId + "&userToken=" + userToken;
        log.info("应用令牌校验签名签名原文：{}", origin);

        String generalSign = smFactory.getSM3().summary(origin).toString().toLowerCase(Locale.ROOT);
        log.info("应用令牌校验签名签名后的值：{}", generalSign);
        log.info("应用令牌校验签名令牌信息中的签名值：{}", sign);

        boolean ignoreCase = generalSign.equalsIgnoreCase(sign);
        log.info("应用令牌：{} 校验签名签名结果：{}", appTokenId, ignoreCase);

        return ignoreCase;
    }
}
