package com.dragonsoft.dcuc.approve.business.external.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.PageUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragoninfo.dcuc.common.entity.ApiResultPage;
import com.dragoninfo.dcuc.common.entity.ApiSearchReq;
import com.dragoninfo.dcuc.common.entity.SearchParam;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.business.external.IDcucBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveBeanConstants;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.ApiOrgResultVo;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.ApiPoliceResultVo;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.OauthUserVO;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.UserLabel2RespVO;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.utils.ProofsUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.net.URI;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lidr huangziquan
 * @date 2021/2/23
 */
@Component
public class DcucBusinessImpl implements IDcucBusiness {
    private static final Logger logger = LoggerFactory.getLogger(DcucBusinessImpl.class);

    public static final String OAUTH_USER_PATH = "/api/user-service/v1/users/%s";
    public static final String ORG_INFO_PATH = "/api/org-service/v3/orgs/%s";

    /**
     * 警员同步API
     */
    public static final String POLICE_SYNC_API = "/api/user-service/v3/users/police/search";
    /**
     * 机构同步API
     */
    public static final String ORG_SYNC_API = "/api/org-service/v3/orgs/search";

    /**
     * 警员详情PI
     */
    public static final String POLICE_DETAIL_API = "/api/user-service/v3/users/police/%s";

    /**
     * 用户标签查询
     */
    public static final String LABEL_USER_API = "/api/user-service/v1/users/label/search";

    private RestTemplate restTemplate;

    private ApproveProperties approveProperties;

    private ObjectMapper objectMapper;

    @Qualifier(ApproveBeanConstants.DRAGON_OBJECT_MAPPER)
    @Autowired
    public void setObjectMapper(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    @Qualifier(ApproveBeanConstants.SKIP_SSL_REST_TEMPLATE)
    @Autowired
    public void setRestTemplate(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Override
    public OauthUserVO getOauthUserInfoByIdcard(String searchIdcard) {
        Assert.notBlank(searchIdcard);
        //构建请求头

        HttpEntity<Map<String, Object>> requestEntity;

        SecurityUser currentUser = UserContextUtils.getCurrentUser();
        if (ObjectUtil.isNotNull(currentUser)) {
            HttpHeaders headers = ProofsUtil.buildHttpHeaders(currentUser);
            requestEntity = new HttpEntity<>(headers);
        } else {
            requestEntity = new HttpEntity<>(Collections.emptyMap());
        }
        //远程调用用户中心接口
        String userServiceUrl = approveProperties.getUserServiceUrl();
        String url = userServiceUrl + String.format(OAUTH_USER_PATH, searchIdcard);
        logger.info("向认证服务查询用户信息请求路径：{}", url);
        ResponseEntity<String> resEntity = restTemplate.exchange(url, HttpMethod.GET, requestEntity, String.class);
        logger.info("向认证服务查询用户信息返回结果resEntity=【{}】,请求url=【{}】", JSONObject.toJSONString(resEntity), url);
        //判断请求是否成功
        OauthUserVO oauthUserVO = null;
        if (resEntity.getStatusCode() == HttpStatus.OK) {
            ResponseDTO<OauthUserVO> responseDTO = JSONObject.parseObject(resEntity.getBody(), ResponseDTO.class);
            if (responseDTO != null) {
                if (ResponseUtil.isSuccess(responseDTO)) {
                    responseDTO.getResult();
                    oauthUserVO = JSONObject.parseObject(JSON.toJSONString(responseDTO.getResult()), OauthUserVO.class);
                }
            }
        }
        return oauthUserVO;
    }

    @Override
    public ApiOrgResultVo orgInfo(String orgCode) {
        //构建请求头

        HttpEntity<Map<String, Object>> requestEntity;
        SecurityUser currentUser = UserContextUtils.getCurrentUser();
        if (ObjectUtil.isNotNull(currentUser)) {
            HttpHeaders headers = ProofsUtil.buildHttpHeaders(currentUser);
            requestEntity = new HttpEntity<>(headers);
        } else {
            requestEntity = new HttpEntity<>(Collections.emptyMap());
        }
        //远程调用用户中心接口
        String userServiceUrl = approveProperties.getUserServiceUrl();
        String url = userServiceUrl + String.format(ORG_INFO_PATH, orgCode);
        logger.info("向认证服务查询机构信息请求路径：{}", url);
        ResponseEntity<String> resEntity = restTemplate.exchange(url, HttpMethod.GET, requestEntity, String.class);
        logger.info("向认证服务查询机构信息返回结果resEntity=【{}】,请求url=【{}】", JSONObject.toJSONString(resEntity), url);
        //判断请求是否成功
        ApiOrgResultVo apiOrgResultVo = null;
        if (resEntity.getStatusCode() == HttpStatus.OK) {
            ResponseDTO<ApiOrgResultVo> responseDTO = JSONObject.parseObject(resEntity.getBody(), ResponseDTO.class);
            if (responseDTO != null) {
                if (ResponseUtil.isSuccess(responseDTO)) {
                    responseDTO.getResult();
                    apiOrgResultVo = JSONObject.parseObject(JSON.toJSONString(responseDTO.getResult()), ApiOrgResultVo.class);
                }
            }
        }
        return apiOrgResultVo;
    }

    @Override
    @SneakyThrows(JsonProcessingException.class)
    public ApiResultPage<ApiOrgResultVo> orgPage(ApiSearchReq apiSearchReq) {
        Assert.notNull(apiSearchReq);

        String userServiceUrl = approveProperties.getUserServiceUrl();
        String url = userServiceUrl + ORG_SYNC_API;

        HttpHeaders httpHeaders = HttpHeaders.EMPTY;
        SecurityUser currentUser = UserContextUtils.getCurrentUser();
        if (ObjectUtil.isNotNull(currentUser)) {
            httpHeaders = ProofsUtil.buildHttpHeaders(currentUser);
        }

        RequestEntity<ApiSearchReq> apiSearchReqRequestEntity = new RequestEntity<>(apiSearchReq, httpHeaders, HttpMethod.POST, URI.create(url));
        logger.info("请求机构信息参数：{}", JsonUtils.toJSONString(apiSearchReqRequestEntity));
        ResponseEntity<String> resEntity = restTemplate.exchange(apiSearchReqRequestEntity, String.class);
        String jsonString = JsonUtils.toJSONString(resEntity);
        logger.info("请求机构信息返回参数：{}", jsonString);
        if (!resEntity.getStatusCode().is2xxSuccessful()) {
            logger.error("请求获取机构信息错误：{}", jsonString);
            return null;
        }

        String body = resEntity.getBody();
        ApiResult apiResult = JSONObject.parseObject(body, ApiResult.class);
        if (ObjectUtil.isNull(apiResult)) {
            logger.error("请求获取机构信息错误：{}", body);
            return null;
        }
        if (!apiResult.getStatusCode().equalsIgnoreCase(ResponseStatus.SUCCESS_CODE)) {
            logger.error("请求获取机构信息错误：{}", body);
            return null;
        }

        Object result = apiResult.getResult();
        String resultJson = objectMapper.writeValueAsString(result);
        TypeReference<ApiResultPage<ApiOrgResultVo>> typeReference = new TypeReference<ApiResultPage<ApiOrgResultVo>>() {
        };
        return objectMapper.readValue(resultJson, typeReference);

    }


    @Override
    public ApiPoliceResultVo policeDetailByIdcard(String searchIdcard) {
        Assert.notBlank(searchIdcard);
        String requestName = "获取警员详情";

        HttpEntity<Map<String, Object>> requestEntity;

        SecurityUser currentUser = UserContextUtils.getCurrentUser();
        if (ObjectUtil.isNotNull(currentUser)) {
            HttpHeaders headers = ProofsUtil.buildHttpHeaders(currentUser);
            requestEntity = new HttpEntity<>(headers);
        } else {
            requestEntity = new HttpEntity<>(Collections.emptyMap());
        }
        //远程调用用户中心接口
        String userServiceUrl = approveProperties.getUserServiceUrl();
        String url = userServiceUrl + String.format(POLICE_DETAIL_API, searchIdcard);
        logger.info("向认证服务查询" + requestName + "请求路径：{}", url);
        ResponseEntity<String> resEntity = restTemplate.exchange(url, HttpMethod.GET, requestEntity, String.class);
        logger.info("向认证服务查询" + requestName + "返回结果resEntity=【{}】,请求url=【{}】", JSONObject.toJSONString(resEntity), url);
        //判断请求是否成功
        ApiPoliceResultVo oauthUserVO = null;
        if (resEntity.getStatusCode() == HttpStatus.OK) {
            ResponseDTO<OauthUserVO> responseDTO = JSONObject.parseObject(resEntity.getBody(), ResponseDTO.class);
            if (responseDTO != null) {
                if (ResponseUtil.isSuccess(responseDTO)) {
                    responseDTO.getResult();
                    oauthUserVO = JSONObject.parseObject(JSON.toJSONString(responseDTO.getResult()), ApiPoliceResultVo.class);
                }
            }
        }
        return oauthUserVO;
    }

    @Override
    public List<String> getUserByLabelCodeUserList(String orgCode, String labelCode) {
        List<UserLabel2RespVO> userByLabelCodeList = Collections.emptyList();
        try {
            userByLabelCodeList = getUserByLabelCode(orgCode, labelCode);
        } catch (RuntimeException e) {
            logger.error("查询审批人标签代码异常", e);
        }

        return Optional.ofNullable(userByLabelCodeList)
                .orElse(Collections.emptyList())
                .stream().map(UserLabel2RespVO::getIdcard)
                .collect(Collectors.toList());
    }

    @Override
    public List<UserLabel2RespVO> getUserByLabelCode(String orgCode, String labelCode) {
        SearchParam labelSearchParam = new SearchParam();
        labelSearchParam.setOperator(SearchOperator.eq.name());
        labelSearchParam.setValue(labelCode);

        SearchParam orgCodeSearchParam = new SearchParam();
        orgCodeSearchParam.setOperator(SearchOperator.eq.name());
        orgCodeSearchParam.setValue(orgCode);

        Map<String, SearchParam> filters = new HashMap<>(2);
        filters.put("orgCode", orgCodeSearchParam);
        filters.put("labelCode", labelSearchParam);
        int size = 1000;
        int currentPage = 1;

        ApiSearchReq apiSearchReq = ApiSearchReq.reqInfo(currentPage, size, filters);

        ApiResultPage<UserLabel2RespVO> page = labelPage(apiSearchReq);
        List<UserLabel2RespVO> allContent = new ArrayList<>(page.getContent());

        int total = 0;
        if (ObjectUtil.isNotNull(page)) {
            total = Math.toIntExact(page.getTotal());
        }

        int totalPage = PageUtil.totalPage(total, size);
        currentPage++;
        // 循环获取全部
        for (int i = currentPage; i <= totalPage; i++) {
            ApiResultPage<UserLabel2RespVO> itemPage = labelPage(apiSearchReq);
            if (ObjectUtil.isNotNull(itemPage)) {
                allContent.addAll(itemPage.getContent());
            }
        }

        return allContent;
    }

    @SneakyThrows(JsonProcessingException.class)
    public ApiResultPage<UserLabel2RespVO> labelPage(ApiSearchReq apiSearchReq) {
        Assert.notNull(apiSearchReq);
        String requestName = "获取人员标签列表";

        String userServiceUrl = approveProperties.getUserServiceUrl();
        String url = userServiceUrl + LABEL_USER_API;

        HttpHeaders httpHeaders = HttpHeaders.EMPTY;
        SecurityUser currentUser = UserContextUtils.getCurrentUser();
        if (ObjectUtil.isNotNull(currentUser)) {
            httpHeaders = ProofsUtil.buildHttpHeaders(currentUser);
        }

        RequestEntity<ApiSearchReq> apiSearchReqRequestEntity = new RequestEntity<>(apiSearchReq, httpHeaders, HttpMethod.POST, URI.create(url));
        logger.info(requestName + "参数：{}", JsonUtils.toJSONString(apiSearchReqRequestEntity));
        ResponseEntity<String> resEntity = restTemplate.exchange(apiSearchReqRequestEntity, String.class);
        String jsonString = JsonUtils.toJSONString(resEntity);
        logger.info(requestName + "返回参数：{}", jsonString);
        if (!resEntity.getStatusCode().is2xxSuccessful()) {
            logger.error(requestName + "信息错误：{}", jsonString);
            return null;
        }

        String body = resEntity.getBody();
        ApiResult apiResult = objectMapper.readValue(body, ApiResult.class);
        if (ObjectUtil.isNull(apiResult)) {
            logger.error(requestName + "信息错误：{}", body);
            return null;
        }
        if (!apiResult.getStatusCode().equalsIgnoreCase(ResponseStatus.SUCCESS_CODE)) {
            logger.error(requestName + "信息错误：{}", body);
            return null;
        }

        Object result = apiResult.getResult();
        String resultJson = objectMapper.writeValueAsString(result);
        TypeReference<ApiResultPage<UserLabel2RespVO>> typeReference = new TypeReference<ApiResultPage<UserLabel2RespVO>>() {
        };
        return objectMapper.readValue(resultJson, typeReference);
    }

    @Override
    @SneakyThrows(JsonProcessingException.class)
    public ApiResultPage<ApiPoliceResultVo> policePage(ApiSearchReq apiSearchReq) {
        Assert.notNull(apiSearchReq);
        String requestName = "获取警员列表";

        String userServiceUrl = approveProperties.getUserServiceUrl();
        String url = userServiceUrl + POLICE_SYNC_API;

        HttpHeaders httpHeaders = HttpHeaders.EMPTY;
        SecurityUser currentUser = UserContextUtils.getCurrentUser();
        if (ObjectUtil.isNotNull(currentUser)) {
            httpHeaders = ProofsUtil.buildHttpHeaders(currentUser);
        }

        RequestEntity<ApiSearchReq> apiSearchReqRequestEntity = new RequestEntity<>(apiSearchReq, httpHeaders, HttpMethod.POST, URI.create(url));
        logger.info(requestName + "参数：{}", JsonUtils.toJSONString(apiSearchReqRequestEntity));
        ResponseEntity<String> resEntity = restTemplate.exchange(apiSearchReqRequestEntity, String.class);
        String jsonString = JsonUtils.toJSONString(resEntity);
        logger.info(requestName + "返回参数：{}", jsonString);
        if (!resEntity.getStatusCode().is2xxSuccessful()) {
            logger.error(requestName + "信息错误：{}", jsonString);
            return null;
        }

        String body = resEntity.getBody();
        ApiResult apiResult = JSONObject.parseObject(body, ApiResult.class);
        if (ObjectUtil.isNull(apiResult)) {
            logger.error(requestName + "信息错误：{}", body);
            return null;
        }
        if (!apiResult.getStatusCode().equalsIgnoreCase(ResponseStatus.SUCCESS_CODE)) {
            logger.error(requestName + "信息错误：{}", body);
            return null;
        }

        Object result = apiResult.getResult();
        String resultJson = objectMapper.writeValueAsString(result);
        TypeReference<ApiResultPage<ApiPoliceResultVo>> typeReference = new TypeReference<ApiResultPage<ApiPoliceResultVo>>() {
        };
        return objectMapper.readValue(resultJson, typeReference);
    }

}
