package com.dragonsoft.dcuc.approve.business.external.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.PageUtil;
import com.dragoninfo.dcuc.common.annotation.CacheLock;
import com.dragoninfo.dcuc.common.entity.ApiResultPage;
import com.dragoninfo.dcuc.common.entity.ApiSearchReq;
import com.dragoninfo.dcuc.common.entity.SearchParam;
import com.dragonsoft.dcuc.approve.business.ICacheBusiness;
import com.dragonsoft.dcuc.approve.business.external.IDcucBusiness;
import com.dragonsoft.dcuc.approve.business.external.IUserOrgSyncBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveLockConstants;
import com.dragonsoft.dcuc.approve.model.ApproveOrgInfo;
import com.dragonsoft.dcuc.approve.model.ApproveUserInfo;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.ApiOrgResultVo;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.ApiPoliceResultVo;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.service.IApproveOrgInfoService;
import com.dragonsoft.dcuc.approve.service.IApproveUserInfoService;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/12
 */
@Slf4j
@Component
public class UserOrgSyncBusinessImpl implements IUserOrgSyncBusiness {

    private IDcucBusiness dcucBusiness;

    private ApproveProperties approveProperties;

    private IApproveUserInfoService approveUserInfoService;

    private IApproveOrgInfoService approveOrgInfoService;

    private ICacheBusiness cacheBusiness;

    @Autowired
    public void setCacheBusiness(ICacheBusiness cacheBusiness) {
        this.cacheBusiness = cacheBusiness;
    }

    @Autowired
    public void setApproveOrgInfoService(IApproveOrgInfoService approveOrgInfoService) {
        this.approveOrgInfoService = approveOrgInfoService;
    }

    @Autowired
    public void setApproveUserInfoService(IApproveUserInfoService approveUserInfoService) {
        this.approveUserInfoService = approveUserInfoService;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Autowired
    public void setDcucBusiness(IDcucBusiness dcucBusiness) {
        this.dcucBusiness = dcucBusiness;
    }

    @CacheLock(prefix = ApproveLockConstants.SYNC_ORG)
    @Override
    public ResponseStatus syncAllOrg() {
        return processOrg(Collections.emptyMap());
    }

    @CacheLock(prefix = ApproveLockConstants.SYNC_USER)
    @Override
    public ResponseStatus syncAllUser() {
        return processUser(Collections.emptyMap());
    }

    @CacheLock(prefix = ApproveLockConstants.SYNC_ORG)
    @Override
    public ResponseStatus syncNewOrg() {
        Date lastUpdateTime = getOrgSyncLastTime();
        Integer syncStartOffsetMinute = approveProperties.getSyncStartOffsetMinute();

        DateTime startDateTime = DateUtil.offsetMinute(lastUpdateTime, -syncStartOffsetMinute);

        DateTime date = DateUtil.date();
        DateTime endDateTime = DateUtil.offsetMinute(date, syncStartOffsetMinute);
        Map<String, SearchParam> stringSearchParamMap = generalFilter(startDateTime, endDateTime, "updateDate");
        return processOrg(stringSearchParamMap);
    }

    @CacheLock(prefix = ApproveLockConstants.SYNC_USER)
    @Override
    public ResponseStatus syncNewUser() {
        Date lastUpdateTime = getUserSyncLastTime();
        Integer syncStartOffsetMinute = approveProperties.getSyncStartOffsetMinute();

        DateTime startDateTime = DateUtil.offsetMinute(lastUpdateTime, -syncStartOffsetMinute);

        DateTime date = DateUtil.date();
        DateTime endDateTime = DateUtil.offsetMinute(date, syncStartOffsetMinute);
        Map<String, SearchParam> stringSearchParamMap = generalFilter(startDateTime, endDateTime, "updateTime");
        return processUser(stringSearchParamMap);
    }

    /**
     * 处理用户信息
     *
     * @param stringSearchParamMap 筛选条件
     * @return 状态
     */
    protected ResponseStatus processOrg(Map<String, SearchParam> stringSearchParamMap) {
        int orgTotal = getOrgTotal(stringSearchParamMap);
        Integer syncPageSize = approveProperties.getSyncPageSize();
        int totalPage = getTotalPage(orgTotal, syncPageSize);
        int currentPage = 1;
        log.info("开始同步机构数据：总数 ：{} ，页码：{} ，需同步页数：{}", orgTotal, syncPageSize, totalPage);
        log.info("开始同步机构数据：时间范围：{}", JsonUtils.toJSONString(stringSearchParamMap));

        Date syncStartTime = DateUtils.getDate();
        for (int i = currentPage; i <= totalPage; i++) {
            log.info("开始同步机构数据：需同步页数：{} ,当前页数:{}", totalPage, i);

            ApiSearchReq apiSearchReq = ApiSearchReq.reqInfo(i, syncPageSize, stringSearchParamMap);
            ApiResultPage<ApiOrgResultVo> apiPoliceResultVoApiResultPage = dcucBusiness.orgPage(apiSearchReq);
            List<ApiOrgResultVo> content = apiPoliceResultVoApiResultPage.getContent();

            List<ApproveOrgInfo> addList = new ArrayList<>();
            List<ApproveOrgInfo> updateList = new ArrayList<>();

            for (ApiOrgResultVo apiPoliceResultVo : content) {
                String orgCode = apiPoliceResultVo.getOrgCode();
                ApproveOrgInfo approveOrgInfo = apiPoliceResultVo.toApproveOrgInfo();

                ApproveOrgInfo exitsApproveUserInfo = approveOrgInfoService.selectByOrgCode(orgCode);
                if (ObjectUtil.isNotNull(exitsApproveUserInfo)) {
                    approveOrgInfo.setCreateTime(exitsApproveUserInfo.getCreateTime());
                    approveOrgInfo.setId(exitsApproveUserInfo.getId());
                    updateList.add(approveOrgInfo);
                } else {
                    addList.add(approveOrgInfo);
                }
            }

            approveOrgInfoService.updateAllList(updateList);
            approveOrgInfoService.saveAllList(addList);
        }
        approveOrgInfoService.updateUpOrgId(syncStartTime);
        cacheBusiness.saveOrgSyncTime(DateUtils.getDate());
        return ResponseStatus.success();
    }

    /**
     * 处理用户信息
     *
     * @param stringSearchParamMap 筛选条件
     * @return 状态
     */
    protected ResponseStatus processUser(Map<String, SearchParam> stringSearchParamMap) {
        int orgTotal = getPoliceTotal(stringSearchParamMap);
        Integer syncPageSize = approveProperties.getSyncPageSize();
        int totalPage = getTotalPage(orgTotal, syncPageSize);
        int currentPage = 1;
        log.info("开始同步用户数据：总数 ：{} ，页码：{} ，需同步页数：{}", orgTotal, syncPageSize, totalPage);
        log.info("开始同步用户数据：时间范围：{}", JsonUtils.toJSONString(stringSearchParamMap));


        for (int i = currentPage; i <= totalPage; i++) {
            log.info("开始同步用户数据：需同步页数：{} ,当前页数:{}", totalPage, i);

            ApiSearchReq apiSearchReq = ApiSearchReq.reqInfo(i, syncPageSize, stringSearchParamMap);
            ApiResultPage<ApiPoliceResultVo> apiPoliceResultVoApiResultPage = dcucBusiness.policePage(apiSearchReq);
            List<ApiPoliceResultVo> content = apiPoliceResultVoApiResultPage.getContent();

            List<ApproveUserInfo> addList = new ArrayList<>();
            List<ApproveUserInfo> updateList = new ArrayList<>();

            for (ApiPoliceResultVo apiPoliceResultVo : content) {
                String idcard = apiPoliceResultVo.getIdcard();
                ApproveUserInfo approveUserInfo = apiPoliceResultVo.toApproveUserInfo();

                ApproveUserInfo exitsApproveUserInfo = approveUserInfoService.selectByIdcard(idcard);
                if (ObjectUtil.isNotNull(exitsApproveUserInfo)) {
                    approveUserInfo.setCreateTime(exitsApproveUserInfo.getCreateTime());
                    approveUserInfo.setId(exitsApproveUserInfo.getId());
                    updateList.add(approveUserInfo);
                } else {
                    addList.add(approveUserInfo);
                }
            }

            approveUserInfoService.updateAllList(updateList);
            approveUserInfoService.saveAllList(addList);
        }
        cacheBusiness.saveUserSyncTime(DateUtils.getDate());
        return ResponseStatus.success();
    }

    /**
     * 生成总页数，从第一页开始
     *
     * @param total 总数量
     * @param size  页面大小
     * @return 总页数
     */
    protected int getTotalPage(int total, int size) {
        return PageUtil.totalPage(total, size);
    }

    /**
     * 获取机构总条数
     *
     * @param filterMap 过滤条件
     * @return 总条数
     */
    protected int getOrgTotal(Map<String, SearchParam> filterMap) {
        ApiSearchReq apiSearchReq = ApiSearchReq.reqInfo(1, 1, filterMap);
        ApiResultPage<ApiOrgResultVo> apiOrgResultVoApiResultPage = dcucBusiness.orgPage(apiSearchReq);
        int total = 0;
        if (ObjectUtil.isNotNull(apiOrgResultVoApiResultPage)) {
            total = Math.toIntExact(apiOrgResultVoApiResultPage.getTotal());
        }
        return total;
    }

    /**
     * 获取警员总条数
     *
     * @param filterMap 过滤条件
     * @return 总条数
     */
    protected int getPoliceTotal(Map<String, SearchParam> filterMap) {
        ApiSearchReq apiSearchReq = ApiSearchReq.reqInfo(1, 1, filterMap);
        ApiResultPage<ApiPoliceResultVo> apiOrgResultVoApiResultPage = dcucBusiness.policePage(apiSearchReq);
        int total = 0;
        if (ObjectUtil.isNotNull(apiOrgResultVoApiResultPage)) {
            total = Math.toIntExact(apiOrgResultVoApiResultPage.getTotal());
        }
        return total;
    }

    /**
     * 生成过滤参数
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @param keyName   键名称
     * @return 过滤参数
     */
    protected Map<String, SearchParam> generalFilter(Date startTime, Date endTime, String keyName) {
        //设置同步时间
        SearchParam updateTimeSearch = new SearchParam();
        updateTimeSearch.setOperator(SearchOperator.rangeDate.name());
        String start = DateUtils.getDateFormat(startTime, DateConst.HYPHEN_DISPLAY_TIME);
        String end = DateUtils.getDateFormat(endTime, DateConst.HYPHEN_DISPLAY_TIME);
        updateTimeSearch.setValue(new String[]{start, end});

        Map<String, SearchParam> filters = new HashMap<>(2);
        filters.put(keyName, updateTimeSearch);
        return filters;
    }


    protected Date getUserSyncLastTime() {
        Date date = cacheBusiness.getLastUserSyncTime();

        if (ObjectUtil.isNull(date)) {
            date = approveUserInfoService.getLastUpdateTime();
        }
        return date;
    }

    protected Date getOrgSyncLastTime() {
        Date date = cacheBusiness.getLastOrgSyncTime();

        if (ObjectUtil.isNull(date)) {
            date = approveOrgInfoService.getLastUpdateTime();
        }
        return date;
    }

}
