package com.dragonsoft.dcuc.approve.business.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.duceap.enums.ManageDisposeTypeEnum;
import com.dragonsoft.dcuc.approve.business.IApproveBaseBusiness;
import com.dragonsoft.dcuc.approve.business.IApproveBusiness;
import com.dragonsoft.dcuc.approve.business.IApproveCommonBusiness;
import com.dragonsoft.dcuc.approve.business.IApproveLogBusiness;
import com.dragonsoft.dcuc.approve.business.IWorkFlowBusiness;
import com.dragonsoft.dcuc.approve.business.external.IDcucAuthBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.enumresources.ApproveResultEnum;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowTaskStateEnum;
import com.dragonsoft.dcuc.approve.enumresources.CommitTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.OperateOptionEnum;
import com.dragonsoft.dcuc.approve.enumresources.PermissionValidTypeEnum;
import com.dragonsoft.dcuc.approve.event.ApproveEventPublish;
import com.dragonsoft.dcuc.approve.model.ApproveCirculationInfo;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.TaskAssignRecord;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.ApproveMessageInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.BaseOperateReqDTO;
import com.dragonsoft.dcuc.approve.model.dto.ContentRespDTO;
import com.dragonsoft.dcuc.approve.model.dto.LogDto;
import com.dragonsoft.dcuc.approve.model.dto.OperateReqDTO;
import com.dragonsoft.dcuc.approve.model.dto.ResourceInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.TaskRequestUserInfoDTO;
import com.dragonsoft.dcuc.approve.model.req.AgentReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoUpdateReqVO;
import com.dragonsoft.dcuc.approve.model.req.ResourceInfoReqVO;
import com.dragonsoft.dcuc.approve.model.resp.ApprovalInfoRespVO;
import com.dragonsoft.dcuc.approve.model.resp.ApproveCommentVo;
import com.dragonsoft.dcuc.approve.model.resp.OperateRespVO;
import com.dragonsoft.dcuc.approve.model.resp.ProcessStatusDetailVo;
import com.dragonsoft.dcuc.approve.model.resp.ProcessStatusVo;
import com.dragonsoft.dcuc.approve.model.resp.TaskOutgoingVo;
import com.dragonsoft.dcuc.approve.model.resp.TaskVo;
import com.dragonsoft.dcuc.approve.model.vo.ApproveTaskRequestVo;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.service.IApproveActHiTaskService;
import com.dragonsoft.dcuc.approve.service.IApproveActRunTaskService;
import com.dragonsoft.dcuc.approve.service.IApproveCirculationService;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.dcuc.approve.service.ITaskAssignRecordService;
import com.dragonsoft.dcuc.approve.utils.ProofsUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.BaseSecurityUser;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.UUIDUtils;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.duwf.api.model.ActivitiHolder;
import com.dragonsoft.duceap.duwf.api.model.ApproveMessage;
import com.dragonsoft.duceap.duwf.api.model.ProcessInstanceVO;
import com.dragonsoft.duceap.duwf.api.model.TaskHistoryDetailVo;
import com.dragonsoft.duceap.duwf.api.model.TaskRequest;
import com.dragonsoft.duceap.duwf.api.model.TransMessage;
import com.dragonsoft.duceap.duwf.api.model.WorkflowRequestData;
import com.dragonsoft.duceap.workflow.client.WorkflowClientFactory;
import com.dragonsoft.duceap.workflow.enums.WorkFlowOprationEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;

import static com.dragonsoft.dcuc.approve.constants.ApproveConstants.END_STR;

/**
 * 代码千万行,注释第一行,编码不规范,同事两行泪
 *
 * @author huang(jy)
 * @version 1.0
 * @date 2021/2/3 19:43
 */
@Service
public class ApproveCommonBusinessImpl implements IApproveCommonBusiness {

    private static final Logger logger = LoggerFactory.getLogger(ApproveCommonBusinessImpl.class);

    private ApproveProperties approveProperties;

    private ITaskAssignRecordService taskAssignRecordService;

    private IApproveLogBusiness approveLogBusiness;

    private ApproveCommonBusinessImpl approveService;

    private IApproveRecordService approveRecordService;

    private IApproveCirculationService approveCirculationInfoService;

    private IApproveActHiTaskService approveActHiTaskService;

    private IApproveActRunTaskService approveActRunTaskService;

    private IDcucAuthBusiness dcucAuthBusiness;

    private IWorkFlowBusiness workFlowBusiness;

    private WorkflowClientFactory workflowClientFactory;

    private IApproveBusiness approveBusiness;

    private IApproveBaseBusiness approveBaseBusiness;

    private ApproveEventPublish approveEventPublish;

    @Autowired
    public void setApproveEventPublish(ApproveEventPublish approveEventPublish) {
        this.approveEventPublish = approveEventPublish;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Autowired
    public void setTaskAssignRecordService(ITaskAssignRecordService taskAssignRecordService) {
        this.taskAssignRecordService = taskAssignRecordService;
    }

    @Autowired
    public void setApproveLogBusiness(IApproveLogBusiness approveLogBusiness) {
        this.approveLogBusiness = approveLogBusiness;
    }

    @Autowired
    public void setApproveService(ApproveCommonBusinessImpl approveService) {
        this.approveService = approveService;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Autowired
    public void setApproveCirculationInfoService(IApproveCirculationService approveCirculationInfoService) {
        this.approveCirculationInfoService = approveCirculationInfoService;
    }

    @Autowired
    public void setApproveActHiTaskService(IApproveActHiTaskService approveActHiTaskService) {
        this.approveActHiTaskService = approveActHiTaskService;
    }

    @Autowired
    public void setApproveActRunTaskService(IApproveActRunTaskService approveActRunTaskService) {
        this.approveActRunTaskService = approveActRunTaskService;
    }

    @Autowired
    public void setDcucAuthBusiness(IDcucAuthBusiness dcucAuthBusiness) {
        this.dcucAuthBusiness = dcucAuthBusiness;
    }

    @Autowired
    public void setWorkFlowBusiness(IWorkFlowBusiness workFlowBusiness) {
        this.workFlowBusiness = workFlowBusiness;
    }

    @Autowired
    public void setWorkflowClientFactory(WorkflowClientFactory workflowClientFactory) {
        this.workflowClientFactory = workflowClientFactory;
    }

    @Autowired
    public void setApproveBusiness(IApproveBusiness approveBusiness) {
        this.approveBusiness = approveBusiness;
    }

    @Autowired
    public void setApproveBaseBusiness(IApproveBaseBusiness approveBaseBusiness) {
        this.approveBaseBusiness = approveBaseBusiness;
    }

    @Override
    public Page<TaskVo> queryApproveList(Searchable searchable) {
        // 获取当前查询条件的状态值
        String state = searchable.getSearchFilterByKey("state_eq").getValue().toString();
        searchable.removeSearchFilter("state", SearchOperator.eq);
        ApproveWorkFlowTaskStateEnum approveWorkFlowTaskStateEnum = EnumUtils.enumOf(ApproveWorkFlowTaskStateEnum.class, state);
        if (ApproveWorkFlowTaskStateEnum.FINISH.equals(approveWorkFlowTaskStateEnum)) {
            // 已办列表
            return approveActHiTaskService.buildApproveActHiTask(searchable);
        } else {
            // 待办列表
            return approveActRunTaskService.buildApproveActRunTask(searchable);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseDTO<String> save(ApprovalInfoReqVO approvalInfoReqVO) {
        ApproveRecordInfo result = saveBase(approvalInfoReqVO, ManageDisposeTypeEnum.INSERT, new Date());
        String processInstanceId = result.getProcessInstanceId();
        return ResponseUtil.newInstance(processInstanceId);
    }

    @Override
    public ApproveRecordInfo saveBase(ApprovalInfoReqVO approvalInfoReqVO, ManageDisposeTypeEnum manageDisposeTypeEnum, Date oldCreateTime) {
        ApproveRecordInfo approveRecordInfo = null;
        ApprovalInfoDTO approvalInfoDTO = null;
        ProcessInstanceVO processInstanceVO = null;
        ApproveResultEnum approveResultEnum = ApproveResultEnum.SUCCESS;
        String failMsg = "";
        boolean sendMessageFlag = true;
        try {
            logger.info("====开始处理表单提交,data:{}====", JSONObject.toJSONString(approvalInfoReqVO));

            // 构建ApprovalInfo对象
            approvalInfoDTO = buildApprovalInfo(approvalInfoReqVO);
            // 设置流程相关信息
            approveBusiness.setProcessInfo(approvalInfoDTO);

            logger.info("构建申请单信息");
            // 无审批内容，默认填充数据
            String applyContent = approvalInfoDTO.getApplyContent();
            if (StrUtil.isBlank(applyContent)) {
                String flowType = approvalInfoDTO.getFlowType();
                FlowTypeEnum flowTypeEnum = EnumUtils.enumOf(FlowTypeEnum.class, flowType);
                ResourceInfoReqVO resourceInfoReqVO = approvalInfoReqVO.getResourceInfoDTO();
                ResourceInfoDTO resourceInfoDTO = ResourceInfoDTO.parseResourceInfoReqVO(resourceInfoReqVO);
                ContentRespDTO contentRespDTO = approveBusiness.generalContentRespVO(flowTypeEnum, resourceInfoDTO);
                approvalInfoDTO.setApplyContent(JSON.toJSONString(contentRespDTO));
            }
            // 构建ApprovalResourceInfo资源对象
            // 生成审批单号
            String approveNo = approvalInfoReqVO.getApproveTaskId();
            if (StrUtil.isBlank(approveNo)) {
                String applicantOrgCode = approvalInfoReqVO.getApplicantOrgCode();
                String appCode = approvalInfoReqVO.getAppCode();
                approveNo = approveBaseBusiness.buildApproveNo(appCode, applicantOrgCode);
            }

            // 判断是否需要提交，创建流程
            String operateType = approvalInfoReqVO.getOperateType();
            CommitTypeEnum commitTypeEnum = EnumUtils.enumOf(CommitTypeEnum.class, operateType);
            if (CommitTypeEnum.APPLY_COMMIT.equals(commitTypeEnum)) {
                // 创建流程
                logger.info("===开始提交=====");
                // 构建ApproveTaskRequestVo，用于工作流给待办中心发送消息，需设置业务类型和业务id，因工作流对接多方应用且都需要给待办发消息
                ApproveTaskRequestVo approveTaskRequestVo = this.buildTaskRequestVo(approvalInfoDTO);
                String flowType = approvalInfoReqVO.getFlowType();
                approveTaskRequestVo.setFlowType(flowType);
                // 构建工作流申请的taskRequest对象
                TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();
                // 设置自定义对象ApproveTaskRequestVo
                taskRequest.setData(approveTaskRequestVo);
                String userId = approvalInfoDTO.getApplicantIdcard();
                logger.info("===提交的用户是:{}===", userId);
                // 调用工作流创建流程
                processInstanceVO = workflowClientFactory.getWorkflowTaskClient().startTaskByCode(approvalInfoReqVO.getBusinessCode(),
                        "", taskRequest, userId);
                String processInstanceId = processInstanceVO.getProcessInstanceId();
                List<TaskHistoryDetailVo> taskRunDetailVos = processInstanceVO.getTaskRunDetailVos();

                approvalInfoDTO.setProcessInstanceId(processInstanceId);

                // 流程开启后生成待办(刚开启的节点没有已办事宜)
                approveBusiness.createTodoAfterProcessStart(processInstanceVO, approvalInfoDTO);

                TaskRequestUserInfoDTO taskRequestUserInfoDTO = new TaskRequestUserInfoDTO();
                taskRequestUserInfoDTO.setIdCard(approvalInfoReqVO.getApplicantIdcard());
                taskRequestUserInfoDTO.setUserName(approvalInfoReqVO.getApplicantName());
                String applicantIdcard = approvalInfoReqVO.getApplicantIdcard();
                ResourceInfoReqVO resourceInfoReqVO = approvalInfoReqVO.getResourceInfoDTO();
                FlowTypeEnum flowTypeEnum = EnumUtils.enumOf(FlowTypeEnum.class, flowType);
                ResourceInfoDTO resourceInfoDTO = ResourceInfoDTO.parseResourceInfoReqVO(resourceInfoReqVO);
                OperateRespVO operateRespVO = dcucAuthBusiness.checkHitRedList(applicantIdcard, flowTypeEnum, resourceInfoDTO);

                // 构建流转的taskRequest
                TaskRequest<ApproveTaskRequestVo> commitTaskRequest = new TaskRequest<>();
                // 再次构建taskRequest
                this.buildTaskRequest(commitTaskRequest, approvalInfoDTO, taskRequestUserInfoDTO, approvalInfoReqVO.getActivitiHolderList(),
                        processInstanceVO.getTaskRunDetailVos().get(0).getId(), operateRespVO.getHitRedList());

                ApproveResultEnum approveResult = ApproveResultEnum.APPROVE_ING;
                if (CollectionUtils.isNotEmpty(taskRunDetailVos)) {
                    for (TaskHistoryDetailVo taskDetailVo : taskRunDetailVos) {

                        BaseOperateReqDTO baseOperateReqDTO = BaseOperateReqDTO.builder()
                                .operateIdcard(applicantIdcard)
                                .taskId(taskDetailVo.getId())
                                .operationEnum(ApproveWorkFlowOperationEnum.COMPLETE_AGREE)
                                .taskRequest(commitTaskRequest)
                                .approvalInfoDTO(approvalInfoDTO)
                                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                                .approveNo(approveNo)
                                .processNo(approvalInfoDTO.getBusinessCode())
                                .approveRecordId(approvalInfoDTO.getId())
                                .hitRedList(operateRespVO.getHitRedList())
                                .build();

                        // 调用工作流提交节点,审批时使用申请人的身份证号码
                        processInstanceVO = baseApproveOperate(baseOperateReqDTO);
                        if (CollectionUtils.isNotEmpty(taskRunDetailVos)) {
                            sendMessageFlag = false;
                        }
                        logger.info("工作流申请流程：【{}】,taskId:【{}】", JSON.toJSONString(processInstanceVO), taskDetailVo.getId());
                        // 生成待办，更新已办数据
                        logger.info("====开始执行提交后续动作====");
                        List<TaskHistoryDetailVo> afterTaskRunDetailVos = processInstanceVO.getTaskRunDetailVos();
                        approveBusiness.afterTaskCommitTask(taskDetailVo.getId(), ApproveWorkFlowOperationEnum.COMPLETE_AGREE,
                                approvalInfoDTO.getApplicantIdcard(), approvalInfoDTO.getApplicantName(), afterTaskRunDetailVos);
                    }
                    logger.info("===发送消息标志:{}====", sendMessageFlag);
                    if (sendMessageFlag) {
                        ApproveMessageInfoDTO approveMessageInfoDTO = ApproveMessageInfoDTO.newInstance(approvalInfoDTO, ApproveWorkFlowOperationEnum.COMPLETE_AGREE, "");
                        // 发送消息
                        approveEventPublish.publishApproveFinalResultEvent(approveMessageInfoDTO);
                        approveResult = ApproveResultEnum.SUCCESS;
                    }
                }
                approvalInfoDTO.setApproveResult(approveResult.value);
                // 构建任务id、任务标识码、任务名称字段，多个均以,相隔
                // 需使用最新的processInstanceVO
                approveBusiness.buildTaskInfoByProcessInstanceId(approvalInfoDTO, processInstanceVO.getTaskRunDetailVos());
                logger.info("===保存表单信息====");
                // 将原本的旧表对象构建成模式3的新表对象
            } else {
                logger.info("===保存表单信息====");
                approvalInfoDTO.setUpdateTime(new Date());
                approvalInfoDTO.setUpdateUser(ProofsUtil.getIdcard());
            }

            // 将原本的旧表对象构建成模式3的新表对象
            approveRecordInfo = approveRecordService.buildApproveRecordInfo(approvalInfoDTO, approveNo);
            approveRecordInfo.setCreateTime(oldCreateTime);
            // 保存数据
            approveRecordService.save(approveRecordInfo);
        } catch (Exception e) {
            approveResultEnum = ApproveResultEnum.FAIL;
            failMsg = e.getMessage();
            logger.error("e", e);
            throw e;
        } finally {
            // 获取最新的
            if (approveRecordInfo != null) {
                approvalInfoDTO = approveRecordInfo.toApprovalInfo();
            }
            LogDto logDto = approveLogBusiness.buildLogDto(approvalInfoDTO, processInstanceVO, approveResultEnum, failMsg, false, null);
            CommitTypeEnum commitTypeEnum = EnumUtils.enumOf(CommitTypeEnum.class, approvalInfoReqVO.getOperateType());
            approveLogBusiness.buildFlowManageLogInsert(logDto, commitTypeEnum, manageDisposeTypeEnum, sendMessageFlag);
        }
        return approveRecordInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseDTO<String> update(ApprovalInfoUpdateReqVO approvalInfoUpdateReqVO) {
        logger.info("=======" + JSONObject.toJSONString(approvalInfoUpdateReqVO) + "========");
        ApproveRecordInfo approveRecordInfo;
        String approvalId = approvalInfoUpdateReqVO.getId();
        ApprovalInfoDTO approvalInfoDTO = getApproveRecord(approvalId);
        ProcessInstanceVO processInstanceVO = null;
        ApproveResultEnum approveResultEnum = ApproveResultEnum.SUCCESS;
        String failMsg = "";
        // 是否需要保存修改日志（若是调用了保存接口则无需。）
        boolean needSaveUpdateLog = true;
        // 是否需要保存操作日志（已创建了流程，此次只是修改表单但是不提交）
        boolean needSaveOperateLog = false;

        // 是否发送消息给权限中心，默认发送
        boolean sendMessageFlag = true;
        try {
            logger.info("=======" + JSONObject.toJSONString(approvalInfoUpdateReqVO) + "========");
            // 实例id
            String instanceId = approvalInfoUpdateReqVO.getProcessInstanceId();
            // 判断实例id是否存在，不存在的话，原本为草稿状态
            if (StringUtils.isEmpty(instanceId)) {
                ApprovalInfoReqVO approvalInfoReqVO = new ApprovalInfoReqVO();
                BeanUtils.copyProperties(approvalInfoUpdateReqVO, approvalInfoReqVO);

                if (StringUtils.isNotEmpty(approvalId)) {
                    logger.info("====删除草稿信息====");
                    // 草稿状态下，先删除原本表单
                    approveRecordService.deleteById(approvalId);
                }
                needSaveUpdateLog = false;
                // 调用保存接口
                ApproveRecordInfo result = approveService.saveBase(approvalInfoReqVO, ManageDisposeTypeEnum.UPDATE, approvalInfoDTO.getCreateTime());
                return ResponseUtil.newInstance(result.getProcessInstanceId());
            } else {
                String processTitle = approvalInfoUpdateReqVO.getProcessTitle();
                if (StrUtil.isBlank(processTitle)) {
                    approvalInfoUpdateReqVO.setProcessTitle(approvalInfoDTO.getProcessTitle());
                }

                if (StrUtil.isBlank(approvalInfoUpdateReqVO.getAppCode())) {
                    approvalInfoUpdateReqVO.setAppCode(approvalInfoDTO.getAppCode());
                }
                if (StrUtil.isBlank(approvalInfoUpdateReqVO.getFlowType())) {
                    approvalInfoUpdateReqVO.setFlowType(approvalInfoDTO.getFlowType());
                }
                ResourceInfoReqVO resourceInfoReqVO = approvalInfoUpdateReqVO.getResourceInfoDTO();
                ResourceInfoDTO resourceInfoDTO = ResourceInfoDTO.parseResourceInfoReqVO(resourceInfoReqVO);
                approvalInfoDTO.setResourceInfoVo(resourceInfoDTO);
                BeanUtils.copyProperties(approvalInfoUpdateReqVO, approvalInfoDTO);
                // 判断有效期类型
                String permissionValidType = approvalInfoUpdateReqVO.getPermissionValidType();
                PermissionValidTypeEnum permissionValidTypeEnum = EnumUtils.enumOf(PermissionValidTypeEnum.class, permissionValidType);
                if (permissionValidTypeEnum.equals(PermissionValidTypeEnum.CUSTOM)) {
                    // 自定义类型，需设置开始、结束时间
                    approvalInfoDTO.setStartTime(approvalInfoUpdateReqVO.getStartTime());
                    approvalInfoDTO.setEndTime(approveBaseBusiness.parseEndTime(approvalInfoUpdateReqVO.getEndTime()));
                } else {
                    // 长期类型，开始、结束时间设置为空
                    approvalInfoDTO.setStartTime(null);
                    approvalInfoDTO.setEndTime(null);
                }
                approvalInfoDTO.setUpdateTime(new Date());
                approvalInfoDTO.setUpdateUser(ProofsUtil.getIdcard());
                String flowType = approvalInfoDTO.getFlowType();
                FlowTypeEnum flowTypeEnum = EnumUtils.enumOf(FlowTypeEnum.class, flowType);
                @Valid @NotNull ResourceInfoDTO resourceInfoRespVO = approvalInfoDTO.getResourceInfoVo();
                ContentRespDTO contentRespDTO = approveBusiness.generalContentRespVO(flowTypeEnum, resourceInfoRespVO);
                approvalInfoDTO.setApplyContent(JSON.toJSONString(contentRespDTO));
                // 构建模式3新表单表对象
                approveRecordInfo = approveRecordService.buildApproveRecordInfo(approvalInfoDTO, approvalInfoDTO.getApproveNo());
                approveRecordInfo.setAlreadyRedListApprove(BooleanEnum.FALSE.value);
                // 修改模式3新表单表
                approveRecordService.updateById(approveRecordInfo);

                // 判断实例id是否存在，存在的话，原本为非草稿状态
                // 原本为非草稿状态，点击保存并提交，需要
                @NotBlank String operateType = approvalInfoUpdateReqVO.getOperateType();
                CommitTypeEnum commitTypeEnum = EnumUtils.enumOf(CommitTypeEnum.class, operateType);
                if (CommitTypeEnum.APPLY_COMMIT.equals(commitTypeEnum)) {
                    // 根据实例id获取流程当前待办数组
                    List<TaskHistoryDetailVo> list = workflowClientFactory.getWorkflowProcessClient()
                            .queryTaskDetailByProcessInstanceId(approvalInfoDTO.getProcessInstanceId());
                    TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();
                    String applicantIdcard = approvalInfoUpdateReqVO.getApplicantIdcard();
                    TaskRequestUserInfoDTO taskRequestUserInfoDTO = new TaskRequestUserInfoDTO();
                    taskRequestUserInfoDTO.setIdCard(applicantIdcard);
                    taskRequestUserInfoDTO.setUserName(approvalInfoUpdateReqVO.getApplicantName());
                    taskRequestUserInfoDTO.setOrgCode(approvalInfoUpdateReqVO.getApplicantOrgCode());
                    taskRequestUserInfoDTO.setOrgName(approvalInfoUpdateReqVO.getApplicantOrgName());

                    OperateRespVO operateRespVO = dcucAuthBusiness.checkHitRedList(applicantIdcard, flowTypeEnum, resourceInfoRespVO);

                    // 构建工作流请求体
                    this.buildTaskRequest(taskRequest, approvalInfoDTO, taskRequestUserInfoDTO, approvalInfoUpdateReqVO.getActivitiHolderList(), null, operateRespVO.getHitRedList());

                    for (TaskHistoryDetailVo taskDetailVo : list) {
                        String taskDetailVoId = taskDetailVo.getId();
                        BaseOperateReqDTO baseOperateReqDTO = BaseOperateReqDTO.builder()
                                .operateIdcard(ProofsUtil.getIdcard())
                                .taskId(taskDetailVoId)
                                .operationEnum(ApproveWorkFlowOperationEnum.COMPLETE_AGREE)
                                .taskRequest(taskRequest)
                                .approvalInfoDTO(approvalInfoDTO)
                                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                                .approveNo(approveRecordInfo.getApproveNo())
                                .processNo(approvalInfoDTO.getBusinessCode())
                                .approveRecordId(approvalInfoDTO.getId())
                                .hitRedList(operateRespVO.getHitRedList())
                                .build();

                        // 循环提交当前待办任务
                        processInstanceVO = baseApproveOperate(baseOperateReqDTO);
                        List<TaskHistoryDetailVo> taskRunDetailVos = processInstanceVO.getTaskRunDetailVos();
                        if (CollectionUtils.isNotEmpty(taskRunDetailVos)) {
                            // 若存在待办任务，说明流程未结束，无需发送消息给权限中心
                            sendMessageFlag = false;
                        }
                        logger.info("工作流申请流程：【{}】,taskId:【{}】", JSON.toJSONString(processInstanceVO), taskDetailVoId);
                        // 构建任务id、任务标识码、任务名称字段，多个均以,相隔
                        approveBusiness.buildTaskInfoByProcessInstanceId(approvalInfoDTO, taskRunDetailVos);

                        // 生成待办&已办
                        logger.info("=====节点提交后开始生成待办和已办=====");

                        approveBusiness.afterTaskCommitTask(taskDetailVoId, ApproveWorkFlowOperationEnum.COMPLETE_AGREE, taskRequestUserInfoDTO.getIdCard(), taskRequestUserInfoDTO.getUserName(), taskRunDetailVos);
                    }
                    logger.info("====发送结果标志:{}", sendMessageFlag);
                    if (sendMessageFlag) {
                        // 构建消息体并发送到kafka
                        ApproveMessageInfoDTO approveMessageInfoDTO = ApproveMessageInfoDTO.newInstance(approvalInfoDTO, ApproveWorkFlowOperationEnum.COMPLETE_AGREE, "");

                        approveEventPublish.publishApproveFinalResultEvent(approveMessageInfoDTO);
                        approveRecordInfo.setFinalApprovalResult(ApproveResultEnum.SUCCESS.getValue());
                    }
                    logger.info("===保存表单信息====");
                    // 构建任务id、任务标识码、任务名称字段，多个均以,相隔
                    buildUpdateInfo(approveRecordInfo, approvalInfoDTO);
                    // 修改
                    approveRecordService.updateById(approveRecordInfo);
                    needSaveOperateLog = true;
                }
            }
        } catch (Exception e) {
            approveResultEnum = ApproveResultEnum.FAIL;
            failMsg = e.getMessage();
            throw e;
        } finally {
            if (needSaveUpdateLog) {
                LogDto logDto = approveLogBusiness.buildLogDto(approvalInfoDTO, processInstanceVO, approveResultEnum, failMsg, false, null);
                approveLogBusiness.buildFlowManageLogUpdate(logDto, needSaveOperateLog, sendMessageFlag);
            }
        }

        String processInstanceId = approveRecordInfo.getProcessInstanceId();
        return ResponseUtil.newInstance(processInstanceId);
    }

    /**
     * 构建工作流请求体
     *
     * @param taskRequest            任务请求
     * @param approvalInfoDTO        审批信息
     * @param taskRequestUserInfoDTO 用户数据
     * @param activitiHolderList     任务信息
     * @param taskId                 任务id，新增时需要手动传（因为新增的时候没有任务id可以查询）
     * @param needRedListApprove     是否需要红名单审批
     * @return 请求
     */
    private TaskRequest<ApproveTaskRequestVo> buildTaskRequest(TaskRequest<ApproveTaskRequestVo> taskRequest, ApprovalInfoDTO approvalInfoDTO,
                                                               TaskRequestUserInfoDTO taskRequestUserInfoDTO, List<ActivitiHolder> activitiHolderList, String taskId,
                                                               boolean needRedListApprove) {
        if (null == approvalInfoDTO) {
            throw new IllegalArgumentException("approvalInfo is null");
        }
        String idCard;
        if (StringUtils.isNotEmpty(approvalInfoDTO.getApplicantIdcard())) {
            idCard = approvalInfoDTO.getApplicantIdcard();
        } else {
            idCard = ProofsUtil.getIdcard();
        }
        Boolean kxPeople = approveBusiness.isKxPeople(idCard);
        logger.info("====是否为科信人员判断结果:{}", kxPeople);
        logger.info("====是否需要红名单审批判断结果:{}", needRedListApprove);

        if (taskRequest == null) {
            taskRequest = new TaskRequest<>();
        }
        // 设置自定义对象
        ApproveTaskRequestVo approveTaskRequestVo = new ApproveTaskRequestVo();
        //，指定是否为科信人员
        approveTaskRequestVo.setKxPeople(kxPeople);
        // 判断是否需要红名单审批
        approveTaskRequestVo.setRedList(needRedListApprove);
        // 设置业务表单信息
        approveTaskRequestVo.setBusinessKey(approvalInfoDTO.getId());
        approveTaskRequestVo.setApplyTitle(approvalInfoDTO.getProcessTitle());
        approveTaskRequestVo.setApplyContent(approvalInfoDTO.getApplyContent());
        String appCode = approveProperties.getAppCode();
        approveTaskRequestVo.setBusinessSystem(appCode);
        approveTaskRequestVo.setBusinessType(approvalInfoDTO.getBusinessCode());
        BaseSecurityUser userInfo = UserContextUtils.getCurrentUser();
        approveTaskRequestVo.setHandleUser(userInfo == null ? taskRequestUserInfoDTO.getIdCard() : userInfo.getIdcard());
        approveTaskRequestVo.setHandleUserName(userInfo == null ? taskRequestUserInfoDTO.getUserName() : userInfo.getUserName());
        taskRequest.setData(approveTaskRequestVo);
        // 判断下一任务处理人是否为空
        if (CollectionUtils.isNotEmpty(activitiHolderList)) {
            TransMessage transMessage = new TransMessage();
            if (StringUtils.isNotEmpty(taskId)) {
                // 通过任务id获取任务分支
                List<TaskOutgoingVo> list = workFlowBusiness.buildTaskOutgoingVoList(taskId);
                if (CollectionUtils.isNotEmpty(list)) {
                    for (ActivitiHolder activitiHolder : activitiHolderList) {
                        activitiHolder.setTaskDefinitionKey(list.get(0).getTaskDefinitionKey());
                    }
                }

            }
            transMessage.setActivitiHolders(activitiHolderList);
            taskRequest.setTransMessage(transMessage);
        }
        return taskRequest;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseDTO<OperateRespVO> approveOperate(OperateReqDTO operateReqDTO) {

        String taskId = operateReqDTO.getTaskId();
        String processInstanceId = operateReqDTO.getProcessInstanceId();
        ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum = operateReqDTO.getApproveWorkFlowOperationEnum();
        TaskRequest<ApproveTaskRequestVo> taskRequest = operateReqDTO.getTaskRequest();
        TaskRequestUserInfoDTO taskRequestUserInfoDTO = operateReqDTO.getTaskRequestUserInfoDTO();
        List<ActivitiHolder> activitiHolderList = operateReqDTO.getActivitiHolderList();
        Boolean needCheckRedList = operateReqDTO.getNeedCheckRedList();

        ApprovalInfoDTO approvalInfoDTO = null;
        ProcessInstanceVO processInstanceVO = null;
        ApproveResultEnum approveResultEnum = ApproveResultEnum.SUCCESS;
        String failMsg = "";
        // 是否发送消息到kafka，默认发
        boolean sendMessageFlag = true;

        boolean needSendRedListRisk = false;
        OperateRespVO operateRespVO = null;
        try {
            // 根据实例id或任务id获取申请表单并构建成旧ApprovalInfo对象
            ApproveRecordInfo approveRecordInfo = approveRecordService.findByProcessInstanceId(processInstanceId);  // 获取表单数据
            approvalInfoDTO = approveRecordInfo.toApprovalInfo();       // 构建为旧表单数据 建成旧ApprovalInfo对象
            if (approvalInfoDTO == null) {
                approveResultEnum = ApproveResultEnum.FAIL;
                failMsg = "审批单不存在";
                return ResponseUtil.dtoFail("审批单不存在");
            }

            // 更新审批操作设置的资源信息
            ResourceInfoDTO resourceInfoVo = updateResourceInfo(operateReqDTO, approvalInfoDTO.getResourceInfoVo());
            approvalInfoDTO.setResourceInfoVo(resourceInfoVo);

            String flowType = approvalInfoDTO.getFlowType();    // 权限申请类型   重要？      应用功能授权  数据授权  任务等
            FlowTypeEnum flowTypeEnum = EnumUtils.enumOf(FlowTypeEnum.class, flowType);
            ContentRespDTO contentRespDTO = approveBusiness.generalContentRespVO(flowTypeEnum, resourceInfoVo);// 响应的界面显示信息
            approvalInfoDTO.setApplyContent(JSON.toJSONString(contentRespDTO));      // 申请内容记录

            String alreadyRedListApprove = approvalInfoDTO.getAlreadyRedListApprove();
            TaskHistoryDetailVo taskHistoryDetailVo = workflowClientFactory.getWorkflowTaskClient().queryTaskDetail(taskId);    // 历史审批信息
            String taskHistoryDetailVoName = taskHistoryDetailVo.getName();
            String taskCode = taskHistoryDetailVo.getTaskCode();

            String approvalInfoTaskId = approvalInfoDTO.getTaskId();
            String approvalInfoId = approvalInfoDTO.getId();
            String approveNo = approvalInfoDTO.getApproveNo();
            String businessCode = approvalInfoDTO.getBusinessCode();    // 流程编号
            String applicantIdcard = approvalInfoDTO.getApplicantIdcard();

            // 判断操作类型是否支持
            // 如果是  回退  或者 撤回，需要先看下  当前审批流程实例 是否支持
            if (approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.RECALL)
                    || approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.BACK)) {
                boolean enableOperate = isEnableOperateRecallBack(operateReqDTO, approvalInfoTaskId, taskCode);
                if (!enableOperate) {
                    return ResponseUtil.dtoFail("该节点不支持此项操作");
                }
                if (StringUtils.isNotEmpty(approvalInfoTaskId)) {
                    taskId = approvalInfoTaskId;
                }
            }

            // TODO 增加 挂起（Suspend），激活（Activate），终止（Redo）  哪种情况的支持操作
            // TODO add wanghao
            if (approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.SUSPEND)
                    || approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.ACTIVATE)
                    || approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.REDO)) {
                boolean enableOperate = isEnableOperateSuspendActivateRedo(operateReqDTO, approvalInfoTaskId, taskCode);
                if (!enableOperate) {
                    return ResponseUtil.dtoFail("该节点不支持此项操作");
                }
                if (StringUtils.isNotEmpty(approvalInfoTaskId)) {
                    taskId = approvalInfoTaskId;
                }
            }

            operateRespVO = dcucAuthBusiness.checkHitRedList(applicantIdcard, flowTypeEnum, resourceInfoVo);
            String redListTip = operateRespVO.getRedListTip();
            Boolean hitRedList = operateRespVO.getHitRedList();

            // 判断是否红名单审批
            boolean alreadyRedListApproveBoolean = EnumUtils.enumOf(BooleanEnum.class, alreadyRedListApprove).booleanValue;

            // 开始执行操作内容--同意
            if (approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.COMPLETE_AGREE)) {
                // 需要检查红名单且还未进行红名单审批
                if (needCheckRedList && (!alreadyRedListApproveBoolean)) {
                    // 判断是否需要红名单审批，只有在同意的时候进行审批
                    if (hitRedList) {
                        approveResultEnum = ApproveResultEnum.FAIL;
                        failMsg = redListTip;
                        return ResponseUtil.newInstance(operateRespVO);
                    }
                }
                // 1、不需要检查红名单审批，（检查红名单完调用通过的接口，不需要红名单检查）
                // 2、还未进行红名单审批
                // 3、不是红名单这个节点
                String taskName = approvalInfoDTO.getTaskName();
                needSendRedListRisk = (!needCheckRedList) && (!alreadyRedListApproveBoolean) && (!taskName.contains(ApproveConstants.RED_LIST));
            }

            // 构建工作流请求体
            this.buildTaskRequest(taskRequest, approvalInfoDTO, taskRequestUserInfoDTO, activitiHolderList, null, operateRespVO.getHitRedList());
            // 节点构建
            BaseOperateReqDTO baseOperateReqDTO = BaseOperateReqDTO.builder()
                    .operateIdcard(ProofsUtil.getIdcard())
                    .taskId(taskId)
                    .operationEnum(approveWorkFlowOperationEnum)
                    .taskRequest(taskRequest)
                    .approvalInfoDTO(approvalInfoDTO)
                    .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                    .approveNo(approveNo)
                    .processNo(businessCode)
                    .approveRecordId(approvalInfoId)
                    .hitRedList(operateRespVO.getHitRedList())
                    .build();

            // 节点信息提交
            processInstanceVO = baseApproveOperate(baseOperateReqDTO);  // 执行审批操作           //TODO 主要改这里

            List<TaskHistoryDetailVo> afterTaskRunDetailVos = processInstanceVO.getTaskRunDetailVos();

            if (CollectionUtils.isNotEmpty(afterTaskRunDetailVos)) {
                // 若有待办，说明流程未结束，则不发
                sendMessageFlag = false;
            }

            // 更新是否红名单审批状态。
            // 规则，不需要检查红名单、任务节点名字包含红名单、审批结果是通过
            if ((!needCheckRedList) && taskHistoryDetailVoName.contains(ApproveConstants.RED_LIST) &&
                    approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.COMPLETE_AGREE)) {
                approvalInfoDTO.setAlreadyRedListApprove(BooleanEnum.TRUE.value);
            }

            // 生成待办&已办
            logger.info("=====节点提交后开始生成待办和已办=====");

            // 删除代办---新增已办，  用的两张表记录的，不是状态位？
            approveBusiness.afterTaskCommitTask(taskId, approveWorkFlowOperationEnum, taskRequestUserInfoDTO.getIdCard(), taskRequestUserInfoDTO.getUserName(), afterTaskRunDetailVos);

            logger.info("====是否发送消息标志位:{}====", sendMessageFlag);
            if (sendMessageFlag) {      // 往kafka 推送消息  给下一步审批人？
                ApproveMessage approveMessage = taskRequest.getApproveMessage();
                String message = approveMessage.getMessage();
                // 构建消息体并发送
                sendMessage(approveWorkFlowOperationEnum, approvalInfoDTO, message);
                // 设置流程状态结果
                switch (approveWorkFlowOperationEnum) {
                    case COMPLETE_AGREE:
                        approvalInfoDTO.setApproveResult(ApproveResultEnum.SUCCESS.getValue());
                        break;
                    case COMPLETE_DISAGREE:
                    case REDO:
                        approvalInfoDTO.setApproveResult(ApproveResultEnum.FAIL.getValue());
                        break;
                    default:
                        break;
                }
                approveRecordInfo.setFinalApprovalResult(approvalInfoDTO.getApproveResult());
            }

            // 更新任务信息
            // 构建任务id、任务标识码、任务名称字段，多个均以,相隔
            approveBusiness.buildTaskInfoByProcessInstanceId(approvalInfoDTO, afterTaskRunDetailVos);
            logger.info("====当前任务详情:{}====", JSONObject.toJSON(afterTaskRunDetailVos));
            logger.info("====开始更新审批单信息====");
            // 设置表单信息
            buildUpdateInfo(approveRecordInfo, approvalInfoDTO);
            // 修改表单
            approveRecordService.updateById(approveRecordInfo);
        } catch (Exception e) {
            logger.error("operate error", e);
            approveResultEnum = ApproveResultEnum.FAIL;
            failMsg = e.getMessage();
            return ResponseUtil.dtoFail(failMsg);
        } finally {
            // 有审批信息,有进行审批操作才记录日志
            if ((approvalInfoDTO != null) && (processInstanceVO != null)) {
                LogDto logDto = approveLogBusiness.buildLogDto(approvalInfoDTO, processInstanceVO,
                        approveResultEnum, failMsg, needSendRedListRisk, operateRespVO);
                approveLogBusiness.buildOperateLog(logDto, approveWorkFlowOperationEnum, sendMessageFlag);
            }
        }

        operateRespVO = OperateRespVO.notRequired();
        return ResponseUtil.newInstance(operateRespVO);
    }

    /**
     * 设置字段值
     *
     * @param approveRecordInfo 审批记录信息
     * @param approvalInfoDTO   审批信息
     */
    private void buildUpdateInfo(ApproveRecordInfo approveRecordInfo, ApprovalInfoDTO approvalInfoDTO) {
        approveRecordInfo.setTaskName(approvalInfoDTO.getTaskName());
        approveRecordInfo.setTaskId(approvalInfoDTO.getTaskId());
        approveRecordInfo.setTaskCode(approvalInfoDTO.getApproveTaskIdentifier());
        approveRecordInfo.setAlreadyRedListApprove(approvalInfoDTO.getAlreadyRedListApprove());
        approveRecordInfo.setUpdateTime(new Date());
        approveRecordInfo.setUpdateUser(ProofsUtil.getIdcard());

        ResourceInfoDTO resourceInfoVo = approvalInfoDTO.getResourceInfoVo();
        approveBusiness.setResourceInfoData(approveRecordInfo, resourceInfoVo);

    }

    /**
     * 基础提交
     *
     * @param baseOperateReqDTO 基础提交请求
     * @return 流程信息
     */
    public ProcessInstanceVO baseApproveOperate(BaseOperateReqDTO baseOperateReqDTO) {
        Assert.notNull(baseOperateReqDTO);

        String operateIdcard = baseOperateReqDTO.getOperateIdcard();
        String taskId = baseOperateReqDTO.getTaskId();
        ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum = baseOperateReqDTO.getOperationEnum();
        TaskRequest<ApproveTaskRequestVo> taskRequest = baseOperateReqDTO.getTaskRequest();
        ApprovalInfoDTO approvalInfoDTO = baseOperateReqDTO.getApprovalInfoDTO();
        TaskRequestUserInfoDTO taskRequestUserInfoDTO = baseOperateReqDTO.getTaskRequestUserInfoDTO();
        String approveNo = baseOperateReqDTO.getApproveNo();
        String processNo = baseOperateReqDTO.getProcessNo();
        String approveRecordId = baseOperateReqDTO.getApproveRecordId();
        boolean hitRedList = baseOperateReqDTO.getHitRedList();

        taskRequest = buildTaskRequest(taskRequest, approvalInfoDTO, taskRequestUserInfoDTO, null, null, hitRedList);
        // 判断是否有审批信息，若无，默认设置
        if (null == taskRequest.getApproveMessage()) {
            ApproveMessage approveMessage = new ApproveMessage();
            approveMessage.setApproveState(approveWorkFlowOperationEnum.getValue());
            approveMessage.setMessage(approveWorkFlowOperationEnum.getLabel());
            taskRequest.setApproveMessage(approveMessage);
        }

        ProcessInstanceVO vo = null;
        logger.info("=====approveInfo->applicantIdCard:{},传入的idCard是:{}=====", approvalInfoDTO.getApplicantIdcard(), operateIdcard);

        logger.info("=====approveInfo->workflowClientFactory taskRequest:{}", JsonUtils.toJSONString(taskRequest));

        switch (approveWorkFlowOperationEnum) {     // TODO 增加调用  挂起等判断
            case COMPLETE_AGREE:
                vo = workflowClientFactory.getWorkflowTaskClient().commitTask(taskId, taskRequest, WorkFlowOprationEnum.COMPLETE_AGREE, operateIdcard);
                // 保存任务指派人记录信息
                insertTaskAssignRecord(vo, approveWorkFlowOperationEnum, operateIdcard, taskRequest, approveRecordId);
                break;
            case COMPLETE_DISAGREE:
                vo = workflowClientFactory.getWorkflowTaskClient().commitTask(taskId, taskRequest, WorkFlowOprationEnum.COMPLETE_DISAGREE, operateIdcard);
                // 保存任务指派人记录信息
                insertTaskAssignRecord(vo, approveWorkFlowOperationEnum, operateIdcard, taskRequest, approveRecordId);
                break;
            case BACK:
                vo = workflowClientFactory.getWorkflowTaskClient().backTask(taskId, taskRequest, operateIdcard);
                break;
            case RECALL:
                vo = workflowClientFactory.getWorkflowTaskClient().recallTask(taskId, taskRequest, operateIdcard);
                break;

            // TODO add wanghao
            case SUSPEND:
                vo = workflowClientFactory.getWorkflowTaskClient().handleTaskForSuspend(taskId, operateIdcard);
                break;
            case ACTIVATE:
                vo = workflowClientFactory.getWorkflowTaskClient().handleTaskForActivate(taskId, operateIdcard);
                break;
            case REDO:
                // vo = workflowClientFactory.getWorkflowTaskClient().handleTaskForRedo(taskId, operateIdcard);
                TaskHistoryDetailVo historyVo = workflowClientFactory.getWorkflowTaskClient().queryTaskDetail(taskId);
                vo = new ProcessInstanceVO();
                TaskHistoryDetailVo historyDetailVo = new TaskHistoryDetailVo();
                historyDetailVo.setName(historyVo.getName());
                historyDetailVo.setTaskCode(END_STR);
                historyDetailVo.setStartTime(historyVo.getStartTime());
                historyDetailVo.setEndTime(new Date());
                historyDetailVo.setTaskDefinitionKey(historyVo.getTaskDefinitionKey());
                vo.setTaskHistoryDetailVo(historyDetailVo);
                vo.setPocessDefinitionId(historyVo.getProcessDefinitionId());
                vo.setProcessInstanceId(historyVo.getProcessInstanceId());
                break;
            default:
                break;
        }
        // 构建审批记录表
        System.err.println(JSONObject.toJSONString(vo));
        buildApproveCirculationInfoAndSave(approveRecordId, vo, approveWorkFlowOperationEnum, taskRequestUserInfoDTO, taskRequest.getApproveMessage().getMessage(), approveNo, processNo);
        return vo;
    }

    /**
     * 审批记录
     *
     * @param approveRecordId        审批表单id
     * @param vo                     工作流返回vo
     * @param operateType            操作类型,详见dm_operate_type码表
     * @param taskRequestUserInfoDTO 当前登录相关信息
     * @param msg                    审批意见
     * @param approveNo              审批编号
     * @param processNo              流程编号
     */
    private void buildApproveCirculationInfoAndSave(
            String approveRecordId, ProcessInstanceVO vo, ApproveWorkFlowOperationEnum operateType,
            TaskRequestUserInfoDTO taskRequestUserInfoDTO, String msg, String approveNo, String processNo) {

        ApproveCirculationInfo approveCirculationInfo = new ApproveCirculationInfo();
        approveCirculationInfo.setApproveRecordId(approveRecordId);
        approveCirculationInfo.setApproveNo(approveNo);
        approveCirculationInfo.setProcessNo(processNo);
        approveCirculationInfo.setOperateType(operateType.getValue());
        approveCirculationInfo.setOperateUserName(taskRequestUserInfoDTO.getUserName());
        approveCirculationInfo.setOperateUserId(taskRequestUserInfoDTO.getIdCard());
        approveCirculationInfo.setOperateUserOrg(taskRequestUserInfoDTO.getOrgName());
        approveCirculationInfo.setOperateUserOrgId(taskRequestUserInfoDTO.getOrgCode());
        approveCirculationInfo.setApproveOpinion(msg);

        // TODO 电子签名内容待定，先设置个null
        approveCirculationInfo.setElectronicSignature(null);
        approveCirculationInfo.setIsDeleted(BooleanEnum.FALSE.getValue());
        approveCirculationInfo.setCreateUser(taskRequestUserInfoDTO.getIdCard());
        approveCirculationInfo.setCreateTime(new Date());

        approveCirculationInfo.setNodeName(vo.getTaskHistoryDetailVo().getName());
        approveCirculationInfo.setNodeNo(vo.getTaskHistoryDetailVo().getTaskCode());
        approveCirculationInfo.setTaskKey(vo.getTaskHistoryDetailVo().getTaskDefinitionKey());
        approveCirculationInfo.setOperateTime(DateUtils.getTimeStr(vo.getTaskHistoryDetailVo().getEndTime(), DateConst.HYPHEN_DISPLAY_TIME));
        // TODO add wanghao  解决 挂起 无endtime 问题
        Date tmpDate = new Date();
        if (vo.getTaskHistoryDetailVo().getEndTime() != null) {
            tmpDate = vo.getTaskHistoryDetailVo().getEndTime();
        }
        approveCirculationInfo.setOperateTimeConsuming(String.valueOf(DateUtil.between(tmpDate, vo.getTaskHistoryDetailVo().getStartTime(), DateUnit.SECOND)));
        approveCirculationInfoService.save(approveCirculationInfo);
    }

    /**
     * 保存任务指派人记录信息
     *
     * @param vo              实例信息
     * @param operationEnum   操作信息
     * @param idCard          身份证号
     * @param taskRequest     任务请求信息
     * @param approveRecordId 审批记录ID
     */
    private void insertTaskAssignRecord(ProcessInstanceVO vo, ApproveWorkFlowOperationEnum operationEnum, String idCard,
                                        TaskRequest<ApproveTaskRequestVo> taskRequest, String approveRecordId) {
        if (taskRequest.getTransMessage() != null && CollectionUtils.isNotEmpty(vo.getTaskRunDetailVos())) {
            for (TaskHistoryDetailVo taskHistoryDetailVo : vo.getTaskRunDetailVos()) {
                TaskAssignRecord assignRecord = new TaskAssignRecord();
                assignRecord.setAssignUser(taskHistoryDetailVo.getAssignee());
                assignRecord.setNextActivityId(taskHistoryDetailVo.getTaskDefinitionKey());
                assignRecord.setNextActivityName(taskHistoryDetailVo.getName());
                assignRecord.setCurrentActivityId(vo.getTaskHistoryDetailVo().getTaskDefinitionKey());
                assignRecord.setCurrentActivityName(vo.getTaskHistoryDetailVo().getName());
                assignRecord.setOperateType(operationEnum.getValue());
                assignRecord.setCreateTime(new Date());
                assignRecord.setCreateUser(idCard);
                assignRecord.setApproveRecordId(approveRecordId);
                taskAssignRecordService.insertTaskAssignRecord(assignRecord);
            }
        }

    }

    @Override
    public void agent(AgentReqVO agentReqVO) {
        WorkflowRequestData workflowRequestData = new WorkflowRequestData();
        ApproveMessage approveMessage = new ApproveMessage();
        workflowRequestData.setApproveMessage(approveMessage);
        // 转办接口
        workflowClientFactory.getWorkflowTaskClient().taskOperationAssign(agentReqVO.getTaskId(), agentReqVO.getAssigneeUser(),
                ProofsUtil.getIdcard(), workflowRequestData);
    }

    @Override
    public ResponseDTO<List<ApproveCommentVo>> approveComments(String processInstanceId) {
        if (StrUtil.isEmptyOrUndefined(processInstanceId)) {
            return ResponseUtil.newInstance(Collections.emptyList());
        }

        // step1 审批历史记录
        List<ApproveCirculationInfo> hisTaskMapNew = this.getProcessPathNew(processInstanceId);

        return approveCirculationInfoService.buildApproveCommentVo(hisTaskMapNew);
    }

    @Override
    public Integer queryApproveQty(Searchable searchable) {
        // 获取已办列表
        Page<TaskVo> page = queryApproveList(searchable);
        return (int) page.getTotalElements();
    }

    @Override
    public Integer queryMyRequestQty() {
        // 获取表单列表
        String currentUserId = ProofsUtil.getIdcard();
        return approveRecordService.getUserRequestCount(currentUserId);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseStatus deleteProcessInstance(String id) {
        ApprovalInfoDTO approvalInfoDTO = null;
        ApproveResultEnum approveResultEnum = ApproveResultEnum.SUCCESS;
        String failMsg = "";
        try {
            // 获取表单信息
            approvalInfoDTO = getApproveRecord(id);
            if (approvalInfoDTO == null) {
                approveResultEnum = ApproveResultEnum.FAIL;
                failMsg = "申请单不存在";
                return ResponseStatus.fail("申请单不存在");
            }

            if (StringUtils.isNotEmpty(approvalInfoDTO.getTaskId())) {
                if (approvalInfoDTO.getTaskId().equalsIgnoreCase(END_STR)) {
                    approveResultEnum = ApproveResultEnum.FAIL;
                    failMsg = "流程已结束，不允许删除";
                    return ResponseDTO.fail("流程已结束，不允许删除");
                }
            }

            boolean callBackEnableFlag = false;
            // 判断是否是草稿状态
            if (StringUtils.isNotEmpty(approvalInfoDTO.getProcessInstanceId())) {
                // 1. 获取当前结点可以操作的类型
                // 2. 判断是否可以执行撤销操作，若可以执行撤销，否则返回错误信息
                // 3. 将流程实例信息(审批端)的删除标志置1

                String taskIds = approvalInfoDTO.getTaskId();
                String taskCodes = approvalInfoDTO.getApproveTaskIdentifier();
                String[] arrTaskId = taskIds.split(",");
                String[] arrTaskCode = taskCodes.split(",");
                int index = 0;
                for (String taskId : arrTaskId) {
                    logger.info("====检验当前结点是否支持撤销操作====");
                    callBackEnableFlag = workFlowBusiness.checkOperateOptionOfCurrentStep(taskId, approvalInfoDTO.getProcessInstanceId(), arrTaskCode[index++], OperateOptionEnum._BTN_REDO_COMMIT);
                    if (callBackEnableFlag) {
                        break;
                    }
                }
                // 判断并执行撤销操作
                if (callBackEnableFlag) {
                    for (String taskId : arrTaskId) {
                        // 执行流程撤销操作，撤销会删除流程
                        workflowClientFactory.getWorkflowTaskClient().handleTaskForRedo(taskId, ProofsUtil.getIdcard());
                    }
                } else {
                    approveResultEnum = ApproveResultEnum.FAIL;
                    failMsg = "当前节点不允许删除操作";
                    return ResponseDTO.fail("当前节点不允许删除操作");
                }
            }
            // 执行表单表逻辑删除
            approveRecordService.deleteById(id);
            ApproveMessageInfoDTO approveMessageInfoDTO = ApproveMessageInfoDTO.newInstance(approvalInfoDTO, ApproveWorkFlowOperationEnum.RECALL, "");
            approveEventPublish.publishApproveFinalResultEvent(approveMessageInfoDTO);

            // 删除所有已办
            approveActHiTaskService.deleteByApplyRecordId(id);

            // 删除所有待办
            approveActRunTaskService.deleteByApplyRecordId(id);
        } catch (Exception e) {
            approveResultEnum = ApproveResultEnum.FAIL;
            failMsg = "";
        } finally {
            LogDto logDto = approveLogBusiness.buildLogDto(approvalInfoDTO, null, approveResultEnum,
                    failMsg, false, null);
            approveLogBusiness.buildFlowManageLogDelete(logDto);
        }

        return ResponseStatus.success();
    }

    @Override
    public ResponseDTO<ApprovalInfoRespVO> queryApproveDetail(String id) {
        // 获取表单信息
        ApprovalInfoDTO approvalInfoDTO = getApproveRecord(id);
        if (approvalInfoDTO == null) {
            return ResponseUtil.dtoFail("申请表单不存在");
        }
        ApprovalInfoRespVO approvalInfoRespVO = approveBusiness.fillTaskInfoToApplyInfo(approvalInfoDTO);
        return ResponseUtil.newInstance(approvalInfoRespVO);
    }

    @Override
    public ResponseDTO<List<ProcessStatusVo>> approveStatus(String processInstanceId) {
        // 获取表单信息
        ApproveRecordInfo approveRecordInfo = approveRecordService.findByProcessInstanceId(processInstanceId);
        ApprovalInfoDTO approvalInfoDTO = approveRecordInfo.toApprovalInfo();
        if (approvalInfoDTO == null) {
            return ResponseUtil.dtoFail("无法获取审批单信息");
        }
        String approvalInfoId = approvalInfoDTO.getId();
        return approveStatusByBusinessId(approvalInfoId);
    }

    @Override
    public ResponseDTO<List<ProcessStatusVo>> approveStatusByBusinessId(String approveRecordId) {
        // 通过审批记录
        List<ApproveCirculationInfo> processPathNew = approveCirculationInfoService.getListByApproveRecordId(approveRecordId);
        // 根据任务定义key去重
        List<ApproveCirculationInfo> processPath = processPathNew.stream()
                .collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
                        new TreeSet<>(Comparator.comparing(ApproveCirculationInfo::getTaskKey))), ArrayList::new));
        // 根据创建时间排序
        List<ApproveCirculationInfo> sortList = processPath.stream()
                .sorted(Comparator.comparing(ApproveCirculationInfo::getCreateTime))
                .collect(Collectors.toList());
        List<ProcessStatusVo> list = new ArrayList<>();
        for (int i = 0; i < sortList.size(); i++) {
            ApproveCirculationInfo approveCirculationInfo = sortList.get(i);

            ProcessStatusVo processStatusVo = new ProcessStatusVo();
            processStatusVo.setNodeName(approveCirculationInfo.getNodeName());
            String taskKey = approveCirculationInfo.getTaskKey();

            // 通过审批记录（分组）
            List<ApproveCirculationInfo> childList = approveCirculationInfoService.getListByApproveRecordIdAndTaskKey(approveRecordId, taskKey);
            Set<ProcessStatusDetailVo> taskVos = new HashSet<>();
            if (CollectionUtils.isNotEmpty(childList)) {
                for (ApproveCirculationInfo circulationInfo : childList) {
                    ProcessStatusDetailVo detailVo = new ProcessStatusDetailVo();
                    detailVo.setAssignee(circulationInfo.getOperateUserId());
                    detailVo.setAssigneeName(circulationInfo.getOperateUserName());
                    detailVo.setOperateState(EnumUtils.enumOf(ApproveWorkFlowOperationEnum.class, circulationInfo.getOperateType()).getLabel());
                    long between = Long.parseLong(circulationInfo.getOperateTimeConsuming());
                    long hours = between / 3600;
                    long minute = (between - hours * 3600) / 60;
                    long second = between % 60;
                    detailVo.setCostTime(String.format("%d时%d分%d秒", hours, minute, second));
                    detailVo.setReceiveTime(DateUtils.addDate(circulationInfo.getCreateTime(), Calendar.SECOND, -Integer.parseInt(circulationInfo.getOperateTimeConsuming())));
                    detailVo.setHandleTime(circulationInfo.getCreateTime());
                    taskVos.add(detailVo);
                }
            }
            processStatusVo.setTaskVos(taskVos.stream().sorted(Comparator.comparing(ProcessStatusDetailVo::getHandleTime)).collect(Collectors.toList()));
            processStatusVo.setFinishOperatorSummary(childList.size());
            processStatusVo.setOperatorSummary(childList.size());
            processStatusVo.setSort(i);
            list.add(processStatusVo);
        }
        List<ProcessStatusVo> collect = list.stream()
                .sorted(Comparator.comparing(ProcessStatusVo::getSort))
                .collect(Collectors.toList());
        // 获取待办任务
        List<ProcessStatusVo> actRunProcessStatusVo = approveActRunTaskService.buildProcessStatusVo(approveRecordId);
        collect.addAll(actRunProcessStatusVo);

        return ResponseUtil.newInstance(collect);
    }

    /**
     * 根据流程实例完成的节点
     *
     * @param processInstanceId 流程实例ID
     * @return 审批意见
     */
    private List<ApproveCirculationInfo> getProcessPathNew(String processInstanceId) {
        // 获取表单信息
        ApproveRecordInfo approveRecordInfo = approveRecordService.findByProcessInstanceId(processInstanceId);
        if (approveRecordInfo == null) {
            throw new IllegalArgumentException(String.format("processInstanceId:%s can't find ApprovalInfo", processInstanceId));
        }
        String approvalInfoId = approveRecordInfo.getId();
        // 通过表单id查询审批记录
        return approveCirculationInfoService.getListByApproveRecordId(approvalInfoId);
    }

    /**
     * 构建任务请求VO
     *
     * @param approvalInfoDTO 审批信息
     * @return 任务请求
     */
    private ApproveTaskRequestVo buildTaskRequestVo(ApprovalInfoDTO approvalInfoDTO) {
        String appCode = approveProperties.getAppCode();
        ApproveTaskRequestVo taskRequestVo = new ApproveTaskRequestVo();
        taskRequestVo.setApplyTitle(approvalInfoDTO.getProcessTitle());
        taskRequestVo.setApplyContent(approvalInfoDTO.getApplyContent());
        taskRequestVo.setBusinessId(approvalInfoDTO.getId());
        taskRequestVo.setBusinessKey(approvalInfoDTO.getId());
        taskRequestVo.setBusinessSystem(appCode);
        taskRequestVo.setBusinessType(approvalInfoDTO.getBusinessCode());
        taskRequestVo.setApplyUser(approvalInfoDTO.getApplicantIdcard());
        taskRequestVo.setApplyUserName(approvalInfoDTO.getApplicantName());
        taskRequestVo.setApplicantOrgCode(approvalInfoDTO.getApplicantOrgCode());
        taskRequestVo.setApplicantOrgName(approvalInfoDTO.getApplicantOrgName());
        return taskRequestVo;
    }

    /**
     * 通过表单id查询并构建旧表单对象
     *
     * @param id 审批记录id
     * @return 审批信息
     */
    public ApprovalInfoDTO getApproveRecord(String id) {
        // 表单查询
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectOne(id);
        if (approveRecordInfo != null) {
            return approveRecordInfo.toApprovalInfo();
        }
        return null;
    }

    /**
     * 构建申请表单数据
     *
     * @param approvalInfoReqVO 审批信息
     * @return 审批信息
     */
    private ApprovalInfoDTO buildApprovalInfo(ApprovalInfoReqVO approvalInfoReqVO) {
        ApprovalInfoDTO approvalInfoDTO = new ApprovalInfoDTO();
        BeanUtil.copyProperties(approvalInfoReqVO, approvalInfoDTO);
        approvalInfoDTO.setIsDeleted(BooleanEnum.FALSE.getValue());
        approvalInfoDTO.setCreateUser(StringUtils.isNotEmpty(approvalInfoReqVO.getApplicantIdcard()) ? approvalInfoReqVO.getApplicantIdcard()
                : ProofsUtil.getIdcard());
        approvalInfoDTO.setId(UUIDUtils.getUUID());
        approvalInfoDTO.setCreateTime(new Date());

        @Valid @NotNull ResourceInfoReqVO resourceInfoRespVO = approvalInfoReqVO.getResourceInfoDTO();

        ResourceInfoDTO resourceInfoDTO = ResourceInfoDTO.parseResourceInfoReqVO(resourceInfoRespVO);
        approvalInfoDTO.setResourceInfoVo(resourceInfoDTO);

        approvalInfoDTO.setCallbackUrl(approvalInfoReqVO.getCallbackUrl());
        approvalInfoDTO.setApproveTaskClass(approvalInfoReqVO.getApproveTaskClass());
        approvalInfoDTO.setApplicantIp(approvalInfoReqVO.getApplicantIp());
        approvalInfoDTO.setApplicantAppToken(approvalInfoReqVO.getApplicantAppToken());

        return approvalInfoDTO;
    }

    /**
     * 是否支持撤回、退回操作
     *
     * @param operateReqDTO      请求
     * @param approvalInfoTaskId 当前任务ID
     * @param taskCode           任务代码
     * @return 是否支持
     */
    protected boolean isEnableOperateRecallBack(OperateReqDTO operateReqDTO, String approvalInfoTaskId, String taskCode) {
        ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum = operateReqDTO.getApproveWorkFlowOperationEnum();
        String taskId = operateReqDTO.getTaskId();
        String processInstanceId = operateReqDTO.getProcessInstanceId();

        Assert.notNull(approveWorkFlowOperationEnum);
        Assert.notBlank(taskId);
        Assert.notBlank(processInstanceId);

        boolean isEnable;
        if (approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.RECALL)) {
            // 查询当前任务是否支持撤回操作（工作流节点是否有配置撤回操作）
            isEnable = workFlowBusiness.checkOperateOptionOfCurrentStep(taskId, processInstanceId, taskCode, OperateOptionEnum._BTN_WITH_DRAW_COMMIT);
        } else {
            // 查询当前任务是否支持退回操作（工作流节点是否有配置退回操作）
            isEnable = workFlowBusiness.checkOperateOptionOfCurrentStep(taskId, processInstanceId, taskCode, OperateOptionEnum._BTN_BACK_COMMIT);
        }
        logger.info("=====节点的{}操作权限判断结果为:{}======", approveWorkFlowOperationEnum, isEnable);
        return isEnable && !approvalInfoTaskId.equalsIgnoreCase(END_STR);
    }

    /**
     * 发送消息
     *
     * @param approveWorkFlowOperationEnum 操作类型
     * @param approvalInfoDTO              审批信息
     */
    protected void sendMessage(ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum, ApprovalInfoDTO approvalInfoDTO, String approveMessage) {
        // 构建消息体并发送
        ApproveMessageInfoDTO approveMessageInfoDTO = ApproveMessageInfoDTO.newInstance(approvalInfoDTO, approveWorkFlowOperationEnum, approveMessage);
        approveEventPublish.publishApproveFinalResultEvent(approveMessageInfoDTO);
    }

    /**
     * 更新审批操作设置的资源信息
     *
     * @param operateReqDTO   操作请求
     * @param resourceInfoDTO 资源信息
     * @return 资源信息
     */
    protected ResourceInfoDTO updateResourceInfo(OperateReqDTO operateReqDTO, ResourceInfoDTO resourceInfoDTO) {
        Assert.notNull(operateReqDTO);
        String visitResourceCode = operateReqDTO.getVisitResourceCode();
        String visitResourceName = operateReqDTO.getVisitResourceName();
        // 如果没有设置访问资源则为空
        if (StrUtil.isNotBlank(visitResourceCode)
                && StrUtil.isNotBlank(visitResourceName)) {
            resourceInfoDTO.setVisitResourceCode(visitResourceCode);
            resourceInfoDTO.setVisitResourceName(visitResourceName);
            resourceInfoDTO.setDisagreeVisitResourceCode(operateReqDTO.getDisagreeVisitResourceCode());
            resourceInfoDTO.setDisagreeVisitResourceName(operateReqDTO.getDisagreeVisitResourceName());
        }
        return resourceInfoDTO;
    }

    // TODO 增加 挂起（Suspend），激活（Activate），终止（Redo）  哪种情况的支持操作
    // TODO add wanghao

    /**
     * 是否支持挂起、激活、终止操作
     *
     * @param operateReqDTO      请求
     * @param approvalInfoTaskId 当前任务ID
     * @param taskCode           任务代码
     * @return 是否支持
     */
    protected boolean isEnableOperateSuspendActivateRedo(OperateReqDTO operateReqDTO, String approvalInfoTaskId, String taskCode) {
        ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum = operateReqDTO.getApproveWorkFlowOperationEnum();
        String taskId = operateReqDTO.getTaskId();
        String processInstanceId = operateReqDTO.getProcessInstanceId();

        Assert.notNull(approveWorkFlowOperationEnum);
        Assert.notBlank(taskId);
        Assert.notBlank(processInstanceId);

        boolean isEnable = false;
        if (approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.SUSPEND)) {
            // 查询当前任务是否支持挂起操作（工作流节点是否有配置 挂起 操作） 以及业务逻辑是否支持
            isEnable = workFlowBusiness.checkOperateOptionOfCurrentStep(taskId, processInstanceId, taskCode, OperateOptionEnum._BTN_SUSPEND_COMMIT);
        } else if (approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.ACTIVATE)) {
            // 查询当前任务是否支持挂起操作（工作流节点是否有配置 激活 操作） 以及业务逻辑是否支持
            isEnable = workFlowBusiness.checkOperateOptionOfCurrentStep(taskId, processInstanceId, taskCode, OperateOptionEnum._BTN_RECOVER);
        } else if (approveWorkFlowOperationEnum.equals(ApproveWorkFlowOperationEnum.REDO)) {
            // 查询当前任务是否支持挂起操作（工作流节点是否有配置 终止 操作） 以及业务逻辑是否支持
            isEnable = workFlowBusiness.checkOperateOptionOfCurrentStep(taskId, processInstanceId, taskCode, OperateOptionEnum._BTN_REDO_COMMIT);
        }

        logger.info("=====节点的{}操作权限判断结果为:{}======", approveWorkFlowOperationEnum, isEnable);
        return isEnable && !approvalInfoTaskId.equalsIgnoreCase(END_STR);
    }
}