package com.dragonsoft.dcuc.approve.business.impl;

import cn.hutool.core.util.ObjectUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.business.IApproveTipsBusiness;
import com.dragonsoft.dcuc.approve.business.IRiskBusiness;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.ApproveSecurityWarningInfo;
import com.dragonsoft.dcuc.approve.model.resp.OperateTipResp;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.dcuc.approve.service.IApproveSecurityWarningInfoService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.BaseSecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/18
 */
@Component
public class ApproveTipsBusinessImpl implements IApproveTipsBusiness {

    private IApproveRecordService approveRecordService;

    private IRiskBusiness riskBusiness;

    private IApproveSecurityWarningInfoService approveSecurityWarningInfoService;

    @Autowired
    public void setApproveSecurityWarningInfoService(IApproveSecurityWarningInfoService approveSecurityWarningInfoService) {
        this.approveSecurityWarningInfoService = approveSecurityWarningInfoService;
    }

    @Autowired
    public void setRiskBusiness(IRiskBusiness riskBusiness) {
        this.riskBusiness = riskBusiness;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Override
    public ResponseDTO<List<OperateTipResp>> operateTips(String processInstanceId) {
        ApproveRecordInfo approveRecordInfo = approveRecordService.findByProcessInstanceId(processInstanceId);
        if (ObjectUtil.isNull(approveRecordInfo)) {
            return ResponseUtil.dtoFail("审批单不存在");
        }
        String approveRecordInfoId = approveRecordInfo.getId();
        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();

        String applicantIdcard = approveRecordInfo.getApplicantIdcard();
        String idCard = currentUser.getIdcard();
        String userName = currentUser.getName();

        List<OperateTipResp> riskTipList = getRiskTipList(approveRecordInfoId, applicantIdcard, idCard, userName);
        List<OperateTipResp> operateTipRespList = new ArrayList<>(riskTipList);

        List<OperateTipResp> securityWarningTipList = getSecurityWarningTipList(applicantIdcard);
        operateTipRespList.addAll(securityWarningTipList);

        return ResponseUtil.newInstance(operateTipRespList);
    }

    /**
     * 获取风险信息提示
     *
     * @param approveRecordInfoId 审批单ID
     * @param applicantIdcard     申请人
     * @param idCard              身份证号
     * @param userName            用户名
     * @return 提示信息
     */
    protected List<OperateTipResp> getRiskTipList(String approveRecordInfoId, String applicantIdcard, String idCard, String userName) {
        List<OperateTipResp> operateTipRespList = new ArrayList<>();

        ResponseStatus responseStatus = riskBusiness.commitApproveRepetitionRisk(applicantIdcard, idCard, userName);
        if (ResponseUtil.isFail(responseStatus)) {
            OperateTipResp operateTipResp = new OperateTipResp();
            operateTipResp.setType("风险");
            operateTipResp.setContent(responseStatus.getMessage());
            operateTipRespList.add(operateTipResp);
        }
        return operateTipRespList;
    }

    /**
     * 获取预警信息提示
     *
     * @param applicantIdcard 申请人
     * @return 提示信息
     */
    protected List<OperateTipResp> getSecurityWarningTipList(String applicantIdcard) {
        List<OperateTipResp> operateTipRespList = new ArrayList<>();

        List<ApproveSecurityWarningInfo> approveSecurityWarningInfos = approveSecurityWarningInfoService.selectByUser(applicantIdcard);
        for (ApproveSecurityWarningInfo approveSecurityWarningInfo : approveSecurityWarningInfos) {
            String threatDesc = approveSecurityWarningInfo.getThreatDesc();
            String content = "接收到的申请人" + applicantIdcard + "预警信息：" + threatDesc;
            OperateTipResp operateTipResp = new OperateTipResp();
            operateTipResp.setType("预警信息");
            operateTipResp.setContent(content);
            operateTipRespList.add(operateTipResp);
        }
        return operateTipRespList;
    }


}
