package com.dragonsoft.dcuc.approve.business.impl;

import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.dragoninfo.dcuc.duceap.enums.SchemeTypeEnum;
import com.dragonsoft.dcuc.approve.business.IRiskBusiness;
import com.dragonsoft.dcuc.approve.model.ApproveCirculationInfo;
import com.dragonsoft.dcuc.approve.model.RiskScheme;
import com.dragonsoft.dcuc.approve.service.IApproveCirculationService;
import com.dragonsoft.dcuc.approve.service.IRiskSchemeService;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/18
 */
@Slf4j
@Component
public class RiskBusinessImpl implements IRiskBusiness {

    private IRiskSchemeService riskSchemeService;

    private IApproveCirculationService approveCirculationService;

    @Autowired
    public void setApproveCirculationService(IApproveCirculationService approveCirculationService) {
        this.approveCirculationService = approveCirculationService;
    }

    @Autowired
    public void setRiskSchemeService(IRiskSchemeService riskSchemeService) {
        this.riskSchemeService = riskSchemeService;
    }

    @Override
    public ResponseStatus commitApproveRepetitionRisk(String applyIdcard, String operateIdcard, String createUserName) {
        Assert.notBlank(applyIdcard);
        Assert.notBlank(operateIdcard);
        Assert.notBlank(createUserName);

        if (applyIdcard.equals(operateIdcard)) {

            RiskScheme riskScheme = riskSchemeService.selectEnableSchemeByType(SchemeTypeEnum.COMMIT_APPROVE_REPETITION_RISK);
            if (riskScheme != null) {
                String riskContent = "审批流程中，提交人与审批人都是：" + createUserName + "(" + operateIdcard + ")。";
                //除第一个节点外，审请人和处理人同为一个人，产生《提交人与审批人重复风险》

                return ResponseStatus.fail(riskContent);
            }
        }
        return ResponseStatus.success();
    }

    @Override
    public ResponseStatus singleApproveRisk(String approveRecordId, String operateIdcard, String createUserName) {
        Assert.notBlank(approveRecordId);
        Assert.notBlank(operateIdcard);
        Assert.notBlank(createUserName);

        RiskScheme riskScheme = riskSchemeService.selectEnableSchemeByType(SchemeTypeEnum.SINGLE_APPROVE_RISK);
        if (riskScheme != null) {

            int continuousApproveNum = Integer.parseInt(StringUtils.isEmpty(riskScheme.getContinuousApproveNum())
                    ? "0" : riskScheme.getContinuousApproveNum());
            //查询该流程单最近风险范围条数的审批记录
            List<ApproveCirculationInfo> list = approveCirculationService.getListByApproveRecordIdAndAgree(approveRecordId);
            // 过滤是当前审批人，去除重复节点，且是同意的条数
            long count = list.stream()
                    // 需去除申请人节点
                    .skip(1)
                    .filter(item -> item.getOperateUserId().equalsIgnoreCase(operateIdcard))
                    .map(ApproveCirculationInfo::getNodeName)
                    .distinct()
                    .count();
            log.info("审批单ID:{} 单人连续审批判断：配置的数量{},去除申请人节点的审批节点数：{}", approveRecordId, continuousApproveNum, count);

            if (count >= continuousApproveNum) {
                String riskContent = "审批流程流转过程中，连续" + continuousApproveNum + "个节点审批人相同，审批人都是："
                        + createUserName + "(" + operateIdcard + "）。";
                return ResponseStatus.fail(riskContent);
            }
        }
        return ResponseStatus.success();
    }

}
