package com.dragonsoft.dcuc.approve.business.impl;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import com.dragoninfo.dcuc.common.entity.SecurityApp;
import com.dragoninfo.dcuc.common.enums.UserExtInfoEnum;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.business.ISecurityUserBusiness;
import com.dragonsoft.dcuc.approve.filter.cache.ISecurityAppResolver;
import com.dragonsoft.dcuc.approve.filter.cache.ISecurityUserResolver;
import com.dragonsoft.dcuc.approve.filter.cache.SecurityAppCache;
import com.dragonsoft.dcuc.approve.filter.cache.SecurityUserCache;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustAppItemUserTokenInfoRespVO;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustAppTokenInfoRespVO;
import com.dragonsoft.dcuc.approve.utils.ProofsUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.commons.util.ip.IpUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/11/15
 */
@Slf4j
@Component
public class SecurityUserBusinessImpl implements ISecurityUserBusiness {

    private SecurityAppCache securityAppCache;

    private ISecurityAppResolver securityAppResolver;

    private SecurityUserCache securityUserCache;

    private ISecurityUserResolver securityAccessResolver;

    @Autowired
    public void setSecurityAppCache(SecurityAppCache securityAppCache) {
        this.securityAppCache = securityAppCache;
    }

    @Autowired
    public void setSecurityAppResolver(ISecurityAppResolver securityAppResolver) {
        this.securityAppResolver = securityAppResolver;
    }

    @Autowired
    public void setSecurityUserCache(SecurityUserCache securityUserCache) {
        this.securityUserCache = securityUserCache;
    }

    @Autowired
    public void setSecurityAccessResolver(ISecurityUserResolver securityAccessResolver) {
        this.securityAccessResolver = securityAccessResolver;
    }

    @Override
    public SecurityApp getSecurityApp(String appCode) {
        SecurityApp securityApp = securityAppCache.getIfPresent(appCode);
        if (ObjectUtil.isNull(securityApp)) {
            securityApp = securityAppResolver.resolve(appCode).orElse(null);
            if (ObjectUtil.isNotNull(securityApp)) {
                securityAppCache.put(appCode, securityApp);
            }
        }
        return securityApp;
    }

    @Override
    public ResponseDTO<SecurityUser> getSecurityUser(String idcard, String appCode) {
        SecurityApp securityApp = getSecurityApp(appCode);
        if (ObjectUtil.isNull(securityApp)) {
            String format = String.format("无法获取到应用：%s", appCode);
            log.error(format);
            return ResponseUtil.dtoFail("应用凭据信息无效");
        }

        SecurityUser securityUser = securityUserCache.getIfPresent(idcard);
        if (ObjectUtil.isNull(securityUser)) {
            securityUser = securityAccessResolver.resolve(idcard).orElse(null);
            if (ObjectUtil.isNotNull(securityUser)) {
                securityUserCache.put(idcard, securityUser);
            }
        }

        if (ObjectUtil.isNull(securityUser)) {
            String format = String.format("无法获取到用户：%s", idcard);
            log.error(format);
            return ResponseUtil.dtoFail("用户凭据信息无效");
        }

        Map<String, Object> extendtionsMap = securityUser.getExtendtions();
        if (MapUtil.isEmpty(extendtionsMap)) {
            extendtionsMap = new HashMap<>(UserExtInfoEnum.values().length);
        }
        extendtionsMap.put(UserExtInfoEnum.SECURITY_APP.getValue(), securityApp);

        securityUser.setExtendtions(extendtionsMap);

        return ResponseUtil.newInstance(securityUser);
    }

    @Override
    public void setUserExtInfo(SecurityUser securityUser, HttpServletRequest request) {

        Map<String, Object> extendtionsMap = securityUser.getExtendtions();
        if (MapUtil.isEmpty(extendtionsMap)) {
            extendtionsMap = new HashMap<>(UserExtInfoEnum.values().length);
        }

        String realIpAdrress = IpUtils.getRealIpAdrress(request);
        String userToken = ProofsUtil.getUserToken(request);
        String appToken = ProofsUtil.getAppToken(request);
        extendtionsMap.put(UserExtInfoEnum.USER_TOKEN_ID.getValue(), userToken);
        extendtionsMap.put(UserExtInfoEnum.APP_TOKEN_ID.getValue(), appToken);
        extendtionsMap.put(UserExtInfoEnum.REQUEST_IP.getValue(), realIpAdrress);

        securityUser.setExtendtions(extendtionsMap);
    }

    @Override
    public void setUserExtInfo(SecurityUser securityUser, ZeroTrustAppTokenInfoRespVO appTokenInfo) {

        Map<String, Object> extendtionsMap = securityUser.getExtendtions();
        if (MapUtil.isEmpty(extendtionsMap)) {
            extendtionsMap = new HashMap<>(UserExtInfoEnum.values().length);
        }
        ZeroTrustAppItemUserTokenInfoRespVO userToken = appTokenInfo.getUserToken();
        String userTokenId = userToken.getUserTokenId();
        String requestIp = userToken.getIp();
        String appTokenId = appTokenInfo.getAppTokenId();

        extendtionsMap.put(UserExtInfoEnum.USER_TOKEN_ID.getValue(), userTokenId);
        extendtionsMap.put(UserExtInfoEnum.APP_TOKEN_ID.getValue(), appTokenId);
        extendtionsMap.put(UserExtInfoEnum.REQUEST_IP.getValue(), requestIp);

        securityUser.setExtendtions(extendtionsMap);
    }

}
