package com.dragonsoft.dcuc.approve.event.listener;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.dragonsoft.auditlog.collection.qmtj.LogSendComponent;
import com.dragonsoft.auditlog.collection.qmtj.pojo.req.ApproveBusLogV2;
import com.dragonsoft.dcuc.approve.event.event.AuditApproveOperateLogEvent;
import com.dragonsoft.dcuc.approve.model.dto.LogInfoMetaDTO;
import com.dragonsoft.dcuc.approve.model.dto.QmOperateLogDTO;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wangrs
 * @date 2021/01/22
 * @desc 事件监听类
 */
@Component
public class ApproveOperateLogEventListener implements ApplicationListener<AuditApproveOperateLogEvent> {

    private LogSendComponent logSendComponent;

    private ApproveProperties approveProperties;

    @Autowired
    public void setLogSendComponent(LogSendComponent logSendComponent) {
        this.logSendComponent = logSendComponent;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    /**
     * 审批操作日志
     */
    public static final String OPERATE_LOG_TYPE = "009";

    /**
     * 是否扩线，就是关联审批流程
     */
    public static final String EXTEND = "1";

//    @Override
//    public void onApplicationEvent(AuditApproveOperateLogEvent event) {
//        if (!approveProperties.getSendToAudit()) {
//            return;
//        }
//
//        String auditSysId = approveProperties.getAuditSysId();
//
//        QmOperateLogDTO qmOperateLogDTO = event.getQmOperateLogDTO();
//        String appToken = qmOperateLogDTO.getAppToken();
//        String requestIdcard = qmOperateLogDTO.getRequestIdcard();
//        String requestUsername = qmOperateLogDTO.getRequestUsername();
//        String requestApproveIp = qmOperateLogDTO.getRequestApproveIp();
//
//        QmApproveTypeEnum qmApproveTypeEnum = qmOperateLogDTO.getQmApproveTypeEnum();
//        String businessType = qmOperateLogDTO.getBusinessType();
//        String approveIdcard = qmOperateLogDTO.getApproveIdcard();
//        String approveUserName = qmOperateLogDTO.getApproveUserName();
//        OperateDisposeTypeEnum operateDisposeTypeEnum = qmOperateLogDTO.getOperateDisposeTypeEnum();
//        Date approveTime = qmOperateLogDTO.getApproveTime();
//        // 文档有误，实际需使用毫秒
//        String approveTimeString = approveTime.getTime() + "";
//        String approveLogId = qmOperateLogDTO.getApproveLogId();
//
//        Boolean flowFinish = qmOperateLogDTO.getFlowFinish();
//        String approveNo = qmOperateLogDTO.getApproveNo();
//
//        ApproveBusLog approveBusLog = new ApproveBusLog();
//        approveBusLog.setAppToken(appToken);
//        approveBusLog.setApplyApproveId(requestIdcard);//审批操作人身份证号
//        approveBusLog.setApplyApproveIdCard(requestIdcard); //审批操作人身份证号
//        approveBusLog.setApplyApproveName(requestUsername);//审批操作人姓名
//        approveBusLog.setApplyApproveIp(requestApproveIp);//终端IP
//        approveBusLog.setOperateTime(approveTimeString);//操作时间
//        approveBusLog.setApplyType(qmApproveTypeEnum.getValue());
//        approveBusLog.setBusinessType(businessType);
//        approveBusLog.setApproveContentHash("");
//        approveBusLog.setApproveTableHash("");
//
//        if (qmApproveTypeEnum.equals(QmApproveTypeEnum.OPERATE)) {
//            approveBusLog.setApproveId(approveIdcard);
//            approveBusLog.setApproveIdCard(approveIdcard);
//            approveBusLog.setApproveName(approveUserName);
//            approveBusLog.setApproveInforceTime(approveTimeString);
//
//            if (flowFinish != null && flowFinish) {
//                approveBusLog.setApproveResult(operateDisposeTypeEnum.getAuditCode());
//                approveBusLog.setApproveTime(approveTimeString);
//            }
//        }
//        approveBusLog.setApproveTaskId(approveLogId);
//        approveBusLog.setApplySystemId("");
//        approveBusLog.setApproveCenterId(approveNo);
//        approveBusLog.setExtend(EXTEND);
//        approveBusLog.setRelatedId("");
//
//        List<ApproveBusLog> approveBusLogList = Collections.singletonList(approveBusLog);
//        logSendComponent.sendApproveBusLog(auditSysId, OPERATE_LOG_TYPE, approveBusLogList);
//        
//    }
    
    
    @Override
    public void onApplicationEvent(AuditApproveOperateLogEvent event) {
        if (!approveProperties.getSendToAudit()) {
            return;
        }

        String auditSysId = approveProperties.getAuditSysId();

        QmOperateLogDTO qmOperateLogDTO = event.getQmOperateLogDTO();
        LogInfoMetaDTO logInfoMetaDTO = qmOperateLogDTO.getLogInfoMetaDTO();
        

        ApproveBusLogV2 approveBusLog = new ApproveBusLogV2();
        
        Map<String,Object> baseStandardInfo = new HashMap<>();
        baseStandardInfo.put("appName", logInfoMetaDTO.getAppName());
        baseStandardInfo.put("operatorCertNo", logInfoMetaDTO.getOperatorCertNo());
        baseStandardInfo.put("operatorName", logInfoMetaDTO.getOperatorName());
        baseStandardInfo.put("operatorOrgName", logInfoMetaDTO.getOperatorOrgName());
        baseStandardInfo.put("operatorOrgCode", logInfoMetaDTO.getOperatorOrgCode());
        baseStandardInfo.put("terminalIp", logInfoMetaDTO.getTerminalIp());
        
        approveBusLog.setBaseStandardInfo(baseStandardInfo);
        approveBusLog.setTaskId(logInfoMetaDTO.getTaskId());
        approveBusLog.setTaskName(logInfoMetaDTO.getTaskName());
        approveBusLog.setOperateTime(logInfoMetaDTO.getOperateTime());
        approveBusLog.setOperateType(logInfoMetaDTO.getOperateType());

        List<ApproveBusLogV2> approveBusLogList = Collections.singletonList(approveBusLog);
        try {
        	ObjectMapper mapper = new ObjectMapper();
			String writeValueAsString = mapper.writeValueAsString(approveBusLogList);
			JSONArray jsonArray = JSONObject.parseArray(writeValueAsString);
			logSendComponent.sendApproveBusLogV2(auditSysId, OPERATE_LOG_TYPE, jsonArray);
        }catch (Exception e) {
        	e.printStackTrace();
		}
        
    }
}
