package com.dragonsoft.dcuc.approve.event.listener;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.dragoninfo.duceap.commons.util.kafka.KafkaServer;
import com.dragonsoft.dcuc.approve.event.event.ApproveFinalResultEvent;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoMsgItemDTO;
import com.dragonsoft.dcuc.approve.model.dto.ApproveMessageInfoDTO;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.properties.KafkaProperties;
import com.dragonsoft.duceap.commons.util.UUIDUtils;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.mq.client.model.vo.MessageInfoReqVo;
import com.dragonsoft.mq.client.model.vo.MessageInfoVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/11
 */
@Slf4j
@Component
public class DcmsSendFinalResultListener implements ApplicationListener<ApproveFinalResultEvent> {

    private ApproveProperties approveProperties;

    private KafkaProperties kafkaProperties;

    private KafkaServer kafkaServer;

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Autowired
    public void setKafkaProperties(KafkaProperties kafkaProperties) {
        this.kafkaProperties = kafkaProperties;
    }

    @Autowired
    public void setKafkaServer(KafkaServer kafkaServer) {
        this.kafkaServer = kafkaServer;
    }

    @Override
    public void onApplicationEvent(ApproveFinalResultEvent event) {
        log.info("====审批流程结束,开始发送消息====");

        ApproveMessageInfoDTO approveMessageInfoDTO = event.getApproveMessageInfoDTO();
        String callbackUrl = approveMessageInfoDTO.getCallbackUrl();

        String appCode = approveProperties.getAppCode();

        String dateFormat = DateUtils.getDateFormat(DateConst.DB_STORE_TIME);
        String uuid = UUIDUtils.getUUID();
        //流程结束且是同意操作才发消息
        String processInstanceId = approveMessageInfoDTO.getProcessInstanceId();
        String approveNo = approveMessageInfoDTO.getApproveNo();
        String sourceAppCode = approveMessageInfoDTO.getSourceAppCode();
        ApprovalInfoMsgItemDTO approvalInfoMsgItemDTO = approveMessageInfoDTO.getApprovalInfoMsgItemDTO();

        if (StrUtil.isNotBlank(callbackUrl)) {
            log.info("审批单：{} 有设置回调地址，不发送审批结果回调消息", approveNo);
            return;
        }

        Map<String, String> map = new HashMap<>(6);
        map.put("approveNo", approveNo);
        map.put("sourceAppCode", sourceAppCode);
        map.put("approveInfo", JsonUtils.toJSONString(approvalInfoMsgItemDTO));
        map.put("processInstanceId", processInstanceId);
        map.put("approveResult", approveMessageInfoDTO.getApproveResult().getValue());
        map.put("messageId", approveMessageInfoDTO.getMessageId());

        List<Map<String, String>> infoSet = new ArrayList<>();
        infoSet.add(map);
        MessageInfoVo messageInfoVo = new MessageInfoVo();

        messageInfoVo.setTitle("审批系统-权限申请审批");
        messageInfoVo.setCreateTime(dateFormat);
        messageInfoVo.setInfoSet(infoSet);
        messageInfoVo.setContent(processInstanceId);
        messageInfoVo.setUserSystem("审批系统");
        messageInfoVo.setUserSystemCode(appCode);
        messageInfoVo.setUserName("审批系统");
        messageInfoVo.setMessageId(uuid);
        messageInfoVo.setInfoType(kafkaProperties.getAuthServiceApplyTopic());
        List<MessageInfoVo> messageInfoVos = new ArrayList<>();
        messageInfoVos.add(messageInfoVo);

        MessageInfoReqVo requestVo = new MessageInfoReqVo();
        //设置消息
        requestVo.setMessages(messageInfoVos);
        requestVo.setTopic(kafkaProperties.getAuthServiceApplyTopic());
        requestVo.setAppKey(kafkaProperties.getAppKey());
        requestVo.setAppSecret(kafkaProperties.getAppSecret());
        log.info("====审批流程结束：开始推送结果,实例ID{}====,消息体为：{}", processInstanceId, JSON.toJSONString(requestVo));
        kafkaServer.send(requestVo, kafkaProperties.getKafkaServers());
    }
}
