package com.dragonsoft.dcuc.approve.event.listener;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.dragonsoft.dcuc.approve.business.external.INoticeBusiness;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.ZeroTrustApproveEndEnum;
import com.dragonsoft.dcuc.approve.enumresources.ZeroTrustEndFlagEnum;
import com.dragonsoft.dcuc.approve.event.event.ApproveFinalResultEvent;
import com.dragonsoft.dcuc.approve.model.req.v3.ZeroTrustApproveCallbackReqV3VO;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoMsgItemDTO;
import com.dragonsoft.dcuc.approve.model.dto.ApproveMessageInfoDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/11
 */
@Slf4j
@Component
public class HttpCallbackSendFinalResultListener implements ApplicationListener<ApproveFinalResultEvent> {

    private INoticeBusiness noticeBusiness;

    @Autowired
    public void setNoticeBusiness(INoticeBusiness noticeBusiness) {
        this.noticeBusiness = noticeBusiness;
    }

    @Override
    public void onApplicationEvent(ApproveFinalResultEvent event) {

        ApproveMessageInfoDTO approveMessageInfoDTO = event.getApproveMessageInfoDTO();
        String processInstanceId = approveMessageInfoDTO.getProcessInstanceId();
        ApprovalInfoMsgItemDTO approvalInfoMsgItemDTO = approveMessageInfoDTO.getApprovalInfoMsgItemDTO();
        ApproveWorkFlowOperationEnum approveResult = approveMessageInfoDTO.getApproveResult();
        String approveNo = approveMessageInfoDTO.getApproveNo();

        String callBackUrl = approveMessageInfoDTO.getCallbackUrl();
        String message = approveMessageInfoDTO.getApproveMessage();

        log.info("====审批流程结束,开始通过http接口发送回调信息====,callBackUrl:{}", callBackUrl);

        if (StrUtil.isNotBlank(callBackUrl)) {

            ZeroTrustApproveEndEnum zeroTrustApproveEndEnum = ZeroTrustApproveEndEnum.parse(approveResult);
            ZeroTrustEndFlagEnum zeroTrustEndFlagEnum = ZeroTrustEndFlagEnum.parseApproveWorkFlowOperationEnum(approveResult);
            ZeroTrustApproveCallbackReqV3VO zeroTrustApproveCallbackReqV3VO = new ZeroTrustApproveCallbackReqV3VO();
            zeroTrustApproveCallbackReqV3VO.setType(zeroTrustApproveEndEnum.getValue());
            zeroTrustApproveCallbackReqV3VO.setProcessInstId(processInstanceId);
            zeroTrustApproveCallbackReqV3VO.setOpenId(approvalInfoMsgItemDTO.getApplicantIdcard());
            zeroTrustApproveCallbackReqV3VO.setUserName(approvalInfoMsgItemDTO.getApplicantName());
            zeroTrustApproveCallbackReqV3VO.setEndFlag(zeroTrustEndFlagEnum.getValue());
            zeroTrustApproveCallbackReqV3VO.setDescription(message);
            zeroTrustApproveCallbackReqV3VO.setTaskId(approveNo);

            log.info("====审批流程结束,开始发送回调信息URL:{} ,回调内容：{}====", callBackUrl, JSONUtil.toJsonStr(zeroTrustApproveCallbackReqV3VO));

            noticeBusiness.retryApproveNotification(callBackUrl, zeroTrustApproveCallbackReqV3VO);
        }
    }
}
