package com.dragonsoft.dcuc.approve.event.listener;

import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.business.external.IBuSyncBusiness;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.event.event.ApproveFinalResultEvent;
import com.dragonsoft.dcuc.approve.model.dto.ApproveMessageInfoDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

/**
 * <p>
 * 审批结束后同步到BU
 * </p>
 *
 * @author huangzqa
 * @date 2023/11/13
 */
@Slf4j
@Component
public class SyncToBuSendFinalResultListener implements ApplicationListener<ApproveFinalResultEvent> {

    private IBuSyncBusiness buSyncBusiness;

    @Autowired
    public void setBuSyncBusiness(IBuSyncBusiness buSyncBusiness) {
        this.buSyncBusiness = buSyncBusiness;
    }

    @Override
    public void onApplicationEvent(ApproveFinalResultEvent event) {
        ApproveMessageInfoDTO approveMessageInfoDTO = event.getApproveMessageInfoDTO();
        ApproveWorkFlowOperationEnum approveResult = approveMessageInfoDTO.getApproveResult();

        if (approveResult.equals(ApproveWorkFlowOperationEnum.COMPLETE_AGREE)) {
            ResponseStatus responseStatus = buSyncBusiness.syncToBu(approveMessageInfoDTO);
            if (ResponseUtil.isFail(responseStatus)) {
                log.error("sync to bu error: {}", responseStatus.getMessage());
            }
        }

    }
}
