package com.dragonsoft.dcuc.approve.facade;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.business.IApproveTipsBusiness;
import com.dragonsoft.dcuc.approve.business.external.IDcucAuthBusiness;
import com.dragonsoft.dcuc.approve.business.IWorkFlowBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.model.dto.OperateReqDTO;
import com.dragonsoft.dcuc.approve.model.dto.ResourceInfoDTO;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowTaskStateEnum;
import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.PageTypeEnum;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.ApproveCirculationInfo;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.req.*;
import com.dragonsoft.dcuc.approve.model.resp.*;
import com.dragonsoft.dcuc.approve.model.vo.ApproveTaskRequestVo;
import com.dragonsoft.dcuc.approve.model.dto.TaskRequestUserInfoDTO;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.service.IApproveCirculationService;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.dcuc.approve.business.IApproveCommonBusiness;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.BaseSecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.duwf.api.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author lidr
 * @date 2021/1/29
 */
@RestController
public class ApproveFacade implements IApproveFacade {

    private IApproveCommonBusiness approveService;

    private IWorkFlowBusiness workFlowBusiness;

    private ApproveProperties approveProperties;

    private IApproveRecordService approveRecordService;

    private IApproveCirculationService approveCirculationInfoService;

    private IDcucAuthBusiness dcucAuthBusiness;

    private IApproveTipsBusiness approveTipsBusiness;

    @Autowired
    public void setApproveTipsBusiness(IApproveTipsBusiness approveTipsBusiness) {
        this.approveTipsBusiness = approveTipsBusiness;
    }

    @Autowired
    public void setApproveService(IApproveCommonBusiness approveService) {
        this.approveService = approveService;
    }

    @Autowired
    public void setWorkFlowBusiness(IWorkFlowBusiness workFlowBusiness) {
        this.workFlowBusiness = workFlowBusiness;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Autowired
    public void setApproveCirculationInfoService(IApproveCirculationService approveCirculationInfoService) {
        this.approveCirculationInfoService = approveCirculationInfoService;
    }

    @Autowired
    public void setDcucAuthBusiness(IDcucAuthBusiness dcucAuthBusiness) {
        this.dcucAuthBusiness = dcucAuthBusiness;
    }

    @Override
    public Page<ApprovalInfoRespVO> queryUserApplyList(Searchable searchable) {
        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        searchable.addSearchFilter("createUser", SearchOperator.eq, idcard);
        return approveRecordService.queryUserApproveList(searchable);
    }


    @Override
    public Page<TaskVo> queryApproveList(Searchable searchable) {
        //设置查询所有数据（待办）
        return approveService.queryApproveList(searchable);
    }

    @Override
    public ResponseStatus delApply(String id) {

        return approveService.deleteProcessInstance(id);
    }

    @Override
    public ResponseDTO<OperateRespVO> approveOperate(ApproveOperateReqVO approveOperateReqVO) {

        TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();
        ApproveMessage approveMessage = new ApproveMessage();
        approveMessage.setMessage(approveOperateReqVO.getMessage());
        approveMessage.setApproveState(approveOperateReqVO.getOperateType());
        taskRequest.setApproveMessage(approveMessage);

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        TaskRequestUserInfoDTO taskRequestUserInfoDTO = TaskRequestUserInfoDTO.parseBaseSecurityUser(currentUser);
        ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum = EnumUtils.enumOf(ApproveWorkFlowOperationEnum.class, approveOperateReqVO.getOperateType());

        OperateReqDTO operateReqDTO = OperateReqDTO.builder()
                .taskId(approveOperateReqVO.getTaskId())
                .processInstanceId(approveOperateReqVO.getProcessInstanceId())
                .approveWorkFlowOperationEnum(approveWorkFlowOperationEnum)
                .taskRequest(taskRequest)
                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                .activitiHolderList(approveOperateReqVO.getActivitiHolderList())
                .needCheckRedList(true)
                .build();

        return approveService.approveOperate(operateReqDTO);
    }

    @Override
    public ResponseDTO<List<OperateTipResp>> approveOperatePreTip(String processInstanceId) {
        return approveTipsBusiness.operateTips(processInstanceId);
    }


    @Override
    public ResponseDTO<OperateRespVO> redListStatus(String approveNo) {
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);
        if (approveRecordInfo == null) {
            return ResponseUtil.dtoFail("审批单号不存在");
        }
        String flowType = approveRecordInfo.getFlowType();
        String applicantIdcard = approveRecordInfo.getApplicantIdcard();
        ApprovalInfoDTO approvalInfoDTO = approveRecordInfo.toApprovalInfo();
        ResourceInfoDTO resourceInfoVo = approvalInfoDTO.getResourceInfoVo();

        FlowTypeEnum flowTypeEnum = EnumUtils.enumOf(FlowTypeEnum.class, flowType);

        OperateRespVO operateRespVO = dcucAuthBusiness.checkHitRedList(applicantIdcard, flowTypeEnum, resourceInfoVo);
        return ResponseUtil.newInstance(operateRespVO);
    }

    @Override
    public ResponseDTO<OperateRespVO> redListApproveOperate(ApproveOperateReqVO approveOperateReqVO) {

        TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();
        ApproveMessage approveMessage = new ApproveMessage();
        approveMessage.setMessage(approveOperateReqVO.getMessage());
        approveMessage.setApproveState(approveOperateReqVO.getOperateType());
        taskRequest.setApproveMessage(approveMessage);
        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        TaskRequestUserInfoDTO taskRequestUserInfoDTO = TaskRequestUserInfoDTO.parseBaseSecurityUser(currentUser);

        ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum = EnumUtils.enumOf(ApproveWorkFlowOperationEnum.class, approveOperateReqVO.getOperateType());

        OperateReqDTO operateReqDTO = OperateReqDTO.builder()
                .taskId(approveOperateReqVO.getTaskId())
                .processInstanceId(approveOperateReqVO.getProcessInstanceId())
                .approveWorkFlowOperationEnum(approveWorkFlowOperationEnum)
                .taskRequest(taskRequest)
                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                .activitiHolderList(approveOperateReqVO.getActivitiHolderList())
                .needCheckRedList(false)
                .build();

        return approveService.approveOperate(operateReqDTO);
    }

    @Override
    public ResponseDTO<ApprovalInfoRespVO> applyDetail(String id) {
        return approveService.queryApproveDetail(id);
    }

    @Override
    public ResponseDTO<String> save(ApprovalInfoReqVO approvalInfoReqVO) {
        approvalInfoReqVO.setAppCode(approveProperties.getAppCode());
        return approveService.save(approvalInfoReqVO);
    }

    @Override
    public ResponseDTO<String> update(ApprovalInfoUpdateReqVO approvalInfoDTO) {
        return approveService.update(approvalInfoDTO);
    }

    @Override
    public ResponseStatus agent(AgentReqVO agentReqVO) {
        approveService.agent(agentReqVO);
        return ResponseStatus.success("操作成功！");
    }

    @Override
    public ResponseDTO<List<ApproveCommentVo>> approveComments(String approveRecordId) {
        //step1 审批历史记录
        List<ApproveCirculationInfo> hisTaskMapNew = approveCirculationInfoService.getListByApproveRecordId(approveRecordId);

        return approveCirculationInfoService.buildApproveCommentVo(hisTaskMapNew);
    }

    @Override
    public ResponseDTO<TraceProcessAndOverVo> processPic(String proInsId) {
        TraceProcessAndOverVo traceProcessAndOverVo = workFlowBusiness.processPic(proInsId);
        return ResponseUtil.newInstance(traceProcessAndOverVo);
    }

    @Override
    public ResponseDTO<ActivityNodeInfoVo> processPicDetail(String proInsId, String statusType, String taskDefinitionId) {
        ActivityNodeInfoVo activityNodeInfoVo = workFlowBusiness.processPicDetail(proInsId, statusType, taskDefinitionId);
        return ResponseUtil.newInstance(activityNodeInfoVo);
    }

    @Override
    public ResponseDTO<StatisticsDataVo> applyStatistics(Searchable searchable) {
        //查询已办数量
        Searchable finishedSearchable = Searchable.newSearchable();
        finishedSearchable.setPage(0, 1);
        finishedSearchable.addSearchFilter("state", SearchOperator.eq, ApproveWorkFlowTaskStateEnum.FINISH.getValue());
        Integer finishQty = approveService.queryApproveQty(finishedSearchable);
        //查询待办数量
        Searchable unFinishSearchable = Searchable.newSearchable();
        unFinishSearchable.setPage(0, 1);
        unFinishSearchable.addSearchFilter("state", SearchOperator.eq, ApproveWorkFlowTaskStateEnum.UN_FINISH.getValue());
        Integer unFinishQty = approveService.queryApproveQty(unFinishSearchable);
        //查询我的请求数量
        Integer myRequestQty = approveService.queryMyRequestQty();
        //返回数据封装
        StatisticsDataVo statisticsDataVo = new StatisticsDataVo();
        statisticsDataVo.setFinishQty(finishQty);
        statisticsDataVo.setUnFinishQty(unFinishQty);
        statisticsDataVo.setMyRequestQty(myRequestQty);
        return ResponseUtil.newInstance(statisticsDataVo);
    }

    @Override
    public ResponseDTO<List<ApproveWorkflowProcessTypeVo>> queryProcessTypeList() {
        List<ApproveWorkflowProcessTypeVo> approveWorkflowProcessTypeVos = workFlowBusiness.queryProcessTypeList();
        return ResponseUtil.newInstance(approveWorkflowProcessTypeVos);
    }

    @Override
    public ResponseDTO<WorkflowProcesstypeVo> queryProcessType(String processTypeCode) {
        WorkflowProcesstypeVo workflowProcesstypeVo = workFlowBusiness.queryProcessType(processTypeCode);
        return ResponseUtil.newInstance(workflowProcesstypeVo);
    }

    @Override
    public ResponseDTO<List<ProcessStatusVo>> processStatus(String approveRecordId) {
        return approveService.approveStatusByBusinessId(approveRecordId);
    }

    @Override
    public ResponseDTO<TaskOperateRespVO> taskOperateForApprove(String taskId, String pageType) {
        PageTypeEnum pageTypeEnum = EnumUtils.enumOf(PageTypeEnum.class, pageType);
        if (pageTypeEnum == null) {
            return ResponseUtil.dtoFail("请传入正确的页面类型");
        }

        if (taskId.equalsIgnoreCase(ApproveConstants.END_STR)) {
            TaskOperateRespVO taskOperateRespVO = new TaskOperateRespVO();
            taskOperateRespVO.setShowRecall(false);
            taskOperateRespVO.setShowBack(false);
            taskOperateRespVO.setShowEdit(false);
            return ResponseUtil.newInstance(taskOperateRespVO);
        }

        TaskOperateRespVO taskOperateRespVO = workFlowBusiness.getTaskSupportOperate(taskId, pageTypeEnum);
        ApproveRecordInfo approveRecordInfo = approveRecordService.getOneByTaskId(taskId);
        String appCode = approveProperties.getAppCode();
        String recordAppCode = approveRecordInfo.getAppCode();

        if (!appCode.equalsIgnoreCase(recordAppCode)) {
            taskOperateRespVO.setShowEdit(false);
        }

        return ResponseUtil.newInstance(taskOperateRespVO);
    }

    @Override
    public ResponseDTO<List<ApproveCommentVo>> taskApproveComments(String taskNo) {
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(taskNo);
        if (ObjectUtil.isNull(approveRecordInfo)) {
            return ResponseUtil.dtoFail("找不到审批单");
        }
        String approveRecordId = approveRecordInfo.getId();
        return approveComments(approveRecordId);
    }

    @Override
    public ResponseDTO<ApprovalInfoRespVO> taskApplyDetail(String taskNo) {
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(taskNo);
        if (ObjectUtil.isNull(approveRecordInfo)) {
            return ResponseUtil.dtoFail("找不到审批单");
        }
        String approveRecordId = approveRecordInfo.getId();

        return approveService.queryApproveDetail(approveRecordId);
    }

    @Override
    public ResponseDTO<OperateRespVO> taskApproveOperate(TaskApproveOperateReqVO taskApproveOperateReqVO) {
        String taskNo = taskApproveOperateReqVO.getTaskNo();
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(taskNo);
        if (ObjectUtil.isNull(approveRecordInfo)) {
            return ResponseUtil.dtoFail("找不到审批单");
        }
        String processInstanceId = approveRecordInfo.getProcessInstanceId();

        ApproveOperateReqVO approveOperateReqVO = new ApproveOperateReqVO();
        BeanUtil.copyProperties(taskApproveOperateReqVO, approveOperateReqVO);
        approveOperateReqVO.setProcessInstanceId(processInstanceId);
        return this.approveOperate(approveOperateReqVO);
    }

    @Override
    public ResponseStatus taskDelApply(String taskNo) {
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(taskNo);
        if (ObjectUtil.isNull(approveRecordInfo)) {
            return ResponseUtil.dtoFail("找不到审批单");
        }
        String approveRecordId = approveRecordInfo.getId();
        return approveService.deleteProcessInstance(approveRecordId);
    }


}
