package com.dragonsoft.dcuc.approve.facade;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.model.ApproveTaskClass;
import com.dragonsoft.dcuc.approve.model.req.ApproveTaskClassAddReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApproveTaskClassUpdateReqVO;
import com.dragonsoft.dcuc.approve.model.resp.ApproveTaskClassDetailRespVO;
import com.dragonsoft.dcuc.approve.service.IApproveTaskClassService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/6/9
 */
@RestController
public class ApproveTaskClassFacade implements IApproveTaskClassFacade {

    private IApproveTaskClassService approveTaskClassService;

    private IApplyInfoFacade applyInfoFacade;

    @Autowired
    public void setApplyInfoFacade(IApplyInfoFacade applyInfoFacade) {
        this.applyInfoFacade = applyInfoFacade;
    }

    @Autowired
    public void setApproveTaskClassService(IApproveTaskClassService approveTaskClassService) {
        this.approveTaskClassService = approveTaskClassService;
    }

    @Override
    public Page<ApproveTaskClassDetailRespVO> page(Searchable searchable) {
        Page<ApproveTaskClassDetailRespVO> page = approveTaskClassService.page(searchable);
        List<ApproveTaskClassDetailRespVO> newLit = page.stream().map(this::fillNowAppName)
                .collect(Collectors.toList());

        return new PageImpl<>(newLit, page.getPageable(), page.getTotalElements());
    }

    @Override
    public ResponseStatus delete(String id) {
        return approveTaskClassService.delete(id);
    }

    @Override
    public ResponseDTO<String> save(ApproveTaskClassAddReqVO approveTaskClassAddReqVO) {
        ResponseDTO<ApproveTaskClass> saveResp = approveTaskClassService.save(approveTaskClassAddReqVO);
        if (ResponseUtil.isFail(saveResp)) {
            return ResponseUtil.dtoFail(saveResp.getMessage());
        }
        ApproveTaskClass approveTaskClass = ResponseUtil.getResult(saveResp);
        return ResponseUtil.newInstance(approveTaskClass.getId());
    }

    @Override
    public ResponseDTO<String> update(ApproveTaskClassUpdateReqVO approveTaskClassUpdateReqVO) {
        ResponseDTO<ApproveTaskClass> updateResp = approveTaskClassService.update(approveTaskClassUpdateReqVO);
        if (ResponseUtil.isFail(updateResp)) {
            return ResponseUtil.dtoFail(updateResp.getMessage());
        }
        ApproveTaskClass approveTaskClass = ResponseUtil.getResult(updateResp);
        return ResponseUtil.newInstance(approveTaskClass.getId());
    }

    @Override
    public ResponseDTO<ApproveTaskClassDetailRespVO> detail(String id) {
        ResponseDTO<ApproveTaskClassDetailRespVO> responseDTO = approveTaskClassService.detail(id);
        if (ResponseUtil.isFail(responseDTO)) {
            return responseDTO;
        }
        ApproveTaskClassDetailRespVO approveTaskClassDetailRespVO = fillNowAppName(ResponseUtil.getResult(responseDTO));
        return ResponseUtil.newInstance(approveTaskClassDetailRespVO);
    }

    /**
     * 填充最新的应用名称
     * 1、如果应用代码存在使用最新的应用名称
     * 2、如果查不到就用旧的应用名称
     *
     * @param approveTaskClassDetailRespVO 任务类型详情
     * @return 填充后的任务类型详情
     */
    protected ApproveTaskClassDetailRespVO fillNowAppName(ApproveTaskClassDetailRespVO approveTaskClassDetailRespVO) {
        String sourceAppCode = approveTaskClassDetailRespVO.getSourceAppCode();
        if (StrUtil.isNotBlank(sourceAppCode)) {
            ApplyInfo applyInfo = applyInfoFacade.getAppByCode(sourceAppCode);
            if (ObjectUtil.isNotNull(applyInfo)) {
                approveTaskClassDetailRespVO.setSourceAppName(applyInfo.getApplyName());
            }
        }
        return approveTaskClassDetailRespVO;
    }

}
