package com.dragonsoft.dcuc.approve.facade.api;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.dragoninfo.dcuc.app.dto.ResourceRequestParamDTO;
import com.dragoninfo.dcuc.app.facade.IResourceFacade;
import com.dragoninfo.dcuc.common.entity.ApiSearchReq;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.common.utils.SearchableUtil;
import com.dragoninfo.dcuc.common.utils.SecurityUserUtil;
import com.dragonsoft.dcuc.approve.business.IApproveBusiness;
import com.dragonsoft.dcuc.approve.business.IApproveCommonBusiness;
import com.dragonsoft.dcuc.approve.business.IApproveUploadBusiness;
import com.dragonsoft.dcuc.approve.business.IWorkFlowBusiness;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.PageTypeEnum;
import com.dragonsoft.dcuc.approve.model.dto.OperateReqDTO;
import com.dragonsoft.dcuc.approve.model.dto.TaskRequestUserInfoDTO;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoUpdateReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApproveOperateReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApproveRecordReportReqVO;
import com.dragonsoft.dcuc.approve.model.resp.*;
import com.dragonsoft.dcuc.approve.model.vo.ApproveTaskRequestVo;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.BaseSecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.core.search.filter.Condition;
import com.dragonsoft.duceap.duwf.api.model.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author huangzqa
 */
@Slf4j
@RestController
public class ApiV1ApproveFacade implements IApiV1ApproveFacade {

    private IApproveCommonBusiness approveService;

    private IResourceFacade resourceFacade;

    private IApproveUploadBusiness approveUploadService;

    private IWorkFlowBusiness workFlowBusiness;

    private IApproveBusiness approveBusiness;

    private ApproveProperties approveProperties;

    @Autowired
    public void setApproveService(IApproveCommonBusiness approveService) {
        this.approveService = approveService;
    }

    @Autowired
    public void setResourceFacade(IResourceFacade resourceFacade) {
        this.resourceFacade = resourceFacade;
    }

    @Autowired
    public void setApproveUploadService(IApproveUploadBusiness approveUploadService) {
        this.approveUploadService = approveUploadService;
    }

    @Autowired
    public void setWorkFlowBusiness(IWorkFlowBusiness workFlowBusiness) {
        this.workFlowBusiness = workFlowBusiness;
    }

    @Autowired
    public void setApproveBusiness(IApproveBusiness approveBusiness) {
        this.approveBusiness = approveBusiness;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Override
    public ResponseDTO<String> save(ApprovalInfoReqVO approvalInfoReqVO) {
        String appCode = approvalInfoReqVO.getAppCode();
        if (StrUtil.isBlank(appCode)) {
            approvalInfoReqVO.setAppCode(approveProperties.getAppCode());
        }
        approvalInfoReqVO.setApproveTaskClass(approveProperties.getAuthTaskClass());
        return approveService.save(approvalInfoReqVO);
    }

    @Override
    public ResponseDTO<String> update(ApprovalInfoUpdateReqVO approvalInfoDTO) {
        return approveService.update(approvalInfoDTO);
    }

    @Override
    public ResponseDTO<ApprovalInfoRespVO> applyDetail(String id) {
        return approveService.queryApproveDetail(id);
    }

    @Override
    public ResponseDTO<List<ApproveWorkflowProcessTypeVo>> queryProcessTypeList() {
        List<ApproveWorkflowProcessTypeVo> approveWorkflowProcessTypeVos = workFlowBusiness.queryProcessTypeList();
        return ResponseUtil.newInstance(approveWorkflowProcessTypeVos);
    }

    @Override
    public ResponseDTO<WorkflowProcesstypeVo> queryProcessType(String processTypeCode) {
        WorkflowProcesstypeVo workflowProcesstypeVo = workFlowBusiness.queryProcessType(processTypeCode);
        return ResponseUtil.newInstance(workflowProcesstypeVo);
    }

    @Override
    public ResponseStatus syncResource() {
        ResourceRequestParamDTO paramDTO = new ResourceRequestParamDTO();
        paramDTO.setListingAll(true);
        resourceFacade.getResourceRequestResults(paramDTO);
        resourceFacade.resourceSync();
        return ResponseDTO.success();
    }

    @Override
    public ResponseDTO<List<TaskOutgoingVo>> taskOut(String taskId) {
        List<TaskOutgoingVo> list = workFlowBusiness.buildTaskOutgoingVoList(taskId);
        return ResponseUtil.newInstance(list);
    }

    @Override
    public ResponseDTO<TaskOperateRespVO> taskOperate(String taskId, String pageType) {
        PageTypeEnum pageTypeEnum = EnumUtils.enumOf(PageTypeEnum.class, pageType);
        if (pageTypeEnum == null) {
            return ResponseUtil.dtoFail("请传入正确的页面类型");
        }
        TaskOperateRespVO taskOperateRespVO = workFlowBusiness.getTaskSupportOperate(taskId, pageTypeEnum);
        return ResponseUtil.newInstance(taskOperateRespVO);
    }

    @Override
    public ResponseDTO<OperateRespVO> approveOperate(ApproveOperateReqVO approveOperateReqVO) {
        log.info("approveOperateReqVO req:{}", JsonUtils.toJSONString(approveOperateReqVO));
        TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();
        ApproveMessage approveMessage = new ApproveMessage();
        approveMessage.setMessage(approveOperateReqVO.getMessage());
        approveMessage.setApproveState(approveOperateReqVO.getOperateType());
        taskRequest.setApproveMessage(approveMessage);
        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();

        log.info("BaseSecurityUser req:{}", JsonUtils.toJSONString(currentUser));

        TaskRequestUserInfoDTO taskRequestUserInfoDTO = TaskRequestUserInfoDTO.parseBaseSecurityUser(currentUser);
        taskRequestUserInfoDTO.setIdCard(approveOperateReqVO.getIdCard());
        taskRequestUserInfoDTO.setUserName(approveOperateReqVO.getUserName());
        ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum = EnumUtils.enumOf(ApproveWorkFlowOperationEnum.class, approveOperateReqVO.getOperateType());

        OperateReqDTO operateReqDTO = OperateReqDTO.builder()
                .taskId(approveOperateReqVO.getTaskId())
                .processInstanceId(approveOperateReqVO.getProcessInstanceId())
                .approveWorkFlowOperationEnum(approveWorkFlowOperationEnum)
                .taskRequest(taskRequest)
                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                .activitiHolderList(approveOperateReqVO.getActivitiHolderList())
                .needCheckRedList(false)
                .build();

        return approveService.approveOperate(operateReqDTO);
    }

    @Override
    public ResponseDTO<TraceProcessAndOverVo> processPic(String processInstanceId) {
        TraceProcessAndOverVo traceProcessAndOverVo = workFlowBusiness.processPic(processInstanceId);
        return ResponseUtil.newInstance(traceProcessAndOverVo);
    }

    @Override
    public ResponseStatus delApply(String id) {
        return approveService.deleteProcessInstance(id);
    }

    @Override
    public ResponseStatus approveComments(String processInstanceId) {
        return approveService.approveComments(processInstanceId);
    }

    @Override
    public ResponseDTO<List<ProcessStatusVo>> processStatus(String processInstanceId) {
        return approveService.approveStatus(processInstanceId);
    }

    @Override
    public ResponseStatus approveUpload(List<ApproveRecordReportReqVO> approveRecordList) {
        String msg = approveUploadService.checkParam(approveRecordList);
        if (StringUtils.isNotEmpty(msg)) {
            return ResponseUtil.dtoFail(msg.substring(0, msg.length() - 1));
        }
        return approveUploadService.approveUpload(approveRecordList);
    }

    @Override
    public ResponseDTO<ProcessDetailRespVO> processDetail(String processNo) {
        WorkflowProcessVo workflowProcessVo = workFlowBusiness.queryWorkflowProcessByBusinessCode(processNo);
        ProcessDetailRespVO processDetailRespVO = ProcessDetailRespVO.paseByWorkflowProcessVo(workflowProcessVo);
        return ResponseUtil.newInstance(processDetailRespVO);
    }

    @Override
    public ResponseDTO<List<BatchTaskOperateRespVO>> batchApproveOperateType(ApiSearchReq apiSearchReq) {
        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseUtil.dtoFail("请传入凭据");
        }

        Map<String, SearchOperator[]> requestMap = new HashMap<>(2);
        requestMap.put("pageType", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("approveNo", new SearchOperator[]{SearchOperator.eq, SearchOperator.in});
        Searchable searchable;
        try {
            searchable = SearchableUtil.parseApiSearchReqToSearchable(apiSearchReq, requestMap);
            searchable.removePageable();
        } catch (IllegalArgumentException e) {
            String message = e.getMessage();
            return ResponseUtil.dtoFail(message);
        }

        String pageType = (String) searchable.getSingleSearchFilter("pageType").getValue();

        PageTypeEnum pageTypeEnum = EnumUtils.enumOf(PageTypeEnum.class, pageType);
        if (pageTypeEnum == null) {
            return ResponseUtil.dtoFail("请传入正确的页面类型");
        }

        Condition approveNoCondition = searchable.getSingleSearchFilter("approveNo");
        SearchOperator operator = approveNoCondition.getOperator();
        List<String> approveNoList = Collections.emptyList();
        if (operator.equals(SearchOperator.eq)) {
            String value = (String) approveNoCondition.getValue();
            approveNoList = Collections.singletonList(value);
        } else if (operator.equals(SearchOperator.in)) {
            approveNoList = (List<String>) approveNoCondition.getValue();
        }
        List<BatchTaskOperateRespVO> batchTaskOperateRespList = approveBusiness.batchTaskOperate(approveNoList, pageTypeEnum);
        return ResponseUtil.newInstance(batchTaskOperateRespList);
    }
}