package com.dragonsoft.dcuc.approve.facade.api;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragoninfo.dcuc.common.entity.ApiSearchReq;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.common.utils.SearchableUtil;
import com.dragoninfo.dcuc.common.utils.SecurityUserUtil;
import com.dragoninfo.dcuc.duceap.enums.ManageDisposeTypeEnum;
import com.dragonsoft.dcuc.approve.business.IApproveBusiness;
import com.dragonsoft.dcuc.approve.business.IApproveCommonBusiness;
import com.dragonsoft.dcuc.approve.business.IWorkFlowBusiness;
import com.dragonsoft.dcuc.approve.business.external.IDcucBusiness;
import com.dragonsoft.dcuc.approve.enumresources.*;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.OperateReqDTO;
import com.dragonsoft.dcuc.approve.model.dto.TaskRequestUserInfoDTO;
import com.dragonsoft.dcuc.approve.model.req.*;
import com.dragonsoft.dcuc.approve.model.req.v2.ApproveOperateV2ReqVO;
import com.dragonsoft.dcuc.approve.model.resp.*;
import com.dragonsoft.dcuc.approve.model.vo.ApproveTaskRequestVo;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.OauthUserVO;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.BaseSecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.duwf.api.model.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.*;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/16
 */
@Slf4j
@RestController
public class ApiV2ApproveFacade implements IApiV2ApproveFacade {

    private IApproveCommonBusiness approveService;

    private IApproveRecordService approveRecordService;

    private IApproveBusiness approveBusiness;

    private IWorkFlowBusiness workFlowBusiness;

    private IDcucBusiness dcucBusiness;

    private ApproveProperties approveProperties;

    @Autowired
    public void setApproveService(IApproveCommonBusiness approveService) {
        this.approveService = approveService;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Autowired
    public void setApproveBusiness(IApproveBusiness approveBusiness) {
        this.approveBusiness = approveBusiness;
    }

    @Autowired
    public void setWorkFlowBusiness(IWorkFlowBusiness workFlowBusiness) {
        this.workFlowBusiness = workFlowBusiness;
    }

    @Autowired
    public void setDcucBusiness(IDcucBusiness dcucBusiness) {
        this.dcucBusiness = dcucBusiness;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Override
    public ResponseDTO<String> approveAuthorities(ApprovalInfoAuthReqVO approvalInfoAuthReqVO) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseUtil.dtoFail("请传入凭据");
        }
        @Valid @NotNull ResourceInfoReqVO resourceInfo = approvalInfoAuthReqVO.getResourceInfo();

        ApprovalInfoReqVO approvalInfoReqVO = new ApprovalInfoReqVO();
        BeanUtil.copyProperties(approvalInfoAuthReqVO, approvalInfoReqVO);
        approvalInfoReqVO.setAppCode(appCode);
        approvalInfoReqVO.setResourceInfoDTO(resourceInfo);
        approvalInfoReqVO.setOperateType(CommitTypeEnum.APPLY_COMMIT.getValue());

        String permissionValidType = approvalInfoReqVO.getPermissionValidType();
        PermissionValidTypeEnum permissionValidTypeEnum = EnumUtils.enumOf(PermissionValidTypeEnum.class, permissionValidType);
        if (permissionValidTypeEnum == null) {
            return ResponseUtil.dtoFail("请传入正确的有效时间类型");
        }

        if (permissionValidTypeEnum.equals(PermissionValidTypeEnum.CUSTOM)) {
            Date startTime = approvalInfoReqVO.getStartTime();
            Date endTime = approvalInfoReqVO.getEndTime();
            if (startTime == null || endTime == null) {
                return ResponseUtil.dtoFail("有效时间类型为自定义时，开始时间、结束时间不能为空");
            }
        }

        @NotBlank String businessCode = approvalInfoReqVO.getBusinessCode();
        WorkflowProcessVo workflowProcessVo = workFlowBusiness.queryWorkflowProcessByBusinessCode(businessCode);
        if (workflowProcessVo == null) {
            return ResponseUtil.dtoFail("请传入正确的流程标识码");
        }
        ProcessDetailRespVO processDetailRespVO = ProcessDetailRespVO.paseByWorkflowProcessVo(workflowProcessVo);
        String processTypeCode = processDetailRespVO.getProcessTypeCode();
        approvalInfoReqVO.setProcessType(processTypeCode);

        @NotBlank String flowType = approvalInfoAuthReqVO.getFlowType();
        FlowTypeEnum flowTypeEnum = EnumUtils.enumOf(FlowTypeEnum.class, flowType);
        if (flowTypeEnum == null) {
            return ResponseUtil.dtoFail("请传入正确的权限申请类型");
        }

        ResponseStatus checkResourceStatus = approveBusiness.checkResource(flowTypeEnum, resourceInfo);
        if (ResponseUtil.isFail(checkResourceStatus)) {
            return ResponseUtil.dtoFail(checkResourceStatus.getMessage());
        }

        String applicantIdcard = approvalInfoReqVO.getApplicantIdcard();

        OauthUserVO oauthUserVO = dcucBusiness.getOauthUserInfoByIdcard(applicantIdcard);
        if (oauthUserVO == null) {
            return ResponseUtil.dtoFail("申请人不存在");
        }

        approvalInfoReqVO.setApproveTaskClass(approveProperties.getAuthTaskClass());
        ApproveRecordInfo approveRecordInfo = approveService.saveBase(approvalInfoReqVO, ManageDisposeTypeEnum.INSERT, new Date());

        String approveNo = approveRecordInfo.getApproveNo();

        return ResponseUtil.newInstance(approveNo);
    }

    @Override
    public ResponseStatus updateApprovesAuthorities(ApprovalInfoAuthUpdateReqVO approvalInfoAuthUpdateReqVO) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseStatus.fail("请传入凭据");
        }

        @NotBlank String approveNo = approvalInfoAuthUpdateReqVO.getApproveNo();

        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);
        if (approveRecordInfo == null) {
            return ResponseStatus.fail("审批单不存在");
        }
        String approveRecordInfoId = approveRecordInfo.getId();

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseStatus.fail("非本应用申请审批单无法操作");
        }
        @NotBlank String businessCode = approvalInfoAuthUpdateReqVO.getBusinessCode();
        WorkflowProcessVo workflowProcessVo = workFlowBusiness.queryWorkflowProcessByBusinessCode(businessCode);
        if (workflowProcessVo == null) {
            return ResponseUtil.dtoFail("请传入正确的流程标识码");
        }

        String taskId = approveRecordInfo.getTaskId();

        TaskOperateRespVO taskOperateRespVO = workFlowBusiness.getTaskSupportOperate(taskId, PageTypeEnum.REQUEST);
        Boolean showEdit = taskOperateRespVO.getShowEdit();
        if (!showEdit) {
            return ResponseUtil.dtoFail("当前节点不支持编辑操作");
        }

        ApprovalInfoUpdateReqVO approvalInfoUpdateReqVO = new ApprovalInfoUpdateReqVO();
        ProcessDetailRespVO processDetailRespVO = ProcessDetailRespVO.paseByWorkflowProcessVo(workflowProcessVo);
        String processTypeCode = processDetailRespVO.getProcessTypeCode();
        approvalInfoUpdateReqVO.setProcessType(processTypeCode);

        @Valid @NotNull ResourceInfoReqVO resourceInfo = approvalInfoAuthUpdateReqVO.getResourceInfo();

        BeanUtil.copyProperties(approvalInfoAuthUpdateReqVO, approvalInfoUpdateReqVO);
        approvalInfoUpdateReqVO.setAppCode(appCode);
        approvalInfoUpdateReqVO.setId(approveRecordInfoId);
        approvalInfoUpdateReqVO.setResourceInfoDTO(resourceInfo);
        approvalInfoUpdateReqVO.setProcessInstanceId(approveRecordInfo.getProcessInstanceId());
        // 默认设置为保存并提交
        approvalInfoUpdateReqVO.setOperateType(CommitTypeEnum.APPLY_COMMIT.getValue());
        ResponseDTO<String> updateStatus = approveService.update(approvalInfoUpdateReqVO);
        if (ResponseUtil.isSuccess(updateStatus)) {
            return ResponseStatus.success();
        } else {
            return ResponseStatus.fail(updateStatus.getMessage());
        }
    }

    @Override
    public ApiResult approvesSearch(ApiSearchReq apiSearchReq) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ApiResult.setFailMessage("请传入凭据");
        }

        Map<String, SearchOperator[]> requestMap = new HashMap<>(6);
        requestMap.put("processTitle", new SearchOperator[]{SearchOperator.eq, SearchOperator.like});
        requestMap.put("businessCode", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("processType", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("flowType", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("approveNo", new SearchOperator[]{SearchOperator.eq, SearchOperator.in});
        requestMap.put("applicantIdcard", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("createTime", new SearchOperator[]{SearchOperator.eq, SearchOperator.ge, SearchOperator.le,
                SearchOperator.rangeDate});

        Searchable searchable;

        try {
            searchable = SearchableUtil.parseApiSearchReqToSearchable(apiSearchReq, requestMap, 1000);
        } catch (IllegalArgumentException e) {
            String message = e.getMessage();
            return ApiResult.setFailMessage(message);
        }

        // 默认只能查询自己应用的信息
        searchable.addSearchFilter("appCode", SearchOperator.eq, appCode);

        Page<ApprovalInfoRespVO> approvalInfoVos = approveRecordService.queryUserApproveList(searchable);

        List<ApprovalInfoV2RespVO> approvalInfoV2RespVOList = new ArrayList<>(approvalInfoVos.getSize());
        for (ApprovalInfoRespVO approvalInfoVo : approvalInfoVos) {
            ApprovalInfoV2RespVO approvalInfoV2RespVO = new ApprovalInfoV2RespVO();
            BeanUtil.copyProperties(approvalInfoVo, approvalInfoV2RespVO);
            approvalInfoV2RespVOList.add(approvalInfoV2RespVO);
        }
        return ApiResult.setSuccessPage(approvalInfoVos.getTotalElements(), approvalInfoV2RespVOList);
    }

    @Override
    public ResponseDTO<ApprovalInfoV2RespVO> approveDetail(String approveNo) {

        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);
        if (approveRecordInfo == null) {
            return ResponseUtil.dtoFail("申请表单不存在");
        }

        ApprovalInfoDTO approvalInfoDTO = approveRecordInfo.toApprovalInfo();
        ApprovalInfoRespVO approvalInfoRespVO = approveBusiness.fillTaskInfoToApplyInfo(approvalInfoDTO);

        ApprovalInfoV2RespVO approvalInfoV2RespVO = new ApprovalInfoV2RespVO();
        BeanUtil.copyProperties(approvalInfoRespVO, approvalInfoV2RespVO);
        return ResponseUtil.newInstance(approvalInfoV2RespVO);
    }

    @Override
    public ResponseStatus recallApprove(String approveNo) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseStatus.fail("请传入凭据");
        }
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseStatus.fail("审批单号不存在");
        }

        String applicantIdcard = approveRecordInfo.getApplicantIdcard();
        if (!idcard.equals(applicantIdcard)) {
            return ResponseStatus.fail("非本人申请审批单无法操作");
        }

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseStatus.fail("非本应用申请审批单无法操作");
        }

        String taskId = approveRecordInfo.getTaskId();
        String processInstanceId = approveRecordInfo.getProcessInstanceId();

        TaskRequestUserInfoDTO taskRequestUserInfoDTO = new TaskRequestUserInfoDTO();
        taskRequestUserInfoDTO.setIdCard(idcard);

        TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();


        OperateReqDTO operateReqDTO = OperateReqDTO.builder()
                .taskId(taskId)
                .processInstanceId(processInstanceId)
                .approveWorkFlowOperationEnum(ApproveWorkFlowOperationEnum.RECALL)
                .taskRequest(taskRequest)
                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                .activitiHolderList(Collections.emptyList())
                .needCheckRedList(false)
                .build();

        ResponseDTO<OperateRespVO> responseDTO = approveService.approveOperate(operateReqDTO);
        if (ResponseUtil.isFail(responseDTO)) {
            return ResponseStatus.fail(responseDTO.getMessage());
        }
        return ResponseStatus.success();
    }

    @Override
    public ResponseStatus deleteApprove(String approveNo) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseStatus.fail("请传入凭据");
        }
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseStatus.fail("审批单号不存在");
        }

        String applicantIdcard = approveRecordInfo.getApplicantIdcard();
        if (!idcard.equals(applicantIdcard)) {
            return ResponseStatus.fail("非本人申请审批单无法操作");
        }

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseStatus.fail("非本应用申请审批单无法操作");
        }

        String id = approveRecordInfo.getId();

        return approveService.deleteProcessInstance(id);
    }

    @Override
    public ResponseDTO<List<ApproveCommentVo>> approveComments(String approveNo) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseUtil.dtoFail("请传入凭据");
        }

        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseUtil.dtoFail("审批单号不存在");
        }

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseUtil.dtoFail("非本应用申请审批单无法操作");
        }

        String processInstanceId = approveRecordInfo.getProcessInstanceId();
        return approveService.approveComments(processInstanceId);
    }

    @Override
    public ResponseDTO<TraceProcessAndOverVo> processPic(String approveNo) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseUtil.dtoFail("请传入凭据");
        }

        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseUtil.dtoFail("审批单号不存在");
        }

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseUtil.dtoFail("非本应用申请审批单无法操作");
        }

        String processInstanceId = approveRecordInfo.getProcessInstanceId();
        TraceProcessAndOverVo traceProcessAndOverVo = workFlowBusiness.processPic(processInstanceId);
        return ResponseUtil.newInstance(traceProcessAndOverVo);
    }

    @Override
    public ResponseDTO<ActivityNodeInfoVo> processPicDetail(String approveNo, String taskDefinitionCode, String statusType) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseUtil.dtoFail("请传入凭据");
        }

        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseUtil.dtoFail("审批单号不存在");
        }

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseUtil.dtoFail("非本应用申请审批单无法操作");
        }
        String processInstanceId = approveRecordInfo.getProcessInstanceId();
        ActivityNodeInfoVo activityNodeInfoVo = workFlowBusiness.processPicDetail(processInstanceId, statusType, taskDefinitionCode);
        return ResponseUtil.newInstance(activityNodeInfoVo);
    }

    @Override
    public ResponseDTO<List<ProcessStatusVo>> processStatus(String approveNo) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseUtil.dtoFail("请传入凭据");
        }

        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseUtil.dtoFail("审批单号不存在");
        }

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseUtil.dtoFail("非本应用申请审批单无法操作");
        }

        String processInstanceId = approveRecordInfo.getProcessInstanceId();

        return approveService.approveStatus(processInstanceId);
    }

    @Override
    public ResponseDTO<TaskOperateRespVO> approveOperateType(String approveNo, String pageType) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseUtil.dtoFail("请传入凭据");
        }

        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseUtil.dtoFail("审批单号不存在");
        }

        String taskId = approveRecordInfo.getTaskId();
        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseUtil.dtoFail("非本应用申请审批单无法操作");
        }
        PageTypeEnum pageTypeEnum = EnumUtils.enumOf(PageTypeEnum.class, pageType);
        if (pageTypeEnum == null) {
            return ResponseUtil.dtoFail("请传入正确的页面类型");
        }

        TaskOperateRespVO taskOperateRespVO = workFlowBusiness.getTaskSupportOperate(taskId, pageTypeEnum);
        return ResponseUtil.newInstance(taskOperateRespVO);
    }

    @Override
    public ResponseDTO<OperateRespVO> operate(ApproveOperateV2ReqVO approveOperateV2ReqVO) {
        log.info("approveOperateReqVO req:{}", JsonUtils.toJSONString(approveOperateV2ReqVO));
        TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();
        ApproveMessage approveMessage = new ApproveMessage();
        approveMessage.setMessage(approveOperateV2ReqVO.getMessage());
        approveMessage.setApproveState(approveOperateV2ReqVO.getOperateType());
        taskRequest.setApproveMessage(approveMessage);

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();

        log.info("BaseSecurityUser req:{}", JsonUtils.toJSONString(currentUser));
        ApproveWorkFlowOperationEnum flowOperationEnum = EnumUtils.enumOf(ApproveWorkFlowOperationEnum.class, approveOperateV2ReqVO.getOperateType());

        String visitResourceCode = approveOperateV2ReqVO.getVisitResourceCode();
        // 访问资源为空，且为同意
        if (StrUtil.isBlank(visitResourceCode)
                && flowOperationEnum.equals(ApproveWorkFlowOperationEnum.COMPLETE_AGREE)) {
            return ResponseUtil.dtoFail("同意时，必须有一个访问资源");
        }
        String visitResourceName = approveOperateV2ReqVO.getVisitResourceName();
        if (StrUtil.isNotBlank(visitResourceCode) && StrUtil.isBlank(visitResourceName)) {
            return ResponseUtil.dtoFail("visitResourceCode不为空时，visitResourceName不允许为空");
        }

        String disagreeVisitResourceCode = approveOperateV2ReqVO.getDisagreeVisitResourceCode();
        String disagreeVisitResourceName = approveOperateV2ReqVO.getDisagreeVisitResourceName();
        if (StrUtil.isNotBlank(disagreeVisitResourceCode) && StrUtil.isBlank(disagreeVisitResourceName)) {
            return ResponseUtil.dtoFail("disagreeVisitResourceCode为空时disagreeVisitResourceName不允许为空");
        }
        if (StrUtil.isNotBlank(disagreeVisitResourceName) && StrUtil.isBlank(disagreeVisitResourceCode)) {
            return ResponseUtil.dtoFail("disagreeVisitResourceName为空时disagreeVisitResourceCode不允许为空");
        }

        TaskRequestUserInfoDTO taskRequestUserInfoDTO = TaskRequestUserInfoDTO.parseBaseSecurityUser(currentUser);
        taskRequestUserInfoDTO.setIdCard(approveOperateV2ReqVO.getIdCard());
        taskRequestUserInfoDTO.setUserName(approveOperateV2ReqVO.getUserName());


        OperateReqDTO operateReqDTO = OperateReqDTO.builder()
                .taskId(approveOperateV2ReqVO.getTaskId())
                .processInstanceId(approveOperateV2ReqVO.getProcessInstanceId())
                .approveWorkFlowOperationEnum(flowOperationEnum)
                .taskRequest(taskRequest)
                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                .activitiHolderList(approveOperateV2ReqVO.getActivitiHolderList())
                .needCheckRedList(false)
                .visitResourceCode(visitResourceCode)
                .visitResourceName(visitResourceName)
                .disagreeVisitResourceCode(disagreeVisitResourceCode)
                .disagreeVisitResourceName(disagreeVisitResourceName)
                .build();

        return approveService.approveOperate(operateReqDTO);
    }


    //TODO add wanghao   挂起 suspend
    @Override
    public ResponseStatus suspendApprove(String approveNo) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseStatus.fail("请传入凭据");
        }
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseStatus.fail("审批单号不存在");
        }

        String applicantIdcard = approveRecordInfo.getApplicantIdcard();
        if (!idcard.equals(applicantIdcard)) {
            return ResponseStatus.fail("非本人申请审批单无法操作");
        }

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseStatus.fail("非本应用申请审批单无法操作");
        }

        String taskId = approveRecordInfo.getTaskId();
        String processInstanceId = approveRecordInfo.getProcessInstanceId();

        TaskRequestUserInfoDTO taskRequestUserInfoDTO = new TaskRequestUserInfoDTO();
        taskRequestUserInfoDTO.setIdCard(idcard);

        TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();


        OperateReqDTO operateReqDTO = OperateReqDTO.builder()
                .taskId(taskId)
                .processInstanceId(processInstanceId)
                .approveWorkFlowOperationEnum(ApproveWorkFlowOperationEnum.SUSPEND)
                .taskRequest(taskRequest)
                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                .activitiHolderList(Collections.emptyList())
                .needCheckRedList(false)
                .build();

        ResponseDTO<OperateRespVO> responseDTO = approveService.approveOperate(operateReqDTO);
        if (ResponseUtil.isFail(responseDTO)) {
            return ResponseStatus.fail(responseDTO.getMessage());
        }
        return ResponseStatus.success();
    }


    //TODO add wanghao   终止 redo
    @Override
    public ResponseStatus redoApprove(String approveNo) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseStatus.fail("请传入凭据");
        }
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseStatus.fail("审批单号不存在");
        }

        String applicantIdcard = approveRecordInfo.getApplicantIdcard();
        if (!idcard.equals(applicantIdcard)) {
            return ResponseStatus.fail("非本人申请审批单无法操作");
        }

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseStatus.fail("非本应用申请审批单无法操作");
        }

        String taskId = approveRecordInfo.getTaskId();
        String processInstanceId = approveRecordInfo.getProcessInstanceId();

        TaskRequestUserInfoDTO taskRequestUserInfoDTO = new TaskRequestUserInfoDTO();
        taskRequestUserInfoDTO.setIdCard(idcard);

        TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();


        OperateReqDTO operateReqDTO = OperateReqDTO.builder()
                .taskId(taskId)
                .processInstanceId(processInstanceId)
                .approveWorkFlowOperationEnum(ApproveWorkFlowOperationEnum.REDO)
                .taskRequest(taskRequest)
                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                .activitiHolderList(Collections.emptyList())
                .needCheckRedList(false)
                .build();

        ResponseDTO<OperateRespVO> responseDTO = approveService.approveOperate(operateReqDTO);
        if (ResponseUtil.isFail(responseDTO)) {
            return ResponseStatus.fail(responseDTO.getMessage());
        }
        return ResponseStatus.success();
    }


    //TODO add wanghao   激活 activate
    @Override
    public ResponseStatus activateApprove(String approveNo) {

        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();
        if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
            return ResponseStatus.fail("请传入凭据");
        }
        ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);

        if (approveRecordInfo == null) {
            return ResponseStatus.fail("审批单号不存在");
        }

        String applicantIdcard = approveRecordInfo.getApplicantIdcard();
        if (!idcard.equals(applicantIdcard)) {
            return ResponseStatus.fail("非本人申请审批单无法操作");
        }

        String recordInfoAppCode = approveRecordInfo.getAppCode();
        if (!appCode.equals(recordInfoAppCode)) {
            return ResponseStatus.fail("非本应用申请审批单无法操作");
        }

        String taskId = approveRecordInfo.getTaskId();
        String processInstanceId = approveRecordInfo.getProcessInstanceId();

        TaskRequestUserInfoDTO taskRequestUserInfoDTO = new TaskRequestUserInfoDTO();
        taskRequestUserInfoDTO.setIdCard(idcard);

        TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();


        OperateReqDTO operateReqDTO = OperateReqDTO.builder()
                .taskId(taskId)
                .processInstanceId(processInstanceId)
                .approveWorkFlowOperationEnum(ApproveWorkFlowOperationEnum.ACTIVATE)
                .taskRequest(taskRequest)
                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                .activitiHolderList(Collections.emptyList())
                .needCheckRedList(false)
                .build();

        ResponseDTO<OperateRespVO> responseDTO = approveService.approveOperate(operateReqDTO);
        if (ResponseUtil.isFail(responseDTO)) {
            return ResponseStatus.fail(responseDTO.getMessage());
        }
        return ResponseStatus.success();
    }
}
