package com.dragonsoft.dcuc.approve.facade.api.v3;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.dragoninfo.dcuc.common.utils.LangUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.duceap.enums.ManageDisposeTypeEnum;
import com.dragonsoft.dcuc.approve.business.IApproveCommonBusiness;
import com.dragonsoft.dcuc.approve.business.ISecurityUserBusiness;
import com.dragonsoft.dcuc.approve.business.IWorkFlowBusiness;
import com.dragonsoft.dcuc.approve.business.api.IApiV3ApproveBusiness;
import com.dragonsoft.dcuc.approve.business.api.IApproveApiBusiness;
import com.dragonsoft.dcuc.approve.business.api.IReportApprovalsBusiness;
import com.dragonsoft.dcuc.approve.business.external.IBimBusiness;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.ZeroTrustApproveManageEnum;
import com.dragonsoft.dcuc.approve.enumresources.ZeroTrustBusinessRespEnum;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.ApproveTaskClass;
import com.dragonsoft.dcuc.approve.model.dto.OperateReqDTO;
import com.dragonsoft.dcuc.approve.model.dto.TaskRequestUserInfoDTO;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoReqVO;
import com.dragonsoft.dcuc.approve.model.req.v3.ZeroTrustApproveCreateReqV3VO;
import com.dragonsoft.dcuc.approve.model.req.v3.ZeroTrustApproveManagerReqV3VO;
import com.dragonsoft.dcuc.approve.model.req.v3.report.ReportApproveSyncReqVO;
import com.dragonsoft.dcuc.approve.model.resp.OperateRespVO;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustAppItemUserTokenInfoRespVO;
import com.dragonsoft.dcuc.approve.model.resp.bim.ZeroTrustAppTokenInfoRespVO;
import com.dragonsoft.dcuc.approve.model.resp.v3.ZeroTrustApproveCreateV3RespVO;
import com.dragonsoft.dcuc.approve.model.resp.v3.ZeroTrustMessageRespVO;
import com.dragonsoft.dcuc.approve.model.resp.v3.ZeroTrustResultDataVO;
import com.dragonsoft.dcuc.approve.model.vo.ApproveTaskRequestVo;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.dcuc.approve.service.IApproveTaskClassService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.duwf.api.model.TaskRequest;
import com.dragonsoft.duceap.duwf.api.model.WorkflowProcessVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/5/22
 */
@Slf4j
@RestController
public class ApiV3ApproveFacade implements IApiV3ApproveFacade {

    private IApproveCommonBusiness approveService;

    private IApproveRecordService approveRecordService;

    private IWorkFlowBusiness workFlowBusiness;

    private IBimBusiness bimBusiness;

    private IApproveApiBusiness approveApiBusiness;

    private IApproveTaskClassService approveTaskClassService;

    private IApiV3ApproveBusiness apiV3ApproveBusiness;

    private ISecurityUserBusiness securityUserBusiness;

    private IReportApprovalsBusiness reportApprovalsBusiness;

    @Autowired
    public void setReportApprovalsBusiness(IReportApprovalsBusiness reportApprovalsBusiness) {
        this.reportApprovalsBusiness = reportApprovalsBusiness;
    }

    @Autowired
    public void setSecurityUserBusiness(ISecurityUserBusiness securityUserBusiness) {
        this.securityUserBusiness = securityUserBusiness;
    }

    @Autowired
    public void setApiV3ApproveBusiness(IApiV3ApproveBusiness apiV3ApproveBusiness) {
        this.apiV3ApproveBusiness = apiV3ApproveBusiness;
    }

    @Autowired
    public void setApproveTaskClassService(IApproveTaskClassService approveTaskClassService) {
        this.approveTaskClassService = approveTaskClassService;
    }

    @Autowired
    public void setApproveApiBusiness(IApproveApiBusiness approveApiBusiness) {
        this.approveApiBusiness = approveApiBusiness;
    }

    @Autowired
    public void setApproveService(IApproveCommonBusiness approveService) {
        this.approveService = approveService;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Autowired
    public void setWorkFlowBusiness(IWorkFlowBusiness workFlowBusiness) {
        this.workFlowBusiness = workFlowBusiness;
    }

    @Autowired
    public void setBimBusiness(IBimBusiness bimBusiness) {
        this.bimBusiness = bimBusiness;
    }

    @Override
    public ZeroTrustResultDataVO<ZeroTrustApproveCreateV3RespVO> create(ZeroTrustApproveCreateReqV3VO zeroTrustApproveCreateReqV3VO) {
        String appTokenId = zeroTrustApproveCreateReqV3VO.getAppTokenId();
        String processDefId = zeroTrustApproveCreateReqV3VO.getProcessDefId();
        String title = zeroTrustApproveCreateReqV3VO.getTitle();
        String approveTaskClassCode = zeroTrustApproveCreateReqV3VO.getTaskClass();
        String approveTaskId = zeroTrustApproveCreateReqV3VO.getTaskId();

        String callbackUrl = zeroTrustApproveCreateReqV3VO.getCallbackUrl();
        String callerSign = zeroTrustApproveCreateReqV3VO.getCallerSign();
        // 校验必填参数
        if (LangUtil.isAnyBlank(appTokenId, processDefId, title, approveTaskClassCode, approveTaskId, callbackUrl, callerSign)) {
            return ZeroTrustResultDataVO.operateFailMessage("请传入必填参数");
        }
        //  验证签名
        boolean checkCallerSign = approveApiBusiness.checkCallerSign(zeroTrustApproveCreateReqV3VO);
        if (!checkCallerSign) {
            return ZeroTrustResultDataVO.operateFailMessage("验证签名出错");
        }

        if (!approveApiBusiness.isStanderTaskNo(approveTaskId)) {
            return ZeroTrustResultDataVO.operateFailMessage("taskId格式出错");
        }
        Map<String, Object> bizData = zeroTrustApproveCreateReqV3VO.getBizData();
        ResponseStatus checkBizDataStatus = apiV3ApproveBusiness.checkBizData(bizData);
        if (ResponseUtil.isFail(checkBizDataStatus)) {
            return ZeroTrustResultDataVO.operateFailMessage("bizData中key值：" + checkBizDataStatus.getMessage());
        }

        //  查询应用令牌信息,获取人员信息
        ZeroTrustAppTokenInfoRespVO appTokenInfo = bimBusiness.getAppTokenInfo(appTokenId);
        if (ObjectUtil.isNull(appTokenInfo)) {
            return ZeroTrustResultDataVO.resultEnumMessage(ZeroTrustBusinessRespEnum.TOKEN_FAIL);
        }
        ZeroTrustAppItemUserTokenInfoRespVO userToken = appTokenInfo.getUserToken();
        String appCode = appTokenInfo.getAppId();
        String applicantIdcard = userToken.getPid();
        ResponseDTO<SecurityUser> securityUserResp = securityUserBusiness.getSecurityUser(applicantIdcard, appCode);
        if (ResponseUtil.isFail(securityUserResp)) {
            log.error("令牌信息异常：{}", securityUserResp.getMessage());
            return ZeroTrustResultDataVO.operateFailMessage("令牌信息异常");
        }

        SecurityUser securityUser = ResponseUtil.getResult(securityUserResp);
        securityUserBusiness.setUserExtInfo(securityUser, appTokenInfo);
        UserContextUtils.setCurrentUser(securityUser);

        // 获取processType并校验
        WorkflowProcessVo workflowProcessVo = workFlowBusiness.queryWorkflowProcessByBusinessCode(processDefId);
        if (ObjectUtil.isNull(workflowProcessVo)) {
            return ZeroTrustResultDataVO.operateFailMessage("流程标识不存在");
        }
        String processType = workflowProcessVo.getWorkflowProcesstypeVo().getProcessTypeCode();
        ApproveRecordInfo selectApproveRecordInfo = approveRecordService.selectByApproveNo(approveTaskId);
        if (ObjectUtil.isNotNull(selectApproveRecordInfo)) {
            return ZeroTrustResultDataVO.operateFailMessage("TaskId已存在");
        }

        ApproveTaskClass approveTaskClass = approveTaskClassService.detailByCode(approveTaskClassCode);
        if (ObjectUtil.isNull(approveTaskClass)) {
            return ZeroTrustResultDataVO.operateFailMessage("taskClass不存在");
        }

        final ResponseDTO<ApprovalInfoReqVO> approvalInfoReqStatus = apiV3ApproveBusiness.parseToReqVO(zeroTrustApproveCreateReqV3VO, appTokenInfo, processType);
        if (ResponseUtil.isFail(approvalInfoReqStatus)) {
            return ZeroTrustResultDataVO.operateFailMessage(approvalInfoReqStatus.getMessage());
        }
        final ApprovalInfoReqVO approvalInfoReqVO = ResponseUtil.getResult(approvalInfoReqStatus);
        ApproveRecordInfo approveRecordInfo = approveService.saveBase(approvalInfoReqVO, ManageDisposeTypeEnum.INSERT, new Date());
        String processInstanceId = approveRecordInfo.getProcessInstanceId();
        ZeroTrustApproveCreateV3RespVO zeroTrustApproveCreateV3RespVO = new ZeroTrustApproveCreateV3RespVO();
        zeroTrustApproveCreateV3RespVO.setProcessInstId(processInstanceId);
        return ZeroTrustResultDataVO.success(zeroTrustApproveCreateV3RespVO);
    }
    
    public static Map<String, String> statusMap = new HashMap<>();

    @Override
    public ZeroTrustMessageRespVO manage(ZeroTrustApproveManagerReqV3VO zeroTrustApproveManagerReqV3VO) {
        System.err.println(JSONObject.toJSONString(zeroTrustApproveManagerReqV3VO));


        String appTokenId = zeroTrustApproveManagerReqV3VO.getAppTokenId();
        String processInstId = zeroTrustApproveManagerReqV3VO.getProcessInstId();   //流程实例Id
        String actionType = zeroTrustApproveManagerReqV3VO.getActionType();

        if (LangUtil.isAnyBlank(appTokenId, processInstId, actionType)) {
            return ZeroTrustResultDataVO.operateFailMessage("请传入必填参数");
        }

//        ZeroTrustApproveManageEnum zeroTrustApproveManageEnum = EnumUtils.enumOf(ZeroTrustApproveManageEnum.class, actionType);
//        if (ObjectUtil.isNull(zeroTrustApproveManageEnum)) {
//            return ZeroTrustResultDataVO.operateFailMessage("actionType,参数不正确");
//        }
//        ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum = zeroTrustApproveManageEnum.toApproveWorkFlowOperationEnum();
//        if (ObjectUtil.isNull(approveWorkFlowOperationEnum)) {
//            return ZeroTrustResultDataVO.operateFailMessage("暂不支持该操作");
//        }


        //TODO wanghao  注释掉以前V3 假的 挂起 激活 终止
        /*

        System.out.println("======================================20240909=========================");
        if(statusMap.get(processInstId) == null) {
        	statusMap.put(processInstId, actionType);
        	return ZeroTrustMessageRespVO.messageEnumMessage(ZeroTrustBusinessRespEnum.SUCCESS);
        }else {
        	String oldStatus = statusMap.get(processInstId);
        	
        	if("01".equals(actionType)) {//
        		
        		if("01".equals(oldStatus)) {//撤销
        			return ZeroTrustResultDataVO.operateFailMessage("已经是撤销状态无法操作");
            	}else if("02".equals(oldStatus)){//挂起
                	statusMap.put(processInstId, actionType);
            		return ZeroTrustMessageRespVO.messageEnumMessage(ZeroTrustBusinessRespEnum.SUCCESS);
            	}else if("03".equals(oldStatus)){//终止
            		return ZeroTrustResultDataVO.operateFailMessage("终止状态下无法执行撤销操作");
            	}else if("04".equals(oldStatus)){//激活
            		statusMap.put(processInstId, actionType);
            		return ZeroTrustMessageRespVO.messageEnumMessage(ZeroTrustBusinessRespEnum.SUCCESS);
            	}
        		
        		
        		
        	}else if("02".equals(actionType)) {//挂起
        		if("01".equals(oldStatus)) {//chexiao
        			return ZeroTrustResultDataVO.operateFailMessage("撤销状态下无法执行挂起操作");
            	}else if("02".equals(oldStatus)){//挂起
            		return ZeroTrustResultDataVO.operateFailMessage("已经是挂起状态无法操作");
            	}else if("03".equals(oldStatus)){//终止
            		return ZeroTrustResultDataVO.operateFailMessage("终止状态下无法执行挂起操作");
            	}else if("04".equals(oldStatus)){//激活
            		statusMap.put(processInstId, actionType);
            		return ZeroTrustMessageRespVO.messageEnumMessage(ZeroTrustBusinessRespEnum.SUCCESS);
            	}
        	}else if("03".equals(actionType)) {//终止
        		if("01".equals(oldStatus)) {//chexiao
        			statusMap.put(processInstId, actionType);
            		return ZeroTrustMessageRespVO.messageEnumMessage(ZeroTrustBusinessRespEnum.SUCCESS);
            	}else if("02".equals(oldStatus)){//挂起
            		statusMap.put(processInstId, actionType);
            		return ZeroTrustMessageRespVO.messageEnumMessage(ZeroTrustBusinessRespEnum.SUCCESS);
            	}else if("03".equals(oldStatus)){//终止
            		return ZeroTrustResultDataVO.operateFailMessage("已经是终止状态无法操作");
            	}else if("04".equals(oldStatus)){//激活
            		statusMap.put(processInstId, actionType);
            		return ZeroTrustMessageRespVO.messageEnumMessage(ZeroTrustBusinessRespEnum.SUCCESS);
            	}
        	}else if("04".equals(actionType)) {//激活
        		if("01".equals(oldStatus)) {//chexiao
        			return ZeroTrustResultDataVO.operateFailMessage("撤销状态下无法执行激活操作");
            	}else if("02".equals(oldStatus)){//挂起
            		statusMap.put(processInstId, actionType);
            		return ZeroTrustMessageRespVO.messageEnumMessage(ZeroTrustBusinessRespEnum.SUCCESS);
            	}else if("03".equals(oldStatus)){//终止
            		return ZeroTrustResultDataVO.operateFailMessage("终止状态下无法执行激活操作");
            	}else if("04".equals(oldStatus)){//激活
            		return ZeroTrustResultDataVO.operateFailMessage("已经是激活状态无法操作");
            	}
        	}
        	
        	
        	
        }
        
        System.out.println("======================================20240909=========================");

        */

        //TODO add  wanghao
        ZeroTrustApproveManageEnum zeroTrustApproveManageEnum = EnumUtils.enumOf(ZeroTrustApproveManageEnum.class, actionType);
        if (ObjectUtil.isNull(zeroTrustApproveManageEnum)) {
            return ZeroTrustResultDataVO.operateFailMessage("actionType,参数不正确");
        }
        ApproveWorkFlowOperationEnum approveWorkFlowOperationEnum = zeroTrustApproveManageEnum.toApproveWorkFlowOperationEnum();
        if (ObjectUtil.isNull(approveWorkFlowOperationEnum)) {
            return ZeroTrustResultDataVO.operateFailMessage("暂不支持该操作");
        }


        //  查询应用令牌信息,获取人员信息------------自测 V3api,需注释代码，测完 打开   wanghao  start

        ZeroTrustAppTokenInfoRespVO appTokenInfo = bimBusiness.getAppTokenInfo(appTokenId);
        if (ObjectUtil.isNull(appTokenInfo)) {
            return ZeroTrustResultDataVO.resultEnumMessage(ZeroTrustBusinessRespEnum.TOKEN_FAIL);
        }
        ZeroTrustAppItemUserTokenInfoRespVO userToken = appTokenInfo.getUserToken();



        String appCode = appTokenInfo.getAppId();
        String applicantIdcard = userToken.getPid();

        ResponseDTO<SecurityUser> securityUserResp = securityUserBusiness.getSecurityUser(applicantIdcard, appCode);
        if (ResponseUtil.isFail(securityUserResp)) {
            log.error("令牌信息异常：{}", securityUserResp.getMessage());
            return ZeroTrustResultDataVO.operateFailMessage("令牌信息异常");
        }

        SecurityUser securityUser = ResponseUtil.getResult(securityUserResp);
        securityUserBusiness.setUserExtInfo(securityUser, appTokenInfo);
        UserContextUtils.setCurrentUser(securityUser);
        String idcard = userToken.getPid();

        // 以上都是用户 令牌--用户信息   end
        //  查询应用令牌信息,获取人员信息-------------自测 V3api,需注释代码,测完 打开 wanghao  end




        //TODO add wanghao  自测 v3api添加代码，  使用测试用户，测完注释掉
//        SecurityUser securityUser = UserContextUtils.getCurrentUser();
//        String idcard = securityUser.getIdcard();



        ApproveRecordInfo approveRecordInfo = approveRecordService.findByProcessInstanceId(processInstId);      //根据实例id、应用代码 获取审批信息   流程实例Id
        if (ObjectUtil.isNull(approveRecordInfo)) {
            return ZeroTrustResultDataVO.operateFailMessage("流程实例不存在");
        }
        String currentFlowTaskId = approveRecordInfo.getTaskId();

        TaskRequestUserInfoDTO taskRequestUserInfoDTO = new TaskRequestUserInfoDTO();
        taskRequestUserInfoDTO.setIdCard(idcard);

        TaskRequest<ApproveTaskRequestVo> taskRequest = new TaskRequest<>();

        OperateReqDTO operateReqDTO = OperateReqDTO.builder()
                .taskId(currentFlowTaskId)
                .processInstanceId(processInstId)
//                .approveWorkFlowOperationEnum(ApproveWorkFlowOperationEnum.RECALL)
                .approveWorkFlowOperationEnum(approveWorkFlowOperationEnum)      //TODO add wanghao   改为实际的操作动作
                .taskRequest(taskRequest)
                .taskRequestUserInfoDTO(taskRequestUserInfoDTO)
                .activitiHolderList(Collections.emptyList())
                .needCheckRedList(false)
                .build();


        ResponseDTO<OperateRespVO> responseDTO = approveService.approveOperate(operateReqDTO);
        if (ResponseUtil.isFail(responseDTO)) {
            return ZeroTrustResultDataVO.operateFailMessage(responseDTO.getMessage());
        }

        return ZeroTrustMessageRespVO.messageEnumMessage(ZeroTrustBusinessRespEnum.SUCCESS);
    }

    @Override
    public ZeroTrustMessageRespVO reportApprovals(ReportApproveSyncReqVO reportApproveSyncReqVO) {
        String jsonString = JsonUtils.toJSONString(reportApproveSyncReqVO);
        log.info("接收到应用上报的数据:{}", jsonString);
        return reportApprovalsBusiness.reportApprovals(reportApproveSyncReqVO);
    }

}
