package com.dragonsoft.dcuc.approve.model;

import com.baomidou.mybatisplus.annotation.*;
import com.dragonsoft.duceap.base.annotations.audit.DeletedBy;
import com.dragonsoft.duceap.base.annotations.audit.DeletedDate;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import org.apache.ibatis.type.JdbcType;
import org.springframework.data.annotation.LastModifiedBy;
import org.springframework.data.annotation.LastModifiedDate;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 审批流通信息
 *
 * @Description
 * @Author wangzw
 * @Date 2021-05-26
 */
@Data
@TableName(value = "t_approve_circulation_info")
public class ApproveCirculationInfo {

    @TableField(value = "ID")
    @TableId(type = IdType.UUID)
    private String id;

    /**
     * 审批单信息记录id
     */
    private String approveRecordId;

    /**
     * 审批单编号
     */
    private String approveNo;

    /**
     * 流程编号
     */
    private String processNo;

    /**
     * 节点名称
     */
    private String nodeName;

    /**
     * 节点标识码
     */
    private String nodeNo;
    /**
     * 节点定义id
     */
    private String taskKey;
    /**
     * 任务id
     */
    private String taskId;

    /**
     * @see com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum
     * 操作类型
     */
    private String operateType;

    /**
     * 操作时间
     */
    private String operateTime;

    /**
     * 操作人姓名
     */
    private String operateUserName;

    /**
     * 操作人身份证号码
     */
    private String operateUserId;

    /**
     * 操作人单位名称
     */
    private String operateUserOrg;

    /**
     * 操作人单位编号
     */
    private String operateUserOrgId;

    /**
     * 审批意见
     */
    private String approveOpinion;

    /**
     * 操作耗时(秒为单位)
     */
    private String operateTimeConsuming;

    /**
     * 电子签名
     */
    private String electronicSignature;

    /**
     * 是否删除
     */
    @TableLogic
    private String isDeleted;

    /**
     * 创建人
     */
    private String createUser;

    /**
     * 创建时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private Date createTime;

    /**
     * 更新人
     */
    @LastModifiedBy
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private String updateUser;

    /**
     * 更新时间
     */
    @LastModifiedDate
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private Date updateTime;

    /**
     * 删除人
     */
    @DeletedBy
    private String deleteUser;

    /**
     * 删除时间
     */
    @DeletedDate
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private Date deleteTime;


    public String getApproveRecordId() {
        return this.approveRecordId;
    }

    public void setApproveRecordId(String approveRecordId) {
        this.approveRecordId = approveRecordId;
    }

    public String getApproveNo() {
        return this.approveNo;
    }

    public void setApproveNo(String approveNo) {
        this.approveNo = approveNo;
    }

    public String getProcessNo() {
        return this.processNo;
    }

    public void setProcessNo(String processNo) {
        this.processNo = processNo;
    }

    public String getNodeName() {
        return this.nodeName;
    }

    public void setNodeName(String nodeName) {
        this.nodeName = nodeName;
    }

    public String getNodeNo() {
        return this.nodeNo;
    }

    public void setNodeNo(String nodeNo) {
        this.nodeNo = nodeNo;
    }

    public String getOperateType() {
        return this.operateType;
    }

    public void setOperateType(String operateType) {
        this.operateType = operateType;
    }

    public String getOperateTime() {
        return this.operateTime;
    }

    public void setOperateTime(String operateTime) {
        this.operateTime = operateTime;
    }

    public String getOperateUserName() {
        return this.operateUserName;
    }

    public void setOperateUserName(String operateUserName) {
        this.operateUserName = operateUserName;
    }

    public String getOperateUserId() {
        return this.operateUserId;
    }

    public void setOperateUserId(String operateUserId) {
        this.operateUserId = operateUserId;
    }

    public String getOperateUserOrg() {
        return this.operateUserOrg;
    }

    public void setOperateUserOrg(String operateUserOrg) {
        this.operateUserOrg = operateUserOrg;
    }

    public String getOperateUserOrgId() {
        return this.operateUserOrgId;
    }

    public void setOperateUserOrgId(String operateUserOrgId) {
        this.operateUserOrgId = operateUserOrgId;
    }

    public String getApproveOpinion() {
        return this.approveOpinion;
    }

    public void setApproveOpinion(String approveOpinion) {
        this.approveOpinion = approveOpinion;
    }

    public String getOperateTimeConsuming() {
        return this.operateTimeConsuming;
    }

    public void setOperateTimeConsuming(String operateTimeConsuming) {
        this.operateTimeConsuming = operateTimeConsuming;
    }

    public String getElectronicSignature() {
        return this.electronicSignature;
    }

    public void setElectronicSignature(String electronicSignature) {
        this.electronicSignature = electronicSignature;
    }

    public String getIsDeleted() {
        return this.isDeleted;
    }

    public void setIsDeleted(String isDeleted) {
        this.isDeleted = isDeleted;
    }

    public String getCreateUser() {
        return this.createUser;
    }

    public void setCreateUser(String createUser) {
        this.createUser = createUser;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public String getUpdateUser() {
        return updateUser;
    }

    public void setUpdateUser(String updateUser) {
        this.updateUser = updateUser;
    }

    public Date getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(Date updateTime) {
        this.updateTime = updateTime;
    }

    public String getDeleteUser() {
        return deleteUser;
    }

    public void setDeleteUser(String deleteUser) {
        this.deleteUser = deleteUser;
    }

    public Date getDeleteTime() {
        return deleteTime;
    }

    public void setDeleteTime(Date deleteTime) {
        this.deleteTime = deleteTime;
    }

    public String getTaskKey() {
        return taskKey;
    }

    public void setTaskKey(String taskKey) {
        this.taskKey = taskKey;
    }
}
