package com.dragonsoft.dcuc.approve.model;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.*;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.ResourceInfoDTO;
import com.dragonsoft.dcuc.approve.enumresources.ApproveResultEnum;
import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.PermissionValidTypeEnum;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import org.apache.ibatis.type.JdbcType;
import org.springframework.beans.BeanUtils;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedDate;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.Map;


/**
 * @Description 审批单信息
 * @Author wangzw
 * @Date 2021-05-26
 */
@Data
@TableName(value = "t_approve_record_info")
public class ApproveRecordInfo {
    @TableField(value = "ID")
    @TableId(type = IdType.UUID)
    private String id;
    /**
     * 流程编号
     */
    private String businessCode;
    /**
     * 流程类型
     */
    private String processType;
    /**
     * 流程类型名称
     */
    private String processTypeName;

    /**
     * 流程名称
     */
    private String processName;

    /**
     * @see com.dragonsoft.dcuc.approve.enumresources.ApproveTypeEnum
     * 审批类型
     */
    private String approveType;

    /**
     * 审批单编号
     */
    private String approveNo;

    /**
     * @see FlowTypeEnum
     * 流程类型
     */
    private String flowType;

    /**
     * 审批单标题
     */
    private String processTitle;

    /**
     * 申请人身份证号码
     */
    private String applicantIdcard;

    /**
     * 申请人姓名
     */
    private String applicantName;

    /**
     * 申请人手机号码
     */
    private String applicantPhoneNo;

    /**
     * 申请人单位代码
     */
    private String applicantOrgCode;

    /**
     * 申请人单位名称
     */
    private String applicantOrgName;

    /**
     * 申请人IP
     */
    private String applicantIp;

    /**
     * 申请人应用令牌
     */
    private String applicantAppToken;

    /**
     * 申请时间
     */
    private String applyTime;

    /**
     * 应用名称
     */
    private String appName;

    /**
     * 应用编号
     */
    private String appCode;

    /**
     * 审批内容
     */
    private String approveContent;

    /**
     * 访问资源
     */
    private String accessResources;

    /**
     * 访问资源编号
     */
    private String accessResourcesNo;


    /**
     * 撤销访问资源
     */
    private String removeAccessResources;

    /**
     * 撤销访问资源编号
     */
    private String removeAccessResourcesNo;

    /**
     * 不同意访问资源
     */
    private String disagreeAccessResources;

    /**
     * 不同意访问资源编号
     */
    private String disagreeAccessResourcesNo;


    /**
     * @see PermissionValidTypeEnum
     * 有效期类别（自定义：01；长期：02；）
     */
    private String permissionValidType;

    /**
     * 有效时间起始
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private Date startTime;

    /**
     * 有效时间终止
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private Date endTime;

    /**
     * 申请事由
     */
    private String applyReason;

    /**
     * @see ApproveResultEnum
     * 最终审批结果
     */
    private String finalApprovalResult;

    /**
     * 附件地址
     */
    private String attachmentUrl;

    /**
     * 流程实例ID
     */
    private String processInstanceId;

    /**
     * 任务标识(Code)
     */
    private String taskCode;

    /**
     * 任务ID
     */
    private String taskId;

    /**
     * 任务名称
     */
    private String taskName;

    /**
     * 消息ID
     */
    private String messageId;

    /**
     * 其他申请信息
     */
    private String otherApplyContext;

    /**
     * 是否已经红名单审批
     */
    private String alreadyRedListApprove;

    /**
     * 是否删除
     */
    @TableLogic
    private String isDeleted;

    /**
     * 创建人
     */
    private String createUser;

    /**
     * 创建时间
     */
    @CreatedDate
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private Date createTime;

    /**
     * 更新人
     */
    private String updateUser;

    /**
     * 更新时间
     */
    @LastModifiedDate
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private Date updateTime;

    /**
     * 删除人
     */
    private String deleteUser;

    /**
     * 删除时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private Date deleteTime;

    /**
     * 签名
     */
    private String sign;

    /**
     * 回调通知地址
     */
    private String callbackUrl;

    /**
     * 任务类型
     */
    private String approveTaskClass;

    /**
     * 超期时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(jdbcType = JdbcType.TIMESTAMP)
    private Date expirationTime;

    /**
     * 同步标志
     *
     * @see com.dragonsoft.duceap.base.enums.BooleanEnum
     */
    private String syncFlag;


    /**
     * 将模式3新表单表构建成旧表单对象
     *
     * @return 旧表单对象
     */
    public ApprovalInfoDTO toApprovalInfo() {
        ApprovalInfoDTO approvalInfoDTO = new ApprovalInfoDTO();
        BeanUtils.copyProperties(this, approvalInfoDTO);
        approvalInfoDTO.setBusinessCode(this.getBusinessCode());
        approvalInfoDTO.setProcessInstanceId(this.getProcessInstanceId());
        approvalInfoDTO.setProcessTitle(this.getProcessTitle());
        approvalInfoDTO.setApplyReason(this.getApplyReason());
        approvalInfoDTO.setApplyContent(this.getApproveContent());
        approvalInfoDTO.setApproveTaskIdentifier(this.getTaskCode());
        approvalInfoDTO.setApproveResult(this.getFinalApprovalResult());
        approvalInfoDTO.setCallbackUrl(this.getCallbackUrl());
        approvalInfoDTO.setApproveTaskClass(this.getApproveTaskClass());

        ResourceInfoDTO resourceInfoVo = new ResourceInfoDTO();

        Map<String, String> codeMap = (Map) JSON.parse(this.getAccessResourcesNo());
        if (codeMap != null && !codeMap.isEmpty()) {
            resourceInfoVo.setVisitorCode(codeMap.entrySet().stream().findFirst().get().getKey());
            resourceInfoVo.setVisitResourceCode(codeMap.entrySet().stream().findFirst().get().getValue());
        }
        Map<String, String> nameMap = (Map) JSON.parse(this.getAccessResources());
        if (nameMap != null && !nameMap.isEmpty()) {
            resourceInfoVo.setVisitorName(nameMap.entrySet().stream().findFirst().get().getKey());
            resourceInfoVo.setVisitResourceName(nameMap.entrySet().stream().findFirst().get().getValue());
        }

        Map<String, String> removeCodeMap = (Map) JSON.parse(this.getRemoveAccessResourcesNo());
        if (removeCodeMap != null && !removeCodeMap.isEmpty()) {
            resourceInfoVo.setVisitorCode(removeCodeMap.entrySet().stream().findFirst().get().getKey());
            resourceInfoVo.setRemoveVisitResourceCode(removeCodeMap.entrySet().stream().findFirst().get().getValue());
        }
        Map<String, String> removeNameMap = (Map) JSON.parse(this.getRemoveAccessResources());
        if (removeNameMap != null && !removeNameMap.isEmpty()) {
            resourceInfoVo.setVisitorName(removeNameMap.entrySet().stream().findFirst().get().getKey());
            resourceInfoVo.setRemoveVisitResourceName(removeNameMap.entrySet().stream().findFirst().get().getValue());
        }

        Map<String, String> disagreeAccessResourcesNoMap = (Map) JSON.parse(this.getDisagreeAccessResourcesNo());
        if (CollUtil.isNotEmpty(disagreeAccessResourcesNoMap)) {
            resourceInfoVo.setDisagreeVisitResourceCode(disagreeAccessResourcesNoMap.entrySet().stream().findFirst().get().getValue());
        }

        Map<String, String> disagreeAccessResourcesMap = (Map) JSON.parse(this.getDisagreeAccessResources());
        if (CollUtil.isNotEmpty(disagreeAccessResourcesMap)) {
            resourceInfoVo.setDisagreeVisitResourceName(disagreeAccessResourcesMap.entrySet().stream().findFirst().get().getValue());
        }

        approvalInfoDTO.setResourceInfoVo(resourceInfoVo);
        approvalInfoDTO.setAppCode(this.getAppCode());
        return approvalInfoDTO;
    }

}
