package com.dragonsoft.dcuc.approve.model.dto;

import com.baomidou.mybatisplus.annotation.TableLogic;
import com.dragonsoft.dcuc.approve.enumresources.ApproveResultEnum;
import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;

/**
 * 旧审批表，新的审批表
 *
 * @author wangrs
 * @date 2021/02/04
 * @desc 流程类型实体
 * @see ApproveRecordInfo
 */
@Data
public class ApprovalInfoDTO implements Serializable {

    private static final long serialVersionUID = 5454155825318885342L;

    private String id;

    /**
     * 流程标题
     */
    private String processTitle;

    /**
     * 工作流流程类别
     */
    private String processType;

    /**
     * 工作流流程类别名称
     */
    private String processTypeName;

    /**
     * 工作流流程定义名称
     */
    private String processName;

    /**
     * 权限审批的流程类型（服务、服务撤回、功能、数据）
     *
     * @see FlowTypeEnum
     */
    private String flowType;

    /**
     * 工作流流程定义标识码
     */
    private String businessCode;

    /**
     * 申请理由
     */
    private String applyReason;

    /**
     * 申请内容
     */
    private String applyContent;

    /**
     * 申请人员身份证号
     */
    private String applicantIdcard;

    /**
     * 申请人员姓名
     */
    private String applicantName;

    /**
     * 申请人单位代码
     */
    private String applicantOrgCode;
    /**
     * 申请人单位名称
     */
    private String applicantOrgName;

    /**
     * 申请人联系方式
     */
    private String applicantPhoneNo;

    /**
     * 工作流流程实例ID
     */
    private String processInstanceId;

    /**
     * 任务标识(Code)
     */
    private String approveTaskIdentifier;

    /**
     * 任务ID
     */
    private String taskId;

    /**
     * 任务名称
     */
    private String taskName;

    /**
     * 消息ID
     */
    private String messageId;

    /**
     * 是否已经红名单审批
     *
     * @see com.dragonsoft.duceap.base.enums.BooleanEnum
     */
    private String alreadyRedListApprove;

    /**
     * 有效时间类别
     *
     * @see com.dragonsoft.dcuc.approve.enumresources.PermissionValidTypeEnum
     */
    private String permissionValidType;

    /**
     * 应用code
     */
    private String appCode;

    /**
     * 应用名称
     */
    private String appName;

    /**
     * 有效时间起始
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date startTime;

    /**
     * 有效时间终止
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date endTime;

    /**
     * 是否删除
     */
    @TableLogic
    private String isDeleted;

    /**
     * 创建时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createTime;

    /**
     * 修改时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date updateTime;
    /**
     * 创建人
     */
    private String createUser;

    /**
     * 修改人
     */
    private String updateUser;

    /**
     * 审批结果
     *
     * @see ApproveResultEnum
     */
    private String approveResult;
    /**
     * 审批编号
     */
    private String approveNo;

    /**
     * 回调通知地址
     */
    private String callbackUrl;

    /**
     * 任务类型
     */
    private String approveTaskClass;

    /**
     * 申请人IP
     */
    private String applicantIp;

    /**
     * 申请人应用令牌
     */
    private String applicantAppToken;
    /**
     * 超期时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expirationTime;


    @ApiModelProperty(value = "访问资源信息")
    private ResourceInfoDTO resourceInfoVo;

    @ApiModelProperty(value = "申请业务数据")
    private String otherApplyContext;

}
