package com.dragonsoft.dcuc.approve.service;

import com.dragonsoft.dcuc.approve.model.ApproveCirculationInfo;
import com.dragonsoft.dcuc.approve.model.resp.ApproveCommentVo;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;

import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/8
 */
public interface IApproveCirculationService {

    /**
     * 保存
     *
     * @param approveCirculationInfo 审批流转信息
     */
    void save(ApproveCirculationInfo approveCirculationInfo);

    /**
     * 根据记录ID删除对应数据
     *
     * @param approveRecordId 记录ID
     */
    void deleteByApproveRecordId(String approveRecordId);

    /**
     * 根据审批记录号获取流转信息
     *
     * @param approveRecordId 审批记录号
     * @return 流转信息列表
     */
    List<ApproveCirculationInfo> getListByApproveRecordId(String approveRecordId);

    /**
     * 根据审批记录号获取最近都是同意的几条流转信息
     *
     * @param approveRecordId 审批记录号
     * @return 流转信息列表
     */
    List<ApproveCirculationInfo> getListByApproveRecordIdAndAgree(String approveRecordId);

    /**
     * 根据审批记录号获取最近都是同意的几条流转信息
     *
     * @param approveRecordId 审批记录号
     * @return 流转信息列表
     */
    ApproveCirculationInfo getListByApproveRecordLastAgree(String approveRecordId);

    /**
     * 根据审批记录号、任务KEY获取流转信息
     *
     * @param approveRecordId 审批记录号
     * @param taskKey         任务KEY
     * @return 流转信息列表
     */
    List<ApproveCirculationInfo> getListByApproveRecordIdAndTaskKey(String approveRecordId, String taskKey);

    /**
     * 转换审批意见列表
     *
     * @param hisTaskMapNew 审批流通信息
     * @return 审批意见列表
     */
    ResponseDTO<List<ApproveCommentVo>> buildApproveCommentVo(List<ApproveCirculationInfo> hisTaskMapNew);

    /**
     *  删除并且保存新的记录
     * @param approveId 审批单号
     * @param approveCirculationInfoList 记录
     */
    void deleteAndSaveList(String approveId, List<ApproveCirculationInfo> approveCirculationInfoList);
}
