package com.dragonsoft.dcuc.approve.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragonsoft.dcuc.approve.business.IApproveBusiness;
import com.dragonsoft.dcuc.approve.mapper.ApproveActRunTaskMapper;
import com.dragonsoft.dcuc.approve.model.ApproveActRunIdentityLink;
import com.dragonsoft.dcuc.approve.model.ApproveActRunTask;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.resp.ProcessStatusDetailVo;
import com.dragonsoft.dcuc.approve.model.resp.ProcessStatusVo;
import com.dragonsoft.dcuc.approve.model.resp.TaskVo;
import com.dragonsoft.dcuc.approve.service.IApproveActRunIdentityLinkService;
import com.dragonsoft.dcuc.approve.service.IApproveActRunTaskService;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.dcuc.approve.utils.ProofsUtil;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.core.entity.page.PageImpl;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.core.search.filter.Condition;
import com.dragonsoft.duceap.core.search.filter.SearchFilter;
import com.dragonsoft.duceap.core.search.filter.SearchFilterHelper;
import org.apache.commons.lang.ArrayUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author wangrs
 * @date 2021-03-23
 */
@Service
public class ApproveActRunTaskServiceImpl implements IApproveActRunTaskService {

    @Autowired
    private ApproveActRunTaskMapper approveActRunTaskMapper;

    @Autowired
    private IApproveActRunIdentityLinkService approveActRunIdentityLinkService;

    @Autowired
    private IApproveBusiness approveBusiness;

    @Autowired
    private IApproveRecordService approveRecordService;

    @Override
    public void save(ApproveActRunTask approveActRunTask) {
        approveActRunTaskMapper.insert(approveActRunTask);
    }

    @Override
    public void deleteByApplyRecordId(String applyRecordId) {
        LambdaUpdateWrapper<ApproveActRunTask> runTaskLambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        runTaskLambdaUpdateWrapper.eq(ApproveActRunTask::getApplyRecordId, applyRecordId);
        runTaskLambdaUpdateWrapper.set(ApproveActRunTask::getDeleted, BooleanEnum.TRUE.getValue());
        approveActRunTaskMapper.update(null, runTaskLambdaUpdateWrapper);
    }

    @Override
    public void deleteById(String id) {
        approveActRunTaskMapper.deleteById(id);
    }

    @Override
    public ApproveActRunTask selectOneByTaskId(String taskId) {
        LambdaQueryWrapper<ApproveActRunTask> where = new LambdaQueryWrapper<>();
        where.eq(ApproveActRunTask::getTaskId, taskId);
        where.eq(ApproveActRunTask::getDeleted, BooleanEnum.FALSE.getValue());
        return approveActRunTaskMapper.selectOne(where);
    }

    @Override
    public Page<ApproveActRunTask> page(Searchable searchable) {
        return approveActRunTaskMapper.pagingBySearchable(searchable);
    }

    @Override
    public Integer totalCounts(Searchable countSearchable) {
        return approveActRunTaskMapper.totalCounts(countSearchable);
    }

    @Override
    public List<ApproveActRunTask> idListBySearchable(Searchable searchable) {
        return approveActRunTaskMapper.todoListBySearchable(searchable);
    }

    @Override
    public List<ApproveActRunTask> findByApproveRecordId(String approveRecordId) {
        LambdaQueryWrapper<ApproveActRunTask> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(ApproveActRunTask::getApplyRecordId, approveRecordId);
        lambdaQueryWrapper.eq(ApproveActRunTask::getDeleted, BooleanEnum.FALSE.value);
        return approveActRunTaskMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public List<ProcessStatusVo> buildProcessStatusVo(String approveRecordId) {
        List<ApproveActRunTask> waitRunList = this.findByApproveRecordId(approveRecordId);

        int waitOperatorSummary = 0;
        List<ProcessStatusVo> processStatusVoList = new ArrayList<>(waitRunList.size());
        int initSort = Integer.MAX_VALUE - waitRunList.size();
        for (ApproveActRunTask approveActRunTask : waitRunList) {
            List<ProcessStatusDetailVo> taskVos = new ArrayList<>();
            String taskId = approveActRunTask.getTaskId();
            List<ApproveActRunIdentityLink> approveActRunIdentityLinks = approveActRunIdentityLinkService.findByTaskId(taskId);
            StringBuilder assigneeBuilder = new StringBuilder();
            StringBuilder assigneeNameBuilder = new StringBuilder();
            waitOperatorSummary += approveActRunIdentityLinks.size();
            for (ApproveActRunIdentityLink approveActRunIdentityLink : approveActRunIdentityLinks) {
                String userId = approveActRunIdentityLink.getUserId();
                String userNameList = approveActRunIdentityLink.getUserNameList();
                if (StrUtil.isNotBlank(userId)) {
                    assigneeBuilder.append(userId).append(StrUtil.COMMA);
                }
                if (StrUtil.isNotBlank(userNameList)) {
                    assigneeNameBuilder.append(userNameList).append(StrUtil.COMMA);
                }
            }

            ProcessStatusDetailVo detailVo = new ProcessStatusDetailVo();

            String assignee = assigneeBuilder.toString();
            if (assignee.endsWith(StrUtil.COMMA)) {
                assignee = assignee.substring(0, assignee.length() - 1);
            }

            String assigneeName = assigneeNameBuilder.toString();
            if (assigneeName.endsWith(StrUtil.COMMA)) {
                assigneeName = assigneeName.substring(0, assigneeName.length() - 1);
            }

            detailVo.setAssignee(assignee);
            detailVo.setAssigneeName(assigneeName);
            detailVo.setReceiveTime(approveActRunTask.getGenerateTime());
            taskVos.add(detailVo);

            ProcessStatusVo processStatusVo = new ProcessStatusVo();
            processStatusVo.setNodeName(approveActRunTask.getTaskName());
            processStatusVo.setTaskVos(taskVos);
            processStatusVo.setFinishOperatorSummary(0);
            processStatusVo.setOperatorSummary(waitOperatorSummary);
            processStatusVo.setSort(initSort++);
            processStatusVoList.add(processStatusVo);
        }
        return processStatusVoList;
    }

    /**
     * 构建待办列表
     *
     * @param searchable 查询条件
     * @return 分页
     */
    @Override
    public Page<TaskVo> buildApproveActRunTask(Searchable searchable) {
        String idcard = ProofsUtil.getIdcard();
        String deptId = UserContextUtils.getCurrentUser() == null ? "" : UserContextUtils.getCurrentUser().getSecurityOrg();
        Condition processDefinitionNameCondition = searchable.getSearchFilter("processDefinitionName", SearchOperator.like);

        if (processDefinitionNameCondition != null) {
            searchable.addSearchFilter("r.businessName", SearchOperator.like, processDefinitionNameCondition.getValue());
            searchable.removeSearchFilter("processDefinitionName", SearchOperator.like);
        }
        List<SearchFilter> searchFilters = new ArrayList<>();
        searchFilters.add(SearchFilterHelper.newCondition("l.userId", SearchOperator.eq, idcard));
        searchFilters.add(SearchFilterHelper.newCondition("l.deptId", SearchOperator.eq, deptId));
        SearchFilter[] lists = new SearchFilter[searchFilters.size()];
        for (int i = 0, j = lists.length; i < j; i++) {
            lists[i] = searchFilters.get(i);
        }
        searchable.or(searchFilters.get(0), (SearchFilter[]) ArrayUtils.remove(lists, 0));
        searchable.addSearchFilter("r.deleted", SearchOperator.eq, BooleanEnum.FALSE.getValue());
        searchable.addSort(Sort.Direction.DESC, "r.generateTime");
        SearchDTO searchDTO = searchable.toSearchDTO();
        Searchable countSearchable = Searchable.toSearchable(searchDTO);
        countSearchable.removePageable();
        // 需使用无分页的SearchAble 不然会获取不到总条数
        Integer countNum = this.totalCounts(countSearchable);
        List<TaskVo> list = Collections.emptyList();
        if (countNum == null) {
            countNum = 0;
        } else {
            List<ApproveActRunTask> taskVoList = this.idListBySearchable(searchable);
            list = new ArrayList<>(taskVoList.size());
            for (ApproveActRunTask runTask : taskVoList) {
                //获取表单信息对象
                String applyRecordId = runTask.getApplyRecordId();
                ApproveRecordInfo approveRecordInfo = approveRecordService.selectOne(applyRecordId);
                TaskVo taskVo = new TaskVo();
                BeanUtils.copyProperties(runTask, taskVo);
                taskVo.setId(runTask.getTaskId());
                //设置流程名称
                taskVo.setProcessName(runTask.getBusinessName());
                if (approveRecordInfo != null) {
                    taskVo.setProcessTypeName(approveRecordInfo.getProcessTypeName());
                    //设置流程标题
                    taskVo.setProcessTitle(approveRecordInfo.getProcessTitle());
                    taskVo.setApplicantName(approveRecordInfo.getApplicantName());
                    taskVo.setProcessType(approveRecordInfo.getProcessType());
                    taskVo.setBusinessId(approveRecordInfo.getId());
                    taskVo.setStartUserName(approveRecordInfo.getApplicantName());
                    taskVo.setStartUserId(approveRecordInfo.getApplicantIdcard());
                    taskVo.setAppCode(approveRecordInfo.getAppCode());
                    taskVo.setApproveNo(approveRecordInfo.getApproveNo());
                }
                taskVo.setCreateTime(runTask.getGenerateTime());
                String unOperateUsers = approveBusiness.getUnOperateUsers(runTask.getTaskId());
                taskVo.setUnOperateUsers(unOperateUsers);
                list.add(taskVo);
            }
        }

        return new PageImpl<>(list, searchable.getPage(), countNum);
    }
}
