package com.dragonsoft.dcuc.approve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.mapper.ApproveCirculationMapper;
import com.dragonsoft.dcuc.approve.model.ApproveCirculationInfo;
import com.dragonsoft.dcuc.approve.model.resp.ApproveCommentVo;
import com.dragonsoft.dcuc.approve.service.IApproveCirculationService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/8
 */
@Service
public class ApproveCirculationImpl implements IApproveCirculationService {

    private ApproveCirculationMapper approveCirculationMapper;

    @Autowired
    public void setApproveCirculationMapper(ApproveCirculationMapper approveCirculationMapper) {
        this.approveCirculationMapper = approveCirculationMapper;
    }

    @Override
    public void save(ApproveCirculationInfo approveCirculationInfo) {
        approveCirculationMapper.insert(approveCirculationInfo);
    }

    @Override
    public void deleteByApproveRecordId(String approveRecordId) {
        LambdaUpdateWrapper<ApproveCirculationInfo> lambdaUpdateWrapper = Wrappers.lambdaUpdate();
        lambdaUpdateWrapper.eq(ApproveCirculationInfo::getApproveRecordId, approveRecordId);
        approveCirculationMapper.delete(lambdaUpdateWrapper);
    }

    @Override
    public List<ApproveCirculationInfo> getListByApproveRecordId(String approveRecordId) {
        LambdaQueryWrapper<ApproveCirculationInfo> where = Wrappers.lambdaQuery();
        where.eq(ApproveCirculationInfo::getApproveRecordId, approveRecordId);
        where.orderByAsc(ApproveCirculationInfo::getCreateTime);
        return approveCirculationMapper.selectList(where);
    }

    @Override
    public List<ApproveCirculationInfo> getListByApproveRecordIdAndAgree(String approveRecordId) {
        LambdaQueryWrapper<ApproveCirculationInfo> where = Wrappers.lambdaQuery();
        where.eq(ApproveCirculationInfo::getApproveRecordId, approveRecordId);
        where.eq(ApproveCirculationInfo::getOperateType, ApproveWorkFlowOperationEnum.COMPLETE_AGREE.value);
        where.orderByDesc(ApproveCirculationInfo::getCreateTime);
        return approveCirculationMapper.selectList(where);
    }

    @Override
    public ApproveCirculationInfo getListByApproveRecordLastAgree(String approveRecordId) {
        LambdaQueryWrapper<ApproveCirculationInfo> where = Wrappers.lambdaQuery();
        where.eq(ApproveCirculationInfo::getApproveRecordId, approveRecordId);
        where.eq(ApproveCirculationInfo::getOperateType, ApproveWorkFlowOperationEnum.COMPLETE_AGREE.value);
        where.orderByDesc(ApproveCirculationInfo::getCreateTime)
                .last(" LIMIT 1 ");
        return approveCirculationMapper.selectOne(where);
    }

    @Override
    public List<ApproveCirculationInfo> getListByApproveRecordIdAndTaskKey(String approveRecordId, String taskKey) {
        LambdaQueryWrapper<ApproveCirculationInfo> where = Wrappers.lambdaQuery();
        where.eq(ApproveCirculationInfo::getApproveRecordId, approveRecordId);
        where.eq(ApproveCirculationInfo::getTaskKey, taskKey);
        where.orderByAsc(ApproveCirculationInfo::getCreateTime);
        return approveCirculationMapper.selectList(where);
    }

    /**
     * 转换审批意见列表
     *
     * @param hisTaskMapNew 审批流通信息
     * @return 审批意见列表
     */
    @Override
    public ResponseDTO<List<ApproveCommentVo>> buildApproveCommentVo(List<ApproveCirculationInfo> hisTaskMapNew) {
        List<ApproveCommentVo> result = new ArrayList<>();
        for (int i = 0; i < hisTaskMapNew.size(); i++) {
            ApproveCirculationInfo circulationInfo = hisTaskMapNew.get(i);
            ApproveCommentVo approveCommentVo = new ApproveCommentVo();
            approveCommentVo.setId(circulationInfo.getId());
            approveCommentVo.setSort(i + 1);
            //设置已办节点的相关信息（处理人姓名、时间、类型等等）
            approveCommentVo.setApproveMessage(circulationInfo.getApproveOpinion());
            approveCommentVo.setApproveTime(circulationInfo.getCreateTime());
            approveCommentVo.setApproveState(circulationInfo.getOperateType());
            approveCommentVo.setAssignee(circulationInfo.getOperateUserName());
            approveCommentVo.setAssigneeIdcard(circulationInfo.getOperateUserId());
            approveCommentVo.setNodeName(circulationInfo.getNodeName());
            result.add(approveCommentVo);
        }
        List<ApproveCommentVo> collect = result.stream()
                .sorted(Comparator.comparing(ApproveCommentVo::getSort))
                .collect(Collectors.toList());
        return ResponseUtil.newInstance(collect);
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void deleteAndSaveList(String approveId, List<ApproveCirculationInfo> approveCirculationInfoList) {
        this.deleteByApproveRecordId(approveId);
        for (ApproveCirculationInfo approveCirculationInfo : approveCirculationInfoList) {
            this.save(approveCirculationInfo);
        }
    }
}
