package com.dragonsoft.dcuc.approve.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SmUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragonsoft.dcuc.approve.business.IApproveBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.model.dto.ResourceInfoDTO;
import com.dragonsoft.dcuc.approve.enumresources.ApproveResultEnum;
import com.dragonsoft.dcuc.approve.enumresources.ApproveTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.dragonsoft.dcuc.approve.mapper.ApproveRecordMapper;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.resp.ApprovalInfoRespVO;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.core.entity.page.PageImpl;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 审批信息
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/1
 */
@Slf4j
@Service
public class ApproveRecordServiceImpl implements IApproveRecordService {

    private ApproveRecordMapper approveRecordMapper;

    private IApplyInfoFacade applyInfoFacade;

    private IApproveBusiness approveBusiness;

    @Autowired
    public void setApproveRecordMapper(ApproveRecordMapper approveRecordMapper) {
        this.approveRecordMapper = approveRecordMapper;
    }

    @Autowired
    public void setApplyInfoFacade(IApplyInfoFacade applyInfoFacade) {
        this.applyInfoFacade = applyInfoFacade;
    }

    @Autowired
    public void setApproveBusiness(IApproveBusiness approveBusiness) {
        this.approveBusiness = approveBusiness;
    }

    @Override
    public void save(ApproveRecordInfo approveRecordInfo) {
        String generalSign = generalSign(approveRecordInfo);
        approveRecordInfo.setSign(generalSign);
        approveRecordMapper.insert(approveRecordInfo);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deleteById(String id) {
        ApproveRecordInfo approveRecordInfo = approveRecordMapper.selectById(id);
        approveRecordInfo.setFinalApprovalResult(ApproveResultEnum.DELETED.value);
        approveRecordMapper.updateById(approveRecordInfo);
        approveRecordMapper.deleteById(id);
    }

    @Override
    public void updateById(ApproveRecordInfo approveRecordInfo) {
        String generalSign = generalSign(approveRecordInfo);
        approveRecordInfo.setSign(generalSign);
        approveRecordMapper.updateById(approveRecordInfo);
    }

    @Override
    public ApproveRecordInfo selectOne(String approveRecordId) {
        return approveRecordMapper.selectById(approveRecordId);
    }

    @Override
    public ApproveRecordInfo selectByApproveNo(String approveNo) {
        LambdaQueryWrapper<ApproveRecordInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(ApproveRecordInfo::getApproveNo, approveNo);
        return approveRecordMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    public ApproveRecordInfo findByProcessInstanceId(String processInstanceId) {
        LambdaQueryWrapper<ApproveRecordInfo> where = Wrappers.lambdaQuery();
        where.eq(ApproveRecordInfo::getProcessInstanceId, processInstanceId);

        return approveRecordMapper.selectOne(where);
    }

    @Override
    public Page<ApproveRecordInfo> paging(Searchable searchable) {
        return approveRecordMapper.pagingBySearchable(searchable);
    }

    @Override
    public Integer getUserRequestCount(String userId) {
        LambdaQueryWrapper<ApproveRecordInfo> where = Wrappers.lambdaQuery();
        where.eq(ApproveRecordInfo::getCreateUser, userId);
        where.eq(ApproveRecordInfo::getIsDeleted, BooleanEnum.FALSE.getValue());
        return approveRecordMapper.selectCount(where);
    }

    @Override
    public ApproveRecordInfo findByCreateTime(Date date) {
        //获取申请信息填充
        LambdaQueryWrapper<ApproveRecordInfo> where = Wrappers.lambdaQuery();
        where.between(ApproveRecordInfo::getCreateTime, DateUtil.beginOfDay(date), DateUtil.endOfDay(date))
                .orderByDesc(ApproveRecordInfo::getCreateTime)
                .last("LIMIT 1");
        return approveRecordMapper.selectOne(where);
    }

    /**
     * 生成签名
     *
     * @param approveRecordInfo 审批信息
     * @return 签名
     */
    private String generalSign(ApproveRecordInfo approveRecordInfo) {
        String applicantIdcard = approveRecordInfo.getApplicantIdcard();
        String approveContent = approveRecordInfo.getApproveContent();
        String finalApprovalResult = approveRecordInfo.getFinalApprovalResult();
        Date updateTime = approveRecordInfo.getUpdateTime();
        String id = approveRecordInfo.getId();
        String approveNo = approveRecordInfo.getApproveNo();

        // 提请审批人员OpenID HASH、 审批内容HASH、最终审批结果、最后审批时间、审批记录ID、审批ID
        String signString = applicantIdcard.hashCode() + approveContent.hashCode() + finalApprovalResult
                + updateTime + id + approveNo;

        return SmUtil.sm3(signString);
    }

    /**
     * 将旧表单表对象构建成模式3新表对象
     *
     * @param approvalInfoDTO 审批信息
     * @param approveNo    审批单号
     * @return 审批记录信息
     */
    @Override
    public ApproveRecordInfo buildApproveRecordInfo(ApprovalInfoDTO approvalInfoDTO, String approveNo) {
        ApproveRecordInfo approveRecordInfo = new ApproveRecordInfo();
        approveRecordInfo.setId(approvalInfoDTO.getId());
        approveRecordInfo.setBusinessCode(approvalInfoDTO.getBusinessCode());
        approveRecordInfo.setProcessName(approvalInfoDTO.getProcessName());

        approveRecordInfo.setProcessType(approvalInfoDTO.getProcessType());
        approveRecordInfo.setProcessTypeName(approvalInfoDTO.getProcessTypeName());

        String flowType = approvalInfoDTO.getFlowType();
        FlowTypeEnum flowTypeEnum = EnumUtils.enumOf(FlowTypeEnum.class, flowType);
        ApproveTypeEnum approveTypeEnum = flowTypeEnum.toApproveTypeEnum();
        //审批中心类型
        approveRecordInfo.setApproveType(approveTypeEnum.value);
        //审批单号规则
        approveRecordInfo.setApproveNo(approveNo);
        approveRecordInfo.setProcessTitle(approvalInfoDTO.getProcessTitle());
        approveRecordInfo.setApplicantIdcard(approvalInfoDTO.getApplicantIdcard());
        approveRecordInfo.setApplicantName(approvalInfoDTO.getApplicantName());
        approveRecordInfo.setApplicantPhoneNo(approvalInfoDTO.getApplicantPhoneNo());
        approveRecordInfo.setApplicantOrgCode(approvalInfoDTO.getApplicantOrgCode());
        approveRecordInfo.setApplicantOrgName(approvalInfoDTO.getApplicantOrgName());
        approveRecordInfo.setApplyTime(DateUtils.getTimeStr(approvalInfoDTO.getCreateTime(), DateConst.HYPHEN_DISPLAY_TIME));

        String appCode = approvalInfoDTO.getAppCode();
        if (StrUtil.isBlank(appCode)) {
            throw new IllegalArgumentException("appCode can't be blank");
        }

        approveRecordInfo.setAppCode(appCode);
        try {
            ApplyInfo applyInfo = applyInfoFacade.getAppByCode(appCode);
            if (applyInfo != null) {
                approveRecordInfo.setAppName(applyInfo.getApplyName());
            }
        } catch (Exception e) {
            log.error("调用app-service失败", e);
        }
        ResourceInfoDTO resourceInfoVo = approvalInfoDTO.getResourceInfoVo();
        approveRecordInfo.setApproveContent(approvalInfoDTO.getApplyContent());

        approveBusiness.setResourceInfoData(approveRecordInfo, resourceInfoVo);

        approveRecordInfo.setPermissionValidType(approvalInfoDTO.getPermissionValidType());
        approveRecordInfo.setStartTime(approvalInfoDTO.getStartTime());
        approveRecordInfo.setEndTime(approvalInfoDTO.getEndTime());
        approveRecordInfo.setApplyReason(approvalInfoDTO.getApplyReason());
        approveRecordInfo.setFinalApprovalResult("");
        approveRecordInfo.setProcessInstanceId(approvalInfoDTO.getProcessInstanceId());
        approveRecordInfo.setTaskCode(approvalInfoDTO.getBusinessCode());
        approveRecordInfo.setTaskId(approvalInfoDTO.getTaskId());
        approveRecordInfo.setTaskName(approvalInfoDTO.getTaskName());
        approveRecordInfo.setMessageId(approvalInfoDTO.getMessageId());
        approveRecordInfo.setOtherApplyContext(approvalInfoDTO.getOtherApplyContext());
        approveRecordInfo.setIsDeleted(BooleanEnum.FALSE.getValue());
        approveRecordInfo.setCreateUser(approvalInfoDTO.getCreateUser());
        approveRecordInfo.setCreateTime(approvalInfoDTO.getCreateTime());
        approveRecordInfo.setUpdateUser(approvalInfoDTO.getUpdateUser());
        approveRecordInfo.setUpdateTime(approvalInfoDTO.getUpdateTime());
        approveRecordInfo.setFlowType(approvalInfoDTO.getFlowType());
        approveRecordInfo.setCallbackUrl(approvalInfoDTO.getCallbackUrl());
        approveRecordInfo.setApproveTaskClass(approvalInfoDTO.getApproveTaskClass());
        approveRecordInfo.setApplicantIp(approvalInfoDTO.getApplicantIp());
        approveRecordInfo.setApplicantAppToken(approvalInfoDTO.getApplicantAppToken());
        approveRecordInfo.setSyncFlag(BooleanEnum.FALSE.value);
        return approveRecordInfo;
    }

    @Override
    public Page<ApprovalInfoRespVO> queryUserApproveList(Searchable searchable) {
        searchable.addSort(Sort.Direction.DESC, "createTime");
        //默认查询审批应用
        searchable.addSearchFilter("isDeleted", SearchOperator.eq, BooleanEnum.FALSE.getValue());
        //查询表单表数据
        Page<ApproveRecordInfo> applyInfoPage = approveRecordMapper.pagingBySearchable(searchable);
        List<ApprovalInfoRespVO> approvalInfoRespVOList = new ArrayList<>();
        List<ApproveRecordInfo> content = applyInfoPage.getContent();
        for (ApproveRecordInfo approveRecordInfo : content) {
            //将模式3新表对象构建成旧表对象（前端和运营中心无需改造）
            ApprovalInfoDTO approvalInfoDTO = approveRecordInfo.toApprovalInfo();
            //设置当前结点和未操作者信息
            ApprovalInfoRespVO approvalInfoRespVo = approveBusiness.fillTaskInfoToApplyInfo(approvalInfoDTO);
            approvalInfoRespVo.setAppCode(approveRecordInfo.getAppCode());
            approvalInfoRespVo.setBusinessId(approvalInfoRespVo.getId());
            approvalInfoRespVOList.add(approvalInfoRespVo);
        }

        return new PageImpl<>(approvalInfoRespVOList, applyInfoPage.getPageable(), applyInfoPage.getTotalElements());
    }


    @Override
    public ApproveRecordInfo getOneByTaskId(String taskId) {
        if (taskId.equalsIgnoreCase(ApproveConstants.END_STR)) {
            return null;
        }

        LambdaQueryWrapper<ApproveRecordInfo> where = Wrappers.lambdaQuery();
        where.eq(ApproveRecordInfo::getTaskId, taskId);

        return approveRecordMapper.selectOne(where);
    }

    @Override
    public List<ApproveRecordInfo> getRecordByIdcardAppCode(String idcard, String appCode) {
        LambdaQueryWrapper<ApproveRecordInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(ApproveRecordInfo::getAppCode, appCode);
        lambdaQueryWrapper.eq(ApproveRecordInfo::getApplicantIdcard, idcard);

        return approveRecordMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public String getTaskNoByDate(String appCode, String dateTimeStr) {
        LambdaQueryWrapper<ApproveRecordInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper
                .eq(ApproveRecordInfo::getAppCode, appCode)
                .like(ApproveRecordInfo::getApproveNo, dateTimeStr)
                .select(ApproveRecordInfo::getApproveNo)
                .orderByDesc(ApproveRecordInfo::getApproveNo)
                .last("LIMIT 1");

        ApproveRecordInfo approveRecordInfo = approveRecordMapper.selectOne(lambdaQueryWrapper);
        if (ObjectUtil.isNotNull(approveRecordInfo)) {
            String approveNo = approveRecordInfo.getApproveNo();
            return getCurrentNo(approveNo);
        }
        return "";
    }

    @Override
    public List<ApproveRecordInfo> findExpirationTimeRecord(Date startExpirationDateTime) {
        LambdaQueryWrapper<ApproveRecordInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.ne(ApproveRecordInfo::getTaskId, ApproveConstants.END_STR)
                .le(ApproveRecordInfo::getUpdateTime, startExpirationDateTime);
        return approveRecordMapper.selectList(lambdaQueryWrapper);
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void updateSyncFlag(String id, BooleanEnum booleanEnum) {
        Assert.notBlank(id);
        Assert.notNull(booleanEnum);

        ApproveRecordInfo approveRecordInfo = approveRecordMapper.selectById(id);
        if (ObjectUtil.isNotNull(approveRecordInfo)) {
            approveRecordInfo.setSyncFlag(booleanEnum.value);
            approveRecordInfo.setUpdateTime(new Date());
            SecurityUser currentUser = UserContextUtils.getCurrentUser();
            if (ObjectUtil.isNotNull(currentUser)) {
                approveRecordInfo.setUpdateUser(currentUser.getIdcard());
            }

            approveRecordMapper.updateById(approveRecordInfo);
        }
    }

    @Override
    public List<ApproveRecordInfo> getUnSyncList() {
        LambdaQueryWrapper<ApproveRecordInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ApproveRecordInfo::getIsDeleted, BooleanEnum.FALSE.value)
                .eq(ApproveRecordInfo::getSyncFlag, BooleanEnum.FALSE.value)
                .eq(ApproveRecordInfo::getFinalApprovalResult, ApproveResultEnum.SUCCESS.getValue())
                .orderByDesc(ApproveRecordInfo::getCreateTime);

        return approveRecordMapper.selectList(queryWrapper);
    }

    protected String getCurrentNo(String approveNo) {
        if (StrUtil.isBlank(approveNo)) {
            return "";
        }
        if (approveNo.startsWith(ApproveConstants.TASK_ID_PREFIX)
                && approveNo.length() >= ApproveConstants.TAK_ID_LENGTH) {
            return approveNo.substring(approveNo.length() - 8);
        }
        return "";
    }

}
