package com.dragonsoft.dcuc.approve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.duceap.enums.SchemeStatusEnum;
import com.dragoninfo.dcuc.duceap.enums.SchemeTypeEnum;
import com.dragonsoft.dcuc.approve.mapper.RiskSchemeMapper;
import com.dragonsoft.dcuc.approve.model.RiskScheme;
import com.dragonsoft.dcuc.approve.model.resp.RiskSchemeVo;
import com.dragonsoft.dcuc.approve.service.IRiskLogService;
import com.dragonsoft.dcuc.approve.service.IRiskSchemeService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 代码千万行,注释第一行,编码不规范,同事两行泪
 *
 * @author huang(jy)
 * @version 1.0
 * @date 2021/5/27 19:43
 */
@Service
public class RiskSchemeServiceImpl implements IRiskSchemeService {

    @Autowired
    private RiskSchemeMapper riskSchemeMapper;

    @Autowired
    private IRiskLogService riskLogService;

    @Override
    public Page<RiskSchemeVo> queryRiskSchemeList(Searchable searchable) {
        searchable.addSort(Sort.Direction.DESC, "updateTime");

        searchable.addSearchFilter("isDeleted", SearchOperator.eq, BooleanEnum.FALSE.getValue());
        Page<RiskScheme> page = riskSchemeMapper.pagingBySearchable(searchable);
        List<RiskSchemeVo> logVoList = new ArrayList<>();
        for (RiskScheme riskScheme : page.getContent()) {
            RiskSchemeVo riskSchemeVo = new RiskSchemeVo();
            BeanUtils.copyProperties(riskScheme, riskSchemeVo);
            riskSchemeVo.setSchemeStatusStr(EnumUtils.enumOf(SchemeStatusEnum.class, riskScheme.getSchemeStatus()).getLabel());
            riskSchemeVo.setSchemeTypeStr(EnumUtils.enumOf(SchemeTypeEnum.class, riskScheme.getSchemeType()).getLabel());
            String schemeType = riskScheme.getSchemeType();
            Integer riskNum = riskLogService.selectCountBySchemeType(schemeType);
            riskSchemeVo.setRiskNum(riskNum != null ? riskNum : 0);
            logVoList.add(riskSchemeVo);
        }
        return new PageImpl<>(logVoList, page.getPageable(), page.getTotalElements());
    }

    @Override
    public ResponseDTO<RiskSchemeVo> riskSchemeDetail(String id) {
        RiskScheme riskScheme = riskSchemeMapper.selectById(id);
        if (riskScheme == null) {
            return ResponseUtil.dtoFail("风险方案详情不存在");
        }
        RiskSchemeVo riskSchemeVo = new RiskSchemeVo();
        BeanUtils.copyProperties(riskScheme, riskSchemeVo);
        riskSchemeVo.setSchemeStatusStr(EnumUtils.enumOf(SchemeStatusEnum.class, riskScheme.getSchemeStatus()).getLabel());
        riskSchemeVo.setSchemeTypeStr(EnumUtils.enumOf(SchemeTypeEnum.class, riskScheme.getSchemeType()).getLabel());
        return ResponseUtil.newInstance(riskSchemeVo);
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseStatus riskSchemeUpdate(RiskSchemeVo riskSchemeVo) {
        RiskScheme riskScheme = riskSchemeMapper.selectById(riskSchemeVo.getId());
        riskScheme.setUpdateTime(new Date());
        riskScheme.setUpdateUser(UserContextUtils.getCurrentUser() != null ? UserContextUtils.getCurrentUser().getIdcard() : "");
        riskScheme.setName(riskSchemeVo.getName());
        riskScheme.setSchemeStatus(riskSchemeVo.getSchemeStatus());
        riskScheme.setContinuousApproveNum(riskSchemeVo.getContinuousApproveNum());
        riskScheme.setDisposeAdvise(riskSchemeVo.getDisposeAdvise());
        riskSchemeMapper.updateById(riskScheme);
        return ResponseStatus.success();
    }

    @Override
    public ResponseStatus riskSchemeDelete(String id) {
        RiskScheme riskScheme = riskSchemeMapper.selectById(id);
        riskScheme.setDeleteTime(new Date());
        riskScheme.setDeleteUser(UserContextUtils.getCurrentUser() != null ? UserContextUtils.getCurrentUser().getIdcard() : "");
        riskScheme.setIsDeleted(BooleanEnum.TRUE.getValue());
        riskSchemeMapper.updateById(riskScheme);
        return ResponseStatus.success();
    }

    @Override
    public RiskScheme selectEnableSchemeByType(SchemeTypeEnum schemeTypeEnum) {
        LambdaQueryWrapper<RiskScheme> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(RiskScheme::getSchemeType, schemeTypeEnum.getValue())
                .eq(RiskScheme::getSchemeStatus, SchemeStatusEnum.ENABLE.code);

        return riskSchemeMapper.selectOne(lambdaQueryWrapper);
    }
}
