package com.dragonsoft.dcuc.approve.business.impl;

import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import org.junit.*;
import org.mockito.Mockito;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import redis.embedded.RedisServer;

import java.util.Date;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/19
 */
public class ApproveBaseBusinessImplTest {

    private static RedisServer redisServer;

    private StringRedisTemplate stringRedisTemplate;

    @BeforeClass
    public static void startRedis() {
        redisServer = RedisServer.builder().port(6380)
                .setting("maxheap 100m")
                .build();
        redisServer.start();
        try {
            // 需睡眠一会儿，防止启动不成功
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    @AfterClass
    public static void stopRedis() {
        redisServer.stop();
    }

    @Before
    public void setUp() {
        LettuceConnectionFactory lettuceConnectionFactory = new LettuceConnectionFactory();
        lettuceConnectionFactory.setPort(6380);
        lettuceConnectionFactory.afterPropertiesSet();
        stringRedisTemplate = new StringRedisTemplate(lettuceConnectionFactory);
    }

    @Test
    public void parseAuthEndTime() {
        Date currDate = DateUtils.getDate();

        ApproveBaseBusinessImpl approveBusiness = new ApproveBaseBusinessImpl();

        Date date = approveBusiness.parseEndTime(currDate);
        System.out.println(DateUtils.getDateFormat(date, DateConst.DB_STORE_TIME));
    }

    @Test
    public void taskNo() {
        IApproveRecordService approveRecordService = Mockito.spy(IApproveRecordService.class);
        Mockito.when(approveRecordService.getTaskNoByDate(Mockito.anyString(), Mockito.anyString()))
                .thenReturn("");

        String appCode1 = "1";
        String appCode2 = "2";

        String orgCode = "110014013008";
        Date date = DateUtils.getDate();
        String dateFormat = DateUtils.getDateFormat(date, DateConst.DB_STORE_TIME);

        ApproveBaseBusinessImpl approveBusiness = new ApproveBaseBusinessImpl();
        approveBusiness.setStringRedisTemplate(stringRedisTemplate);
        approveBusiness.setApproveRecordService(approveRecordService);
        String generalTaskNo = approveBusiness.generalTaskNo(appCode1, orgCode, date);

        Assert.assertEquals(ApproveConstants.TASK_ID_PREFIX + orgCode + dateFormat + "00000001", generalTaskNo);

        String generalTaskNo1 = approveBusiness.generalTaskNo(appCode1, orgCode, date);

        Assert.assertEquals(ApproveConstants.TASK_ID_PREFIX + orgCode + dateFormat + "00000002", generalTaskNo1);


        String generalTaskNo2 = approveBusiness.generalTaskNo(appCode2, orgCode, date);

        Assert.assertEquals(ApproveConstants.TASK_ID_PREFIX + orgCode + dateFormat + "00000001", generalTaskNo2);
    }
}