package com.dragonsoft.dcuc.approve.business.impl;

import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import org.junit.*;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import redis.embedded.RedisServer;

import java.text.ParseException;
import java.util.Date;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/13
 */
public class RedisCacheBusinessImplTest {

    private static RedisServer redisServer;

    private StringRedisTemplate stringRedisTemplate;

    @BeforeClass
    public static void startRedis() {
        redisServer = RedisServer.builder().port(6380)
                .setting("maxheap 100m")
                .build();
        redisServer.start();
        try {
            // 需睡眠一会儿，防止启动不成功
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    @AfterClass
    public static void stopRedis() {
        redisServer.stop();
    }

    @Before
    public void setUp() {
        LettuceConnectionFactory lettuceConnectionFactory = new LettuceConnectionFactory();
        lettuceConnectionFactory.setPort(6380);
        lettuceConnectionFactory.afterPropertiesSet();
        stringRedisTemplate = new StringRedisTemplate(lettuceConnectionFactory);
    }


    @Test
    public void getLastOrgSyncTime() throws ParseException {
        RedisCacheBusinessImpl redisCacheBusiness = new RedisCacheBusinessImpl();
        redisCacheBusiness.setStringRedisTemplate(stringRedisTemplate);
        String dateFormat = DateUtils.getDateFormat(DateConst.DB_STORE_TIME);
        Date date = DateUtils.getDate(dateFormat, DateConst.DB_STORE_TIME);

        redisCacheBusiness.saveOrgSyncTime(date);

        Date lastOrgSyncTime = redisCacheBusiness.getLastOrgSyncTime();
        String dateFormat1 = DateUtils.getDateFormat(lastOrgSyncTime, DateConst.DB_STORE_TIME);
        System.out.println(dateFormat1);
        Assert.assertEquals(dateFormat, dateFormat1);

    }

    @Test
    public void getLastUserSyncTime() throws ParseException {
        RedisCacheBusinessImpl redisCacheBusiness = new RedisCacheBusinessImpl();
        redisCacheBusiness.setStringRedisTemplate(stringRedisTemplate);
        String dateFormat = DateUtils.getDateFormat(DateConst.DB_STORE_TIME);
        Date date = DateUtils.getDate(dateFormat, DateConst.DB_STORE_TIME);

        redisCacheBusiness.saveUserSyncTime(date);

        Date lastOrgSyncTime = redisCacheBusiness.getLastUserSyncTime();
        String dateFormat1 = DateUtils.getDateFormat(lastOrgSyncTime, DateConst.DB_STORE_TIME);
        System.out.println(dateFormat1);
        Assert.assertEquals(dateFormat, dateFormat1);

    }

}